Package: wARMASVp
Type: Package
Title: Winsorized ARMA Estimation for Higher-Order Stochastic
        Volatility Models
Version: 0.2.0
Authors@R: c(
    person("Gabriel", "Rodriguez-Rondon",
           email = "gabriel.rodriguezrondon@mail.mcgill.ca",
           role = c("aut", "cre"),
           comment = c(ORCID = "0009-0005-3769-9921")),
    person("Md. Nazmul", "Ahsan", role = "aut"),
    person("Jean-Marie", "Dufour", role = "aut"))
Description: Estimation, simulation, hypothesis testing, AR-order selection,
    and forecasting for univariate higher-order stochastic volatility SV(p)
    models. Supports Gaussian, Student-t, and Generalized Error Distribution
    (GED) innovations, with optional leverage effects. Estimation uses
    closed-form Winsorized ARMA-SV (W-ARMA-SV) moment-based methods that
    avoid numerical optimization. Hypothesis testing includes Local Monte
    Carlo (LMC) and Maximized Monte Carlo (MMC) procedures for leverage
    effects, heavy tails, and autoregressive order. AR-order selection is
    also available via information criteria (BIC/AIC) using the Kalman-filter
    quasi-likelihood and the Hannan-Rissanen ARMA residual variance.
    Forecasting is based on Kalman filtering and smoothing. See Ahsan and
    Dufour (2021) <doi:10.1016/j.jeconom.2021.03.008>, Ahsan, Dufour, and
    Rodriguez-Rondon (2025) <doi:10.1111/jtsa.12851>, and Ahsan, Dufour, and
    Rodriguez-Rondon (2026) <doi:10.34989/swp-2026-8> for details.
License: GPL (>= 3)
URL: https://github.com/roga11/wARMASVp
BugReports: https://github.com/roga11/wARMASVp/issues
Encoding: UTF-8
Imports: Rcpp (>= 1.0.0), gsignal, stats
Suggests: pso, GenSA, testthat (>= 3.0.0), knitr, rmarkdown
LinkingTo: Rcpp, RcppArmadillo
RoxygenNote: 7.3.3
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2026-05-15 12:58:15 UTC; gabrielrodriguez
Author: Gabriel Rodriguez-Rondon [aut, cre] (ORCID:
    <https://orcid.org/0009-0005-3769-9921>),
  Md. Nazmul Ahsan [aut],
  Jean-Marie Dufour [aut]
Maintainer: Gabriel Rodriguez-Rondon <gabriel.rodriguezrondon@mail.mcgill.ca>
Repository: CRAN
Date/Publication: 2026-05-15 15:50:01 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-15 17:05:29 UTC; unix
Archs: wARMASVp.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(coef,svp)
S3method(coef,svp_ged)
S3method(coef,svp_t)
S3method(plot,svp_filter)
S3method(plot,svp_forecast)
S3method(print,svp)
S3method(print,svp_filter)
S3method(print,svp_forecast)
S3method(print,svp_ged)
S3method(print,svp_t)
S3method(print,svp_test)
S3method(summary,svp)
S3method(summary,svp_filter)
S3method(summary,svp_ged)
S3method(summary,svp_t)
export(filter_svp)
export(forecast_svp)
export(lmc_ar)
export(lmc_ged)
export(lmc_lev)
export(lmc_t)
export(mmc_ar)
export(mmc_ged)
export(mmc_lev)
export(mmc_t)
export(sim_svp)
export(svp)
export(svpSE)
export(svp_AR_order)
export(svp_IC)
importFrom(Rcpp,sourceCpp)
importFrom(gsignal,poly)
importFrom(stats,dnorm)
importFrom(stats,pgamma)
importFrom(stats,qgamma)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rchisq)
importFrom(stats,rf)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,rt)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(stats,uniroot)
importFrom(stats,var)
useDynLib(wARMASVp, .registration = TRUE)

# wARMASVp

Winsorized ARMA Estimation for Higher-Order Stochastic Volatility Models

## Overview

**wARMASVp** provides estimation, simulation, hypothesis testing, and forecasting for univariate higher-order stochastic volatility SV(p) models. It supports Gaussian, Student-t, and Generalized Error Distribution (GED) innovations, with optional leverage effects.

The estimation method is based on closed-form Winsorized ARMA-SV (W-ARMA-SV) moment-based estimators that avoid numerical optimization, making them fast and reliable.

## Installation

You can install the development version from GitHub:

```r
# install.packages("devtools")
devtools::install_github("roga11/wARMASVp")
```

## Features

- **Estimation**: SV(p) models with Gaussian, Student-t, or GED errors via `svp()`
- **Leverage effects**: Asymmetric volatility estimation for Gaussian SV(p)
- **Simulation**: Generate SV(p) data with `sim_svp()`
- **Hypothesis testing**: LMC and MMC procedures for autoregressive order, leverage, and heavy tails
- **Forecasting**: Kalman filter-based h-step-ahead volatility forecasts via `forecast_svp()`
- **Standard errors**: Simulation-based confidence intervals via `svpSE()`

## Quick Start

```r
library(wARMASVp)

# Simulate Gaussian SV(1)
y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.3)

# Estimate
fit <- svp(y, p = 1)
summary(fit)

# Standard errors
se <- svpSE(fit, n_sim = 99)
se$CI

# Forecast
fc <- forecast_svp(fit, H = 10)
plot(fc)
```

## References

- Ahsan, M. N. and Dufour, J.-M. (2021). Simple estimators and inference for higher-order stochastic volatility models. *Journal of Econometrics*, 224(1), 181-197. [doi:10.1016/j.jeconom.2021.03.008](https://doi.org/10.1016/j.jeconom.2021.03.008)

- Ahsan, M. N., Dufour, J.-M., and Rodriguez-Rondon, G. (2025). Estimation and inference for higher-order stochastic volatility models with leverage. *Journal of Time Series Analysis*, 46(6), 1064-1084. [doi:10.1111/jtsa.12851](https://doi.org/10.1111/jtsa.12851)

- Ahsan, M. N., Dufour, J.-M., and Rodriguez-Rondon, G. (2026). Estimation and inference for stochastic volatility models with heavy-tailed distributions. Bank of Canada Staff Working Paper 2026-8. [doi:10.34989/swp-2026-8](https://doi.org/10.34989/swp-2026-8)

## License

GPL (>= 3)

#' Estimate a Stochastic Volatility Model
#'
#' Master estimation function for SV(p) models using the Winsorized ARMA-SV
#' (W-ARMA-SV) method. Supports Gaussian, Student-t, and GED error
#' distributions, with optional leverage effects.
#'
#' @details
#' The model is:
#' \deqn{y_t = \sigma_y \exp(w_t / 2) z_t}
#' \deqn{w_t = \phi_1 w_{t-1} + \cdots + \phi_p w_{t-p} + \sigma_v v_t}
#'
#' where \eqn{z_t} follows a distribution specified by \code{errorType}
#' (Gaussian, Student-t, or GED), and \eqn{v_t} is i.i.d. standard normal.
#' When \code{leverage = TRUE}, the correlation between \eqn{z_t} and
#' \eqn{v_t} is estimated as \eqn{\rho}.
#'
#' For Student-t errors with leverage, the correction factor
#' \eqn{C_t(\nu)} from the scale-mixture representation is applied.
#' For GED errors with leverage, the exact implicit equation is solved
#' via 1D root-finding with Gauss-Hermite quadrature.
#'
#' @param y Numeric vector. Observed returns (e.g., de-meaned log returns).
#' @param p Integer. Order of the volatility process. Default is 1.
#' @param J Integer. Winsorizing parameter controlling the number of
#'   autocovariance blocks used. Default is 10.
#' @param leverage Logical. If \code{TRUE}, estimate leverage parameter
#'   \eqn{\rho}. Default is \code{FALSE}.
#' @param errorType Character. Error distribution: \code{"Gaussian"} (default),
#'   \code{"Student-t"}, or \code{"GED"}.
#' @param rho_type Character. Correlation type for leverage estimation. One of
#'   \code{"pearson"} (default), \code{"kendall"}, or \code{"both"}.
#' @param del Numeric. Small constant for log transformation:
#'   \eqn{\log(y_t^2 + \delta)}. Default is \code{1e-10}.
#' @param trunc_lev Logical. If \code{TRUE}, truncate the estimated leverage
#'   parameter to \code{[-0.999, 0.999]}. Default is \code{TRUE}. Setting to
#'   \code{FALSE} can reduce bias in some cases but may yield estimates
#'   outside the parameter space.
#' @param wDecay Logical. Use linearly decaying weights in the WLS estimation.
#'   Default is \code{FALSE}.
#' @param logNu Logical. Solve for \eqn{\nu} in log-space for numerical
#'   stability (Student-t only). Default is \code{FALSE}.
#' @param sigvMethod Character. Method for estimating \eqn{\sigma_v}. One of:
#'   \code{"factored"} (default) — factored-variance estimator (recommended;
#'   dominates RMSE in most settings, see ADRR 2025);
#'   \code{"direct"} — direct variance decomposition;
#'   \code{"hybrid"} — AD2021 closed-form for \eqn{p = 1}, falls back to
#'   \code{"direct"} for \eqn{p \ge 2} (Student-t and GED only).
#' @param winsorize_eps Integer. Number of extreme autocovariance lags to
#'   winsorize (\code{0} = none). Used in Student-t and GED
#'   \eqn{\sigma_\varepsilon^2} estimation. Default \code{0}.
#'
#' @return Depending on \code{errorType}:
#' \itemize{
#'   \item \code{"Gaussian"}: An object of class \code{"svp"} (see below).
#'   \item \code{"Student-t"}: An object of class \code{"svp_t"}.
#'   \item \code{"GED"}: An object of class \code{"svp_ged"}.
#' }
#'
#' The \code{"svp"} class contains:
#' \describe{
#'   \item{mu}{Mean of \eqn{\log(y_t^2 + \delta)}.}
#'   \item{phi}{Numeric vector of AR coefficients.}
#'   \item{sigv}{Standard deviation of volatility innovations.}
#'   \item{sigy}{Unconditional standard deviation.}
#'   \item{rho}{Leverage parameter (if estimated, otherwise \code{NA}).}
#'   \item{y}{The original data.}
#'   \item{p, J}{Model order and winsorizing parameter.}
#'   \item{errorType}{The error distribution used.}
#'   \item{call}{The matched call.}
#' }
#'
#' @examples
#' \donttest{
#' # Gaussian SV(1) without leverage (default)
#' y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2)$y
#' fit <- svp(y)
#' summary(fit)
#'
#' # With leverage
#' y2 <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2, leverage = TRUE, rho = -0.3)$y
#' fit2 <- svp(y2, leverage = TRUE)
#' coef(fit2)
#'
#' # Student-t errors
#' y3 <- sim_svp(1000, phi = 0.9, sigy = 1, sigv = 0.2, errorType = "Student-t", nu = 5)$y
#' fit3 <- svp(y3, errorType = "Student-t")
#' summary(fit3)
#' }
#'
#' @seealso \code{\link{svpSE}} for standard errors.
#'
#' @references
#' Ahsan, M. N. and Dufour, J.-M. (2021). Simple estimators and inference for
#' higher-order stochastic volatility models. \emph{Journal of Econometrics},
#' 224(1), 181-197. \doi{10.1016/j.jeconom.2021.03.008}
#'
#' Ahsan, M. N., Dufour, J.-M., and Rodriguez-Rondon, G. (2026). Estimation and
#' inference for stochastic volatility models with heavy-tailed distributions.
#' Bank of Canada Staff Working Paper 2026-8. \doi{10.34989/swp-2026-8}
#'
#' @export
svp <- function(y, p = 1, J = 10, leverage = FALSE, errorType = "Gaussian",
                rho_type = "pearson", del = 1e-10, trunc_lev = TRUE,
                wDecay = FALSE, logNu = FALSE,
                sigvMethod = "factored", winsorize_eps = 0) {
  cl <- match.call()
  y <- as.numeric(y)
  if (length(y) < 1L) stop("'y' must be a non-empty numeric vector.")
  if (any(!is.finite(y))) stop("'y' must not contain NA, NaN, or Inf values.")
  p <- as.integer(p)
  if (p < 1L) stop("'p' must be a positive integer (>= 1).")
  J <- as.integer(J)
  if (J < 1L) stop("'J' must be a positive integer (>= 1).")
  min_length <- 2L * p + J
  if (length(y) < min_length)
    stop("'y' must have length >= 2*p + J = ", min_length,
         " (got ", length(y), ").")
  if (!is.numeric(del) || del <= 0) stop("'del' must be a positive number.")
  errorType <- match.arg(errorType, c("Gaussian", "Student-t", "GED"))
  sigvMethod <- match.arg(sigvMethod, c("hybrid", "direct", "factored"))
  # Dispatch: estimate base model (without leverage)
  if (errorType == "Gaussian") {
    out <- .svp_gaussian(y, p, J, leverage, rho_type, del, trunc_lev, wDecay,
                         sigvMethod)
  } else if (errorType == "Student-t") {
    out <- .svp_t(y, p, J, del, wDecay, logNu, sigvMethod, winsorize_eps)
    if (leverage) {
      out <- .add_leverage(out, y, as.integer(p), rho_type, del, trunc_lev,
                           wDecay, "Student-t")
    }
  } else if (errorType == "GED") {
    out <- .svp_ged(y, p, J, del, wDecay, sigvMethod, winsorize_eps)
    if (leverage) {
      out <- .add_leverage(out, y, as.integer(p), rho_type, del, trunc_lev,
                           wDecay, "GED")
    }
  }
  out$errorType <- errorType
  out$call <- cl
  return(out)
}


#' Simulation-Based Standard Errors for SV(p) Models
#'
#' Computes standard errors and confidence intervals for estimated parameters
#' by simulating from the fitted model and re-estimating. Supports all model
#' types returned by \code{\link{svp}}: Gaussian (with or without leverage),
#' Student-t, and GED.
#'
#' @param object A fitted model object from \code{\link{svp}}. Can be of class
#'   \code{"svp"}, \code{"svp_t"}, or \code{"svp_ged"}.
#' @param n_sim Integer. Number of Monte Carlo replications. Default 199.
#' @param alpha Numeric. Significance level for confidence intervals. Default 0.05.
#' @param burnin Integer. Burn-in period for simulation. Default 500.
#' @param logNu Logical. Solve for \eqn{\nu} in log-space for numerical
#'   stability (Student-t only). Default is \code{FALSE}.
#'
#' @return A list with:
#' \describe{
#'   \item{CI}{2 x k matrix of confidence intervals (lower, upper).}
#'   \item{SEsim0}{Standard errors relative to true parameter values.}
#'   \item{SEsim}{Standard errors relative to sample mean.}
#'   \item{ISEconservative}{Conservative interval-based standard errors.}
#'   \item{ISEliberal}{Liberal interval-based standard errors.}
#'   \item{thetamat}{Matrix of parameter estimates from simulations.}
#' }
#'
#' @examples
#' \donttest{
#' # Gaussian SV(1)
#' y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2)$y
#' fit <- svp(y)
#' se <- svpSE(fit, n_sim = 49)
#' se$CI
#' }
#'
#' @export
svpSE <- function(object, n_sim = 199, alpha = 0.05, burnin = 500,
                  logNu = FALSE) {
  if (!inherits(object, c("svp", "svp_t", "svp_ged"))) {
    stop("object must be of class 'svp', 'svp_t', or 'svp_ged'.")
  }
  if (n_sim < 1) stop("n_sim must be >= 1.")
  if (!is.numeric(alpha) || alpha <= 0 || alpha >= 1)
    stop("'alpha' must be a number in (0, 1).")
  if (!is.numeric(burnin) || burnin < 0)
    stop("'burnin' must be a non-negative integer.")

  if (inherits(object, "svp_t")) {
    .svpSE_t(object, n_sim, alpha, burnin, logNu)
  } else if (inherits(object, "svp_ged")) {
    .svpSE_ged(object, n_sim, alpha, burnin)
  } else {
    .svpSE_gaussian(object, n_sim, alpha, burnin)
  }
}

test_that("sim_svp Gaussian SV(1) returns list of length-n vectors", {
  set.seed(1)
  sim <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.3)
  expect_type(sim, "list")
  expect_named(sim, c("y", "h", "z", "v"))
  expect_length(sim$y, 500)
  expect_length(sim$h, 500)
  expect_length(sim$z, 500)
  expect_length(sim$v, 500)
  expect_type(sim$y, "double")
})

test_that("sim_svp Gaussian SV(2) returns correct length", {
  set.seed(1)
  sim <- sim_svp(500, phi = c(0.20, 0.63), sigy = 1, sigv = 0.5)
  expect_named(sim, c("y", "h", "z", "v"))
  expect_length(sim$y, 500)
})

test_that("sim_svp Student-t returns correct length", {
  set.seed(1)
  sim <- sim_svp(500, phi = 0.90, sigy = 1, sigv = 0.3,
                 errorType = "Student-t", nu = 5)
  expect_named(sim, c("y", "h", "z", "v"))
  expect_length(sim$y, 500)
})

test_that("sim_svp GED returns correct length", {
  set.seed(1)
  sim <- sim_svp(500, phi = 0.90, sigy = 1, sigv = 0.3,
                 errorType = "GED", nu = 1.5)
  expect_named(sim, c("y", "h", "z", "v"))
  expect_length(sim$y, 500)
})

test_that("sim_svp leverage returns list with expected components", {
  set.seed(1)
  sim <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.3,
                 leverage = TRUE, rho = -0.5)
  expect_type(sim, "list")
  expect_named(sim, c("y", "h", "z", "v"))
  expect_length(sim$y, 500)
})

test_that("sim_svp output satisfies model equation y = sigy * exp(h/2) * z", {
  set.seed(1)
  sim <- sim_svp(500, phi = 0.95, sigy = 1.5, sigv = 0.3)
  expect_equal(sim$y, 1.5 * exp(sim$h / 2) * sim$z, tolerance = 1e-10)
})

test_that("sim_svp rejects invalid inputs", {
  expect_error(sim_svp(500, phi = 0.9, sigy = 1, sigv = 0.3,
                       errorType = "Student-t", nu = 1.5),
               "nu must be > 2")
  expect_error(sim_svp(500, phi = 0.9, sigy = 1, sigv = 0.3,
                       errorType = "GED", nu = -1),
               "nu must be > 0")
  expect_error(sim_svp(500, phi = 0.9, sigy = 1, sigv = 0.3,
                       errorType = "Student-t"),
               "nu is required")
})

# =========================================================================== #
# Regression test: simulation uses contemporaneous epsilon (Bug 1 fix)
# =========================================================================== #

test_that("Gaussian sim: E[log(y_t^2)] and variance match SV model (no lag contamination)", {
  set.seed(2024)
  # Large sample to detect any systematic bias from lagged eps
  sim <- sim_svp(5000, phi = 0.95, sigy = 1, sigv = 0.3)
  y <- sim$y
  # Under the correct model: E[log(y^2)] = log(sigy^2) + E[log(eps^2)]
  # = log(1) + digamma(0.5) + log(2) ≈ -0.2704
  expected_mean <- log(1^2) + digamma(0.5) + log(2)
  # With lagged eps the mean would be the same (iid eps), but the cross-corr
  # structure changes. Test that estimation recovers phi well (key check).
  fit <- svp(y)
  expect_true(abs(fit$phi - 0.95) < 0.15)
  expect_true(fit$sigy > 0.8 && fit$sigy < 1.2)
})

# =========================================================================== #
# Tests for Bootstrap Particle Filter (BPF)
# These tests are slow due to particle filter simulations
# =========================================================================== #

test_that("PF returns correct class and components (Gaussian)", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(300, phi = 0.95, sigy = 1, sigv = 0.3)$y
  fit <- svp(y, p = 1)
  pf <- filter_svp(fit, method = "particle", M = 200, seed = 42)
  expect_s3_class(pf, "svp_filter")
  expect_equal(pf$method, "particle")
  expect_equal(length(pf$w_filtered), 300)
  expect_equal(length(pf$P_filtered), 300)
  expect_equal(length(pf$ESS), 300)
  expect_true(is.finite(pf$loglik))
})

test_that("PF: ESS doesn't collapse (Gaussian)", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(300, phi = 0.95, sigy = 1, sigv = 0.3)$y
  fit <- svp(y, p = 1)
  pf <- filter_svp(fit, method = "particle", M = 500, seed = 42)
  # Mean ESS should be > M/10 = 50
  expect_true(mean(pf$ESS) > 500 / 10)
})

test_that("PF: filtered states close to CKF (Gaussian)", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.3)$y
  fit <- svp(y, p = 1)
  ckf <- filter_svp(fit, method = "corrected")
  pf <- filter_svp(fit, method = "particle", M = 1000, seed = 42)
  cor_val <- cor(pf$w_filtered, ckf$w_filtered)
  expect_true(cor_val > 0.7)
})

test_that("PF: no NaN/Inf in output (all distributions)", {
  skip_on_cran()
  # Gaussian
  set.seed(42)
  y <- sim_svp(200, phi = 0.95, sigy = 1, sigv = 0.3)$y
  fit <- svp(y, p = 1)
  pf <- filter_svp(fit, method = "particle", M = 200, seed = 42)
  expect_true(all(is.finite(pf$w_filtered)))
  expect_true(all(is.finite(pf$ESS)))

  # Student-t
  set.seed(42)
  y_t <- sim_svp(200, phi = 0.95, sigy = 1, sigv = 0.3,
                 errorType = "Student-t", nu = 5)$y
  fit_t <- svp(y_t, p = 1, errorType = "Student-t")
  pf_t <- filter_svp(fit_t, method = "particle", M = 200, seed = 42)
  expect_true(all(is.finite(pf_t$w_filtered)))

  # GED
  set.seed(42)
  y_g <- sim_svp(200, phi = 0.95, sigy = 1, sigv = 0.3,
                 errorType = "GED", nu = 1.5)$y
  fit_g <- suppressWarnings(svp(y_g, p = 1, errorType = "GED"))
  pf_g <- filter_svp(fit_g, method = "particle", M = 200, seed = 42)
  expect_true(all(is.finite(pf_g$w_filtered)))
})

test_that("PF: ESS doesn't collapse (Student-t)", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(300, phi = 0.95, sigy = 1, sigv = 0.3,
               errorType = "Student-t", nu = 5)$y
  fit <- svp(y, p = 1, errorType = "Student-t")
  pf <- filter_svp(fit, method = "particle", M = 500, seed = 42)
  expect_true(mean(pf$ESS) > 500 / 10)
})

test_that("PF: leverage (Gaussian)", {
  skip_on_cran()
  set.seed(42)
  sim <- sim_svp(300, phi = 0.95, sigy = 1, sigv = 0.3,
                 leverage = TRUE, rho = -0.5)
  fit <- svp(sim$y, p = 1, leverage = TRUE)
  pf <- filter_svp(fit, method = "particle", M = 300, seed = 42)
  expect_true(all(is.finite(pf$w_filtered)))
  expect_true(mean(pf$ESS) > 300 / 10)
})

test_that("PF: leverage (Student-t, exact z recovery via lambda sampling)", {
  skip_on_cran()
  set.seed(42)
  sim <- sim_svp(300, phi = 0.95, sigy = 1, sigv = 0.3,
                 errorType = "Student-t", nu = 5,
                 leverage = TRUE, rho = -0.5)
  fit <- svp(sim$y, p = 1, leverage = TRUE, errorType = "Student-t")
  pf <- filter_svp(fit, method = "particle", M = 300, seed = 42)
  expect_true(all(is.finite(pf$w_filtered)))
  expect_true(mean(pf$ESS) > 300 / 10)
})

test_that("PF: leverage (GED, exact z via copula inversion)", {
  skip_on_cran()
  set.seed(42)
  sim <- sim_svp(300, phi = 0.95, sigy = 1, sigv = 0.3,
                 errorType = "GED", nu = 1.5,
                 leverage = TRUE, rho = -0.5)
  fit <- svp(sim$y, p = 1, leverage = TRUE, errorType = "GED")
  pf <- filter_svp(fit, method = "particle", M = 300, seed = 42)
  expect_true(all(is.finite(pf$w_filtered)))
  expect_true(mean(pf$ESS) > 300 / 10)
})

test_that("PF: SV(2) companion form works", {
  skip_on_cran()
  set.seed(77)
  y <- sim_svp(300, phi = c(0.2, 0.63), sigy = 1, sigv = 0.5)$y
  fit <- svp(y, p = 2)
  pf <- filter_svp(fit, method = "particle", M = 300, seed = 42)
  expect_true(all(is.finite(pf$w_filtered)))
  expect_true(mean(pf$ESS) > 300 / 10)
})

test_that("PF: reproducibility with same seed", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(200, phi = 0.95, sigy = 1, sigv = 0.3)$y
  fit <- svp(y, p = 1)
  pf1 <- filter_svp(fit, method = "particle", M = 200, seed = 123)
  pf2 <- filter_svp(fit, method = "particle", M = 200, seed = 123)
  expect_equal(pf1$w_filtered, pf2$w_filtered)
  expect_equal(pf1$loglik, pf2$loglik)
})

test_that("print.svp_filter works for particle filter", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(200, phi = 0.95, sigy = 1, sigv = 0.3)$y
  fit <- svp(y, p = 1)
  pf <- filter_svp(fit, method = "particle", M = 200, seed = 42)
  expect_output(print(pf), "particle")
})

# Tests for the 2026-05-09 filter parameterization fixes:
#   Fix 1: var_zt = 1 in CKF/GMKF/forecast (was nu/(nu-2) for Student-t leverage)
#   Fix 3: BPF posterior lambda for Student-t leverage (was prior)
#   Fix 4: GED copula proxy in CKF/GMKF leverage shift
# Plus the proxy = c("u", "bayes_optimal") API.

test_that("filter_svp accepts proxy argument; default is 'u'", {
  set.seed(1)
  y <- sim_svp(800, phi = 0.95, sigy = 1, sigv = 0.3,
               errorType = "Student-t", nu = 5,
               leverage = TRUE, rho = -0.4)$y
  fit <- suppressWarnings(svp(y, p = 1, errorType = "Student-t",
                               leverage = TRUE))
  expect_silent(filter_svp(fit, method = "mixture"))
  expect_silent(filter_svp(fit, method = "mixture", proxy = "u"))
  expect_silent(filter_svp(fit, method = "mixture", proxy = "bayes_optimal"))
})

test_that("proxy choice differs only for Student-t with leverage", {
  set.seed(2)
  # Gaussian no-leverage: proxy is irrelevant (zeta = u exactly)
  y_g <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.4)$y
  fit_g <- svp(y_g, p = 1)
  f_g_u <- filter_svp(fit_g, method = "corrected", proxy = "u")
  f_g_b <- filter_svp(fit_g, method = "corrected", proxy = "bayes_optimal")
  expect_equal(f_g_u$loglik, f_g_b$loglik, tolerance = 1e-10)

  # Gaussian with leverage: still proxy-irrelevant (zeta = u for Gaussian)
  y_gl <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.3,
                   leverage = TRUE, rho = -0.3)$y
  fit_gl <- svp(y_gl, p = 1, leverage = TRUE)
  f_gl_u <- filter_svp(fit_gl, method = "corrected", proxy = "u")
  f_gl_b <- filter_svp(fit_gl, method = "corrected", proxy = "bayes_optimal")
  expect_equal(f_gl_u$loglik, f_gl_b$loglik, tolerance = 1e-10)

  # Student-t with leverage: proxy DOES matter
  y_t <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.3,
                  errorType = "Student-t", nu = 5,
                  leverage = TRUE, rho = -0.4)$y
  fit_t <- suppressWarnings(svp(y_t, p = 1, errorType = "Student-t",
                                 leverage = TRUE))
  f_t_u <- filter_svp(fit_t, method = "corrected", proxy = "u")
  f_t_b <- filter_svp(fit_t, method = "corrected", proxy = "bayes_optimal")
  # The two log-likelihoods should differ for Student-t leverage
  expect_false(isTRUE(all.equal(f_t_u$loglik, f_t_b$loglik,
                                 tolerance = 1e-6)))

  # GED with leverage: proxy is irrelevant (the GED copula proxy is
  # deterministic and doesn't depend on the proxy_type flag)
  y_ged <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.3,
                    errorType = "GED", nu = 1.5,
                    leverage = TRUE, rho = -0.4)$y
  fit_ged <- suppressWarnings(svp(y_ged, p = 1, errorType = "GED",
                                   leverage = TRUE))
  f_ged_u <- filter_svp(fit_ged, method = "corrected", proxy = "u")
  f_ged_b <- filter_svp(fit_ged, method = "corrected", proxy = "bayes_optimal")
  expect_equal(f_ged_u$loglik, f_ged_b$loglik, tolerance = 1e-10)
})

test_that("BPF Student-t recovery uses the posterior (Bug 3 fix)", {
  skip_on_cran()
  set.seed(3)
  y <- sim_svp(600, phi = 0.95, sigy = 1, sigv = 0.3,
                errorType = "Student-t", nu = 5,
                leverage = TRUE, rho = -0.4)$y
  fit <- suppressWarnings(svp(y, p = 1, errorType = "Student-t",
                               leverage = TRUE))
  # Just ensure BPF runs and returns finite log-likelihood after the
  # posterior-lambda fix.  A stronger empirical-mean test would require
  # exposing the recovery step internals.
  pf <- filter_svp(fit, method = "particle", M = 500, seed = 7)
  expect_true(is.finite(pf$loglik))
  expect_true(all(is.finite(pf$w_filtered)))
})

test_that("svp_IC default proxy is 'bayes_optimal'", {
  skip_on_cran()
  set.seed(4)
  y <- sim_svp(800, phi = 0.95, sigy = 1, sigv = 0.3,
                errorType = "Student-t", nu = 5,
                leverage = TRUE, rho = -0.4)$y
  fit <- suppressWarnings(svp(y, p = 1, errorType = "Student-t",
                               leverage = TRUE))
  # Default svp_IC should match svp_IC(proxy = "bayes_optimal") exactly
  ic_default <- svp_IC(fit)
  ic_b       <- svp_IC(fit, proxy = "bayes_optimal")
  expect_equal(ic_default, ic_b, tolerance = 1e-8)
  # (Whether proxy="u" gives a numerically different value depends on the
  # estimated delta_p — small delta_p → small effect.  We verify the
  # forwarding instead of assuming a numerical difference here.)
})

test_that("var_zt fix does not affect Gaussian or no-leverage cells", {
  skip_on_cran()
  # All three combos that should be invariant to the fix:
  combos <- list(
    list(et = "Gaussian",  lev = FALSE),
    list(et = "Gaussian",  lev = TRUE,  rho = -0.3),
    list(et = "Student-t", lev = FALSE, nu = 5),
    list(et = "GED",       lev = FALSE, nu = 1.5)
  )
  for (i in seq_along(combos)) {
    set.seed(50 + i)
    args <- list(n = 500, phi = 0.95, sigy = 1, sigv = 0.3,
                 errorType = combos[[i]]$et, leverage = combos[[i]]$lev)
    if (!is.null(combos[[i]]$nu))  args$nu  <- combos[[i]]$nu
    if (!is.null(combos[[i]]$rho)) args$rho <- combos[[i]]$rho
    y <- do.call(sim_svp, args)$y
    fit <- suppressWarnings(svp(y, p = 1,
                                 errorType = combos[[i]]$et,
                                 leverage = combos[[i]]$lev))
    f <- filter_svp(fit, method = "corrected")
    expect_true(is.finite(f$loglik),
                info = paste("combo", i, combos[[i]]$et,
                              "leverage", combos[[i]]$lev))
  }
})

test_that("forecast_svp accepts proxy and forwards correctly", {
  skip_on_cran()
  set.seed(5)
  y <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.3,
                errorType = "Student-t", nu = 5,
                leverage = TRUE, rho = -0.4)$y
  fit <- suppressWarnings(svp(y, p = 1, errorType = "Student-t",
                               leverage = TRUE))
  fc_u <- forecast_svp(fit, H = 5, filter_method = "mixture", proxy = "u")
  fc_b <- forecast_svp(fit, H = 5, filter_method = "mixture",
                        proxy = "bayes_optimal")
  expect_true(all(is.finite(fc_u$vol_forecast)))
  expect_true(all(is.finite(fc_b$vol_forecast)))
  # Should differ (not identical) for Student-t leverage
  expect_false(isTRUE(all.equal(fc_u$vol_forecast, fc_b$vol_forecast,
                                 tolerance = 1e-6)))
})


test_that("forecast Q is conditional at h=1 and marginal at h>=2 (Bug 5 fix)", {
  skip_on_cran()
  set.seed(2026)
  y <- sim_svp(1500, phi = 0.95, sigy = 1, sigv = 0.4,
               leverage = TRUE, rho = -0.5)$y
  fit <- suppressWarnings(svp(y, p = 1, leverage = TRUE))
  fc <- forecast_svp(fit, H = 5, filter_method = "corrected")
  # Build the expected Q values
  phi <- as.numeric(fit$phi)
  sigv2 <- fit$sigv^2
  delta <- if (is.null(fit$rho) || is.na(fit$rho)) 0 else fit$rho
  Q_h1   <- sigv2 * (1 - delta^2)   # conditional at h=1
  Q_hge2 <- sigv2                    # marginal at h>=2
  # The Q recursion increment equals Q_h at each step:
  #   P_h - phi^2 * P_{h-1} = Q_h
  P <- fc$P_forecast
  # Use filter's P_filt_T as the seed for h=1
  P_T_11 <- fc$filter_output$P_filt_T[1, 1]
  expect_equal(P[1] - phi^2 * P_T_11, Q_h1, tolerance = 1e-6)
  # h>=2 increments should equal Q_hge2
  expect_equal(P[2] - phi^2 * P[1], Q_hge2, tolerance = 1e-6)
  expect_equal(P[3] - phi^2 * P[2], Q_hge2, tolerance = 1e-6)
})

test_that("svp Gaussian SV(1) returns correct class and reasonable estimates", {
  set.seed(42)
  y <- sim_svp(5000, phi = 0.95, sigy = 1, sigv = 0.3)$y
  fit <- svp(y, p = 1, J = 10)
  expect_s3_class(fit, "svp")
  expect_length(fit$phi, 1)
  # phi should be within 0.1 of true value

  expect_true(abs(fit$phi - 0.95) < 0.1)
  # sigy should be positive
  expect_true(fit$sigy > 0)
})

test_that("svp Gaussian SV(2) returns 2 phi values", {
  set.seed(42)
  y <- sim_svp(3000, phi = c(0.20, 0.63), sigy = 1, sigv = 0.5)$y
  fit <- svp(y, p = 2, J = 10)
  expect_s3_class(fit, "svp")
  expect_length(fit$phi, 2)
})

test_that("svp leverage returns rho estimate", {
  set.seed(42)
  sim <- sim_svp(3000, phi = 0.95, sigy = 1, sigv = 0.3,
                 leverage = TRUE, rho = -0.5)
  fit <- svp(sim$y, p = 1, leverage = TRUE)
  expect_s3_class(fit, "svp")
  expect_false(is.na(fit$rho))
  # rho should be negative (true = -0.5)
  expect_true(fit$rho < 0)
})

test_that("svp Student-t returns correct class and reasonable nu", {
  set.seed(42)
  y <- sim_svp(5000, phi = 0.90, sigy = 1, sigv = 0.3,
               errorType = "Student-t", nu = 5)$y
  fit <- suppressWarnings(svp(y, p = 1, errorType = "Student-t"))
  expect_s3_class(fit, "svp_t")
  expect_true(!is.na(fit$v))
  # nu should be in reasonable range (true = 5)
  expect_true(fit$v > 2 && fit$v < 50)
  # sigy should be positive and not too far from 1
  expect_true(fit$sigy > 0)
  expect_true(abs(fit$sigy - 1) < 0.5)
})

test_that("svp GED returns correct class and reasonable nu", {
  set.seed(42)
  y <- sim_svp(5000, phi = 0.90, sigy = 1, sigv = 0.3,
               errorType = "GED", nu = 1.5)$y
  fit <- suppressWarnings(svp(y, p = 1, errorType = "GED"))
  expect_s3_class(fit, "svp_ged")
  expect_true(!is.na(fit$v))
  # nu should be in reasonable range (true = 1.5)
  expect_true(fit$v > 0.3 && fit$v < 5)
})

test_that("svp Student-t SV(2) works", {
  set.seed(42)
  y <- sim_svp(3000, phi = c(0.20, 0.63), sigy = 1, sigv = 0.5,
               errorType = "Student-t", nu = 10)$y
  fit <- suppressWarnings(svp(y, p = 2, errorType = "Student-t"))
  expect_s3_class(fit, "svp_t")
  expect_length(fit$phi, 2)
})

test_that("svpSE returns CI and SE components", {
  set.seed(42)
  y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.3)$y
  fit <- svp(y, p = 1)
  se <- svpSE(fit, n_sim = 19)
  expect_type(se, "list")
  expect_true("CI" %in% names(se))
  expect_true("SEsim" %in% names(se))
  # CI should have 2 rows (lower, upper) and 3 columns (phi, sigy, sigv)
  expect_equal(nrow(se$CI), 2)
  expect_equal(ncol(se$CI), 3)
})

# =========================================================================== #
# Regression test: nonstationary_ind propagated to svp_t and svp_ged (Bug 6 fix)
# =========================================================================== #

test_that("svp_t object contains nonstationary_ind field", {
  set.seed(42)
  y <- sim_svp(1000, phi = 0.90, sigy = 1, sigv = 0.3,
               errorType = "Student-t", nu = 5)$y
  fit <- suppressWarnings(svp(y, errorType = "Student-t"))
  expect_true("nonstationary_ind" %in% names(fit))
  expect_type(fit$nonstationary_ind, "logical")
})

test_that("svp_ged object contains nonstationary_ind field", {
  set.seed(42)
  y <- sim_svp(1000, phi = 0.90, sigy = 1, sigv = 0.3,
               errorType = "GED", nu = 1.5)$y
  fit <- suppressWarnings(svp(y, errorType = "GED"))
  expect_true("nonstationary_ind" %in% names(fit))
  expect_type(fit$nonstationary_ind, "logical")
})

test_that("coef method works for all model types", {
  set.seed(42)
  y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.3)$y
  fit <- svp(y)
  expect_true(is.numeric(coef(fit)))

  yt <- sim_svp(1000, phi = 0.90, sigy = 1, sigv = 0.3,
                errorType = "Student-t", nu = 5)$y
  fit_t <- suppressWarnings(svp(yt, errorType = "Student-t"))
  if (!is.na(fit_t$v)) {
    expect_true(is.numeric(coef(fit_t)))
  }
})

# =========================================================================== #
# Tests for forecasting
# =========================================================================== #

test_that("filter_svp CKF runs and returns expected components", {
  set.seed(42)
  y <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.3)$y
  mdl <- svp(y, p = 1)
  filt <- filter_svp(mdl)
  expect_s3_class(filt, "svp_filter")
  expect_true("w_filtered" %in% names(filt))
  expect_true("w_smoothed" %in% names(filt))
  expect_true("P_filtered" %in% names(filt))
  expect_true("loglik" %in% names(filt))
  expect_true("P_filt_T" %in% names(filt))
})

test_that("forecast_svp accepts model object and produces h-step forecasts", {
  set.seed(42)
  sim <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.3,
                 leverage = TRUE, rho = -0.3)
  fit <- svp(sim$y, p = 1, leverage = TRUE)
  fc <- forecast_svp(fit, H = 5)
  expect_s3_class(fc, "svp_forecast")
  expect_true("w_forecasted" %in% names(fc))
  expect_length(fc$w_forecasted, 5)
})

test_that("forecast_svp errors on raw numeric input", {
  set.seed(42)
  y <- sim_svp(200, phi = 0.95, sigy = 1, sigv = 0.3)$y
  expect_error(forecast_svp(y, H = 5), "svp/svp_t/svp_ged")
})

test_that("forecast_svp: all three output representations are stored", {
  set.seed(42)
  y <- sim_svp(300, phi = 0.95, sigy = 1, sigv = 0.3)$y
  fit <- svp(y, p = 1)
  fc <- forecast_svp(fit, H = 10)
  expect_length(fc$log_var_forecast, 10)
  expect_length(fc$var_forecast, 10)
  expect_length(fc$vol_forecast, 10)
  expect_length(fc$P_forecast, 10)
  # Default output is log-variance
  expect_equal(fc$w_forecasted, fc$log_var_forecast)
})

test_that("forecast_svp: output parameter selects primary output", {
  set.seed(42)
  y <- sim_svp(300, phi = 0.95, sigy = 1, sigv = 0.3)$y
  fit <- svp(y, p = 1)
  fc_var <- forecast_svp(fit, H = 5, output = "variance")
  expect_equal(fc_var$w_forecasted, fc_var$var_forecast)
  fc_vol <- forecast_svp(fit, H = 5, output = "volatility")
  expect_equal(fc_vol$w_forecasted, fc_vol$vol_forecast)
})

test_that("forecast_svp: P_forecast is positive and increasing", {
  set.seed(42)
  y <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.3)$y
  fit <- svp(y, p = 1)
  fc <- forecast_svp(fit, H = 20)
  expect_true(all(fc$P_forecast > 0))
  # MSE should generally increase (not strictly for all h, but trend)
  expect_true(fc$P_forecast[20] > fc$P_forecast[1])
})

test_that("forecast_svp: sigma2_forecast is positive", {
  set.seed(42)
  y <- sim_svp(300, phi = 0.95, sigy = 1, sigv = 0.3)$y
  fit <- svp(y, p = 1)
  fc <- forecast_svp(fit, H = 10)
  expect_true(all(fc$var_forecast > 0))
  expect_true(all(fc$vol_forecast > 0))
})

test_that("forecast_svp: long horizon converges toward 0 (log-var)", {
  set.seed(42)
  y <- sim_svp(500, phi = 0.90, sigy = 1, sigv = 0.3)$y
  fit <- svp(y, p = 1)
  fc <- forecast_svp(fit, H = 100)
  # Long-horizon log-variance forecast should converge toward 0
  expect_true(abs(fc$log_var_forecast[100]) < abs(fc$log_var_forecast[1]) + 0.1)
})

test_that("forecast_svp: Student-t uses E[u^2] = nu/(nu-2)", {
  set.seed(42)
  sim <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.3,
                 errorType = "Student-t", nu = 5)
  fit <- svp(sim$y, p = 1, errorType = "Student-t")
  fc <- forecast_svp(fit, H = 5)
  # var_forecast should incorporate nu/(nu-2) = 5/3 factor
  expect_true(all(fc$var_forecast > 0))
})

test_that("forecast_svp: GED uses E[u^2] = 1", {
  set.seed(42)
  sim <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.3,
                 errorType = "GED", nu = 1.5)
  fit <- svp(sim$y, p = 1, errorType = "GED")
  fc <- forecast_svp(fit, H = 5)
  expect_true(all(fc$var_forecast > 0))
})

test_that("plot.svp_forecast works without error", {
  set.seed(42)
  sim <- sim_svp(300, phi = 0.95, sigy = 1, sigv = 0.3,
                 leverage = TRUE, rho = -0.3)
  fit <- svp(sim$y, p = 1, leverage = TRUE)
  fc <- forecast_svp(fit, H = 5)
  expect_silent(plot(fc))
})

test_that("companionMat returns correct dimensions", {
  C <- wARMASVp:::companionMat(c(0.7, 0.2), p = 2, q = 1)
  expect_equal(dim(C), c(2, 2))
  expect_equal(unname(C[1, 1]), 0.7)
  expect_equal(unname(C[1, 2]), 0.2)
})


# =========================================================================== #
# Regression test: forecast MSE uses full P_filt_T matrix (Bug 3 fix)
# =========================================================================== #

test_that("forecast_svp p=2: P_forecast increases with horizon (full P_T)", {
  set.seed(42)
  y <- sim_svp(500, phi = c(0.20, 0.63), sigy = 1, sigv = 1)$y
  fit <- svp(y, p = 2)
  fc <- forecast_svp(fit, H = 5)
  # MSE should be non-decreasing with horizon for a stationary AR
  expect_true(all(diff(fc$P_forecast) >= -1e-10))
  # All MSE values should be positive
  expect_true(all(fc$P_forecast > 0))
  # Var forecast should all be positive
  expect_true(all(fc$var_forecast > 0))
})

# =========================================================================== #
# BPF forecasting now supported (Fix 4, 2026-04-03)
# =========================================================================== #

test_that("forecast_svp works with filter_method='particle'", {
  set.seed(42)
  y <- sim_svp(300, phi = 0.95, sigy = 1, sigv = 0.3)$y
  fit <- svp(y, p = 1)
  fc <- forecast_svp(fit, H = 3, filter_method = "particle")
  expect_s3_class(fc, "svp_forecast")
  expect_equal(length(fc$log_var_forecast), 3)
  expect_true(all(is.finite(fc$log_var_forecast)))
  # P_filt_T should be a 1x1 matrix from BPF
  expect_equal(dim(fc$filter_output$P_filt_T), c(1L, 1L))
})

# =========================================================================== #
# Hypothesis testing functions for SV(p) models
# =========================================================================== #

# Internal validation for common test arguments
.validate_test_common <- function(y, J, N, burnin, del) {
  if (length(y) < 1L) stop("'y' must be a non-empty numeric vector.")
  if (any(!is.finite(y))) stop("'y' must not contain NA, NaN, or Inf values.")
  if (!is.numeric(J) || length(J) != 1L || J < 1L)
    stop("'J' must be a positive integer (>= 1).")
  if (!is.numeric(N) || length(N) != 1L || N < 1L)
    stop("'N' must be a positive integer (>= 1).")
  if (!is.numeric(burnin) || length(burnin) != 1L || burnin < 0)
    stop("'burnin' must be a non-negative integer.")
  if (!is.numeric(del) || length(del) != 1L || del <= 0)
    stop("'del' must be a positive number.")
}


# =========================================================================== #
# AR Order Tests
# =========================================================================== #

#' LMC Test for AR Order in SV(p) Models
#'
#' Performs a Local Monte Carlo (LMC) test of the null hypothesis
#' \eqn{H_0: \phi_{p_0+1} = \cdots = \phi_p = 0} (i.e., that an SV(\eqn{p_0})
#' model is sufficient against an SV(\eqn{p}) alternative).
#'
#' @param y Numeric vector. Observed returns.
#' @param p_null Integer. Order under the null hypothesis.
#' @param p_alt Integer. Order under the alternative (\code{p_alt > p_null}).
#' @param J Integer. Winsorizing parameter. Default 10.
#' @param N Integer. Number of Monte Carlo replications. Default 99.
#' @param burnin Integer. Burn-in for simulation. Default 500.
#' @param del Numeric. Small constant for log transformation. Default \code{1e-10}.
#' @param wDecay Logical. Use decaying weights. Default \code{FALSE}.
#' @param Bartlett Logical. If \code{TRUE}, use Bartlett kernel HAC weighting
#'   matrix for a GMM-LRT-type test statistic. If \code{FALSE} (default), use
#'   the sum of squared extra AR coefficients.
#' @param Amat Weighting matrix specification. \code{NULL} (default) for identity
#'   weighting, or \code{"Weighted"} for data-driven HAC. Takes precedence over
#'   \code{Bartlett}. User-supplied matrices are not supported for AR order tests.
#' @param errorType Character. Error distribution of the return innovations:
#'   \code{"Gaussian"} (default), \code{"Student-t"}, or \code{"GED"}. Heavy-tail
#'   options reuse the same moment-based GMM-LRT machinery as \code{lmc_t}/
#'   \code{lmc_ged}; \eqn{\nu} is held at the null MLE during the simulation
#'   (it is not a varied nuisance for the AR-order test).
#' @param sigvMethod Character. Method for \eqn{\sigma_v} estimation:
#'   \code{"factored"} (default), \code{"hybrid"}, or \code{"direct"}.
#' @param logNu Logical. Use log-space for \eqn{\nu} estimation (Student-t/GED
#'   only). Default \code{TRUE}.
#' @param winsorize_eps Number of extreme autocovariance lags to winsorize
#'   (heavy-tail only). Default 0.
#'
#' @return An object of class \code{"svp_test"}, a list containing:
#' \describe{
#'   \item{s0}{Test statistic from observed data (capped at 1e-10 if negative).}
#'   \item{sN}{Simulated null distribution (vector of length N).}
#'   \item{pval}{Monte Carlo p-value.}
#'   \item{test_type}{Character string identifying the test.}
#'   \item{null_param}{Name of the parameter(s) tested.}
#'   \item{null_value}{Value(s) under the null hypothesis.}
#'   \item{errorType}{Error distribution used.}
#'   \item{call}{The matched call.}
#' }
#'
#' @details
#' When \code{Bartlett = FALSE} (default), the test statistic is
#' \eqn{T \sum_{j=p_0+1}^{p} \hat\phi_j^2}, i.e., the sum of squared extra
#' AR coefficients scaled by sample size.
#'
#' When \code{Bartlett = TRUE}, the test statistic is based on the GMM-LRT
#' approach with a Bartlett kernel HAC weighting matrix:
#' \eqn{S = T \times (M_{H_0} - M_{H_1})}, where \eqn{M} denotes the
#' GMM criterion evaluated at the null and alternative estimates. Both the
#' observed and simulated test statistics are capped at \code{1e-10} when
#' negative; a negative observed statistic raises a warning (it indicates strong
#' evidence in favour of the null, since the alternative does not improve the
#' GMM criterion).
#'
#' @examples
#' \donttest{
#' y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2)$y
#' test <- lmc_ar(y, p_null = 1, p_alt = 2, J = 10, N = 49)
#' print(test)
#' }
#'
#' @export
lmc_ar <- function(y, p_null, p_alt, J = 10, N = 99, burnin = 500,
                   del = 1e-10, wDecay = FALSE, Bartlett = FALSE,
                   Amat = NULL, errorType = "Gaussian",
                   sigvMethod = "factored", logNu = TRUE,
                   winsorize_eps = 0) {
  cl <- match.call()
  y_vec <- as.numeric(y)
  .validate_test_common(y_vec, J, N, burnin, del)
  Tsize <- length(y_vec)
  if (p_null >= p_alt) stop("p_alt must be greater than p_null.")
  if (p_null < 1) stop("p_null must be >= 1.")
  errorType <- match.arg(errorType, c("Gaussian", "Student-t", "GED"))
  sigvMethod <- match.arg(sigvMethod, c("hybrid", "direct", "factored"))
  wt <- .resolve_weighting(Amat, Bartlett)
  if (is.matrix(wt$Amat))
    stop("User-supplied weighting matrices are not supported for AR order tests. Use Amat = 'Weighted' for HAC.")

  # Estimate models under null and alternative
  if (errorType == "Gaussian") {
    mdl_alt <- svp(y_vec, p = p_alt, J = J, leverage = FALSE, del = del,
                   wDecay = wDecay, sigvMethod = sigvMethod)
    mdl_null_est <- svp(y_vec, p = p_null, J = J, leverage = FALSE, del = del,
                        wDecay = wDecay, sigvMethod = sigvMethod)
  } else {
    mdl_alt <- svp(y_vec, p = p_alt, J = J, leverage = FALSE,
                   errorType = errorType, del = del, wDecay = wDecay,
                   logNu = logNu, sigvMethod = sigvMethod,
                   winsorize_eps = winsorize_eps)
    mdl_null_est <- svp(y_vec, p = p_null, J = J, leverage = FALSE,
                        errorType = errorType, del = del, wDecay = wDecay,
                        logNu = logNu, sigvMethod = sigvMethod,
                        winsorize_eps = winsorize_eps)
  }

  # Compute test statistic
  y_mat <- as.matrix(y_vec)
  if (wt$use_hac) {
    if (errorType == "Gaussian") {
      M_null <- LRT_moment_ar_Amat(y_vec, mdl_null_est, del = del, Bartlett = TRUE)
      M_alt  <- LRT_moment_ar_Amat(y_vec, mdl_alt, del = del, Bartlett = TRUE)
    } else if (errorType == "Student-t") {
      wa_null <- .parse_Amat("Weighted", p_null)
      wa_alt  <- .parse_Amat("Weighted", p_alt)
      M_null <- LRT_moment_t(y_mat, mdl_null_est, wa_null$Amat, wa_null$WAmat, del, TRUE)
      M_alt  <- LRT_moment_t(y_mat, mdl_alt, wa_alt$Amat, wa_alt$WAmat, del, TRUE)
    } else {  # GED
      wa_null <- .parse_Amat("Weighted", p_null)
      wa_alt  <- .parse_Amat("Weighted", p_alt)
      M_null <- LRT_moment_ged(y_mat, mdl_null_est, wa_null$Amat, wa_null$WAmat, del, TRUE)
      M_alt  <- LRT_moment_ged(y_mat, mdl_alt, wa_alt$Amat, wa_alt$WAmat, del, TRUE)
    }
    s0_tmp <- Tsize * (M_null - M_alt)
  } else {
    phi_extra <- mdl_alt$phi[(p_null + 1):p_alt]
    s0_tmp <- Tsize * sum(phi_extra^2)
  }

  # Cap negative observed test statistic for consistency with simulated null
  if (is.na(s0_tmp)) stop("Test statistic is NA.")
  if (s0_tmp < 0) warning("Test statistic is negative (", round(s0_tmp, 4),
                          "); capped at 1e-10. May indicate strong evidence for null.")
  s0 <- max(s0_tmp, 1e-10)

  # Build betasim_null (length p_null+2 for Gaussian; p_null+3 for heavy-tail)
  if (errorType == "Gaussian") {
    betasim_null <- c(mdl_null_est$phi, mdl_null_est$sigy, mdl_null_est$sigv)
  } else {
    betasim_null <- c(mdl_null_est$phi, mdl_null_est$sigy, mdl_null_est$sigv,
                      mdl_null_est$v)
  }

  # Simulate null distribution (nu fixed at null MLE for heavy-tail innovation draws)
  nu_fixed <- if (errorType == "Gaussian") NA_real_ else mdl_null_est$v
  sN <- .simnull_ar(betasim_null, p_null, p_alt, J, Tsize, N, burnin,
                    del, wDecay, wt$use_hac, sigvMethod = sigvMethod,
                    errorType = errorType, nu_fixed = nu_fixed,
                    logNu = logNu, winsorize_eps = winsorize_eps)
  pval <- (N + 1 - sum(s0 >= sN)) / (N + 1)
  label_dist <- if (errorType == "Gaussian") "" else sprintf(" [%s]", errorType)
  test_label <- if (wt$use_hac) {
    sprintf("LMC AR Order Bartlett%s (p0=%d vs p=%d)", label_dist, p_null, p_alt)
  } else {
    sprintf("LMC AR Order%s (p0=%d vs p=%d)", label_dist, p_null, p_alt)
  }
  out <- list(s0 = s0, sN = as.numeric(sN), pval = pval,
              test_type = test_label,
              null_param = paste0("phi_", (p_null + 1):p_alt),
              null_value = rep(0, p_alt - p_null),
              errorType = errorType,
              call = cl)
  class(out) <- "svp_test"
  return(out)
}


#' MMC Test for AR Order in SV(p) Models
#'
#' Performs a Maximized Monte Carlo (MMC) test of
#' \eqn{H_0: \phi_{p_0+1} = \cdots = \phi_p = 0}
#' by maximizing the MC p-value over nuisance parameters
#' (\eqn{\phi_1, \ldots, \phi_{p_0}, \sigma_y, \sigma_v}).
#'
#' @inheritParams lmc_ar
#' @param eps Numeric vector. Half-width of search region around estimated
#'   nuisance parameters. Default \code{rep(0.3, p_null+2)}.
#' @param threshold Numeric. Target p-value. Default 1.
#' @param method Character. Optimization method: \code{"pso"} or \code{"GenSA"}.
#'   Default \code{"pso"}.
#' @param maxit Integer. Maximum iterations/evaluations. Default depends on method.
#'
#' @return A list with optimization output including \code{value} (maximized p-value)
#'   and \code{par} (nuisance parameters at the maximum).
#'
#' @examples
#' \donttest{
#' y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2)$y
#' mmc <- mmc_ar(y, p_null = 1, p_alt = 2, J = 10, N = 19,
#'               method = "pso", maxit = 10)
#' mmc$value
#' }
#'
#' @export
mmc_ar <- function(y, p_null, p_alt, J = 10, N = 99, burnin = 500,
                   eps = NULL, threshold = 1, method = "pso", maxit = NULL,
                   del = 1e-10, wDecay = FALSE, Bartlett = FALSE,
                   Amat = NULL, errorType = "Gaussian",
                   sigvMethod = "factored", logNu = TRUE,
                   winsorize_eps = 0) {
  cl <- match.call()
  y_vec <- as.numeric(y)
  .validate_test_common(y_vec, J, N, burnin, del)
  Tsize <- length(y_vec)
  if (p_null >= p_alt) stop("p_alt must be greater than p_null.")
  if (p_null < 1) stop("p_null must be >= 1.")
  errorType <- match.arg(errorType, c("Gaussian", "Student-t", "GED"))
  sigvMethod <- match.arg(sigvMethod, c("hybrid", "direct", "factored"))
  wt <- .resolve_weighting(Amat, Bartlett)
  if (is.matrix(wt$Amat))
    stop("User-supplied weighting matrices are not supported for AR order tests. Use Amat = 'Weighted' for HAC.")

  # Estimate under alternative and null for test statistic
  if (errorType == "Gaussian") {
    mdl_alt <- svp(y_vec, p = p_alt, J = J, leverage = FALSE, del = del,
                   wDecay = wDecay, sigvMethod = sigvMethod)
    mdl_null_est <- svp(y_vec, p = p_null, J = J, leverage = FALSE, del = del,
                        wDecay = wDecay, sigvMethod = sigvMethod)
    nu_fixed <- NA_real_
  } else {
    mdl_alt <- svp(y_vec, p = p_alt, J = J, leverage = FALSE,
                   errorType = errorType, del = del, wDecay = wDecay,
                   logNu = logNu, sigvMethod = sigvMethod,
                   winsorize_eps = winsorize_eps)
    mdl_null_est <- svp(y_vec, p = p_null, J = J, leverage = FALSE,
                        errorType = errorType, del = del, wDecay = wDecay,
                        logNu = logNu, sigvMethod = sigvMethod,
                        winsorize_eps = winsorize_eps)
    # nu held fixed at null MLE during optimization (not a varied nuisance for AR test)
    nu_fixed <- mdl_null_est$v
  }

  theta_0 <- c(mdl_null_est$phi, mdl_null_est$sigy, mdl_null_est$sigv)
  n_nuisance <- p_null + 2
  if (is.null(eps)) {
    eps <- rep(0.3, n_nuisance)
    eps[p_null + 1] <- 0  # sigma_y: test stat is scale-invariant, fixing avoids spurious optimization
  }
  if (length(eps) != n_nuisance)
    stop("eps must have length ", n_nuisance,
         " (p_null+2: one entry per nuisance parameter phi_1,...,phi_p_null, sigma_y, sigma_v).")
  # Bounds for nuisance parameters
  lower <- c(pmax(theta_0[1:p_null] - eps[1:p_null], rep(-0.999, p_null)),
             max(theta_0[p_null + 1] - eps[p_null + 1], 0.01),
             max(theta_0[p_null + 2] - eps[p_null + 2], 0.01))
  upper <- c(pmin(theta_0[1:p_null] + eps[1:p_null], rep(0.999, p_null)),
             theta_0[p_null + 1] + eps[p_null + 1],
             theta_0[p_null + 2] + eps[p_null + 2])

  # Observed test statistic (computed once, kept fixed during optimization per Dufour 2006 eq 4.22)
  y_mat <- as.matrix(y_vec)
  if (wt$use_hac) {
    if (errorType == "Gaussian") {
      M_null <- LRT_moment_ar_Amat(y_vec, mdl_null_est, del = del, Bartlett = TRUE)
      M_alt  <- LRT_moment_ar_Amat(y_vec, mdl_alt, del = del, Bartlett = TRUE)
    } else if (errorType == "Student-t") {
      wa_null <- .parse_Amat("Weighted", p_null)
      wa_alt  <- .parse_Amat("Weighted", p_alt)
      M_null <- LRT_moment_t(y_mat, mdl_null_est, wa_null$Amat, wa_null$WAmat, del, TRUE)
      M_alt  <- LRT_moment_t(y_mat, mdl_alt, wa_alt$Amat, wa_alt$WAmat, del, TRUE)
    } else {  # GED
      wa_null <- .parse_Amat("Weighted", p_null)
      wa_alt  <- .parse_Amat("Weighted", p_alt)
      M_null <- LRT_moment_ged(y_mat, mdl_null_est, wa_null$Amat, wa_null$WAmat, del, TRUE)
      M_alt  <- LRT_moment_ged(y_mat, mdl_alt, wa_alt$Amat, wa_alt$WAmat, del, TRUE)
    }
    s0_tmp <- Tsize * (M_null - M_alt)
  } else {
    phi_extra <- mdl_alt$phi[(p_null + 1):p_alt]
    s0_tmp <- Tsize * sum(phi_extra^2)
  }

  if (is.na(s0_tmp)) stop("Test statistic is NA.")
  if (s0_tmp < 0) warning("Test statistic is negative (", round(s0_tmp, 4),
                          "); capped at 1e-10. May indicate strong evidence for null.")
  s0 <- max(s0_tmp, 1e-10)

  # Pre-draw innovations for fixed-error MMC (Dufour 2006)
  n_total <- burnin + Tsize
  N_draw <- ceiling(N * 1.5) + 10L
  if (errorType == "Gaussian") {
    innov_ar <- list(
      eta = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw),
      eps = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw)
    )
  } else if (errorType == "Student-t") {
    innov_ar <- list(
      eta = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw),
      eps = matrix(rt(n_total * N_draw, df = nu_fixed),
                   nrow = n_total, ncol = N_draw)
    )
  } else {  # GED
    innov_ar <- list(
      eta = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw),
      eps = matrix(rged_cpp(n_total * N_draw, 0, 1, nu_fixed),
                   nrow = n_total, ncol = N_draw)
    )
  }

  out <- .run_mmc_optimizer(method, theta_0, .mmc_pval_ar, lower, upper,
                            threshold, maxit,
                            y = y_vec, p_null = p_null, p_alt = p_alt,
                            j = J, N = N, s0 = s0, ini = burnin,
                            del = del, wDecay = wDecay, Bartlett = wt$use_hac,
                            sigvMethod = sigvMethod,
                            errorType = errorType,
                            nu_fixed = nu_fixed,
                            logNu = logNu,
                            winsorize_eps = winsorize_eps,
                            innovations = innov_ar)
  out$value <- -out$value
  out$s0 <- s0
  out$errorType <- errorType
  out$call <- cl
  return(out)
}


# =========================================================================== #
# Leverage Tests
# =========================================================================== #

#' LMC Test for Leverage in SV(p) Models
#'
#' Performs a Local Monte Carlo (LMC) test of the null hypothesis
#' \eqn{H_0: \rho = \rho_0} (typically \eqn{\rho_0 = 0}, i.e., no leverage)
#' using a GMM likelihood-ratio type statistic.
#'
#' @param y Numeric vector. Observed returns.
#' @param p Integer. Order of the volatility process. Default 1.
#' @param J Integer. Winsorizing parameter. Default 10.
#' @param N Integer. Number of Monte Carlo replications. Default 99.
#' @param rho_null Numeric. Value of \eqn{\rho} under the null. Default 0.
#' @param burnin Integer. Burn-in for simulation. Default 500.
#' @param rho_type Character. Correlation type. Default \code{"pearson"}.
#' @param del Numeric. Small constant for log transformation. Default \code{1e-10}.
#' @param trunc_lev Logical. Truncate leverage correlation estimate to
#'   \code{[-0.999, 0.999]}. Default \code{TRUE}.
#' @param wDecay Logical. Use decaying weights. Default \code{FALSE}.
#' @param Bartlett Logical. If \code{TRUE}, use Bartlett kernel HAC weighting
#'   matrix. If \code{FALSE}, use identity matrix. Default \code{FALSE}.
#' @param Amat Weighting matrix specification. \code{NULL} (default) for identity
#'   weighting, \code{"Weighted"} for data-driven HAC, or a numeric matrix of
#'   dimension \code{(p+3)x(p+3)} (Gaussian) or \code{(p+4)x(p+4)} (heavy-tail).
#'   Takes precedence over \code{Bartlett}.
#' @param errorType Character. Error distribution: \code{"Gaussian"} (default),
#'   \code{"Student-t"}, or \code{"GED"}.
#' @param logNu Logical. Use log-space for nu estimation (Student-t only).
#'   Default \code{FALSE}.
#' @param sigvMethod Method for sigma_v estimation: \code{"factored"} (default),
#'   \code{"direct"}, or \code{"hybrid"}.
#' @param winsorize_eps Number of extreme autocovariance lags to winsorize
#'   (0 = none). Default 0.
#'
#' @return An object of class \code{"svp_test"}, a list containing:
#' \describe{
#'   \item{s0}{Test statistic from observed data.}
#'   \item{sN}{Simulated null distribution (vector of length N).}
#'   \item{pval}{Monte Carlo p-value.}
#'   \item{test_type}{Character string identifying the test.}
#'   \item{null_param}{Name of the parameter tested.}
#'   \item{null_value}{Value under the null hypothesis.}
#'   \item{call}{The matched call.}
#' }
#'
#' @examples
#' \donttest{
#' y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2, leverage = TRUE, rho = -0.3)$y
#' test <- lmc_lev(y, p = 1, J = 10, N = 99)
#' print(test)
#' }
#'
#' @export
lmc_lev <- function(y, p = 1, J = 10, N = 99, rho_null = 0,
                    burnin = 500, rho_type = "pearson", del = 1e-10,
                    trunc_lev = TRUE, wDecay = FALSE,
                    Bartlett = FALSE, Amat = NULL,
                    errorType = "Gaussian",
                    logNu = FALSE, sigvMethod = "factored",
                    winsorize_eps = 0) {
  cl <- match.call()
  y_vec <- as.numeric(y)
  .validate_test_common(y_vec, J, N, burnin, del)
  if (!is.numeric(p) || length(p) != 1L || p < 1L)
    stop("'p' must be a positive integer (>= 1).")
  y_mat <- as.matrix(y_vec)
  Tsize <- length(y_vec)
  errorType <- match.arg(errorType, c("Gaussian", "Student-t", "GED"))
  sigvMethod <- match.arg(sigvMethod, c("hybrid", "direct", "factored"))
  wt <- .resolve_weighting(Amat, Bartlett)
  # Estimate model under alternative
  mdl_alt <- svp(y_vec, p, J, leverage = TRUE, errorType = errorType,
                 rho_type = rho_type, del = del, trunc_lev = trunc_lev,
                 wDecay = wDecay, logNu = logNu, sigvMethod = sigvMethod,
                 winsorize_eps = winsorize_eps)
  mdl_null <- mdl_alt
  mdl_null$rho <- rho_null
  if (errorType == "Gaussian") {
    # --- Gaussian leverage test (p+3 moments) ---
    n_mom <- p + 3
    if (wt$use_hac) {
      s0_tmp <- Tsize * (LRT_moment_lev_svp_Amat(y_mat, mdl_null, rho_type, del, TRUE) -
                           LRT_moment_lev_svp_Amat(y_mat, mdl_alt, rho_type, del, TRUE))
    } else {
      Amat_use <- if (is.matrix(wt$Amat)) wt$Amat else diag(n_mom)
      s0_tmp <- Tsize * (LRT_moment_lev_svp(y_mat, mdl_null, Amat_use, rho_type, del) -
                           LRT_moment_lev_svp(y_mat, mdl_alt, Amat_use, rho_type, del))
    }
    if (is.na(s0_tmp)) stop("Test statistic is NA.")
    if (s0_tmp < 0) warning("Test statistic is negative (", round(s0_tmp, 4), "); capped at 1e-10.")
    s0 <- max(s0_tmp, 1e-10)
    betasim_null <- c(mdl_alt$phi, mdl_alt$sigy, mdl_alt$sigv, rho_null)
    if (wt$use_hac) {
      sN <- .simnull_Amat(betasim_null, rho_null, p, J, Tsize, N, burnin,
                          rho_type, del, TRUE, wDecay = wDecay,
                          trunc_lev = trunc_lev,
                          sigvMethod = sigvMethod)
    } else {
      Amat_use <- if (is.matrix(wt$Amat)) wt$Amat else diag(n_mom)
      sN <- .simnull(betasim_null, rho_null, p, J, Tsize, N, burnin,
                     Amat_use, rho_type, del, wDecay = wDecay,
                     trunc_lev = trunc_lev,
                     sigvMethod = sigvMethod)
    }
  } else if (errorType == "Student-t") {
    # --- Student-t leverage test (p+4 moments) ---
    n_mom <- p + 4
    if (wt$use_hac && !is.null(mdl_alt$v) && mdl_alt$v <= 4) {
      warning("HAC weighting may be unreliable for Student-t with nu <= 4. ",
              "LMC/MMC p-values remain valid regardless.")
    }
    if (wt$use_hac) {
      s0_tmp <- Tsize * (LRT_moment_lev_t_Amat(y_mat, mdl_null, rho_type, del, TRUE) -
                           LRT_moment_lev_t_Amat(y_mat, mdl_alt, rho_type, del, TRUE))
    } else {
      Amat_use <- if (is.matrix(wt$Amat)) wt$Amat else diag(n_mom)
      s0_tmp <- Tsize * (LRT_moment_lev_t(y_mat, mdl_null, Amat_use, rho_type, del) -
                           LRT_moment_lev_t(y_mat, mdl_alt, Amat_use, rho_type, del))
    }
    if (is.na(s0_tmp)) stop("Test statistic is NA.")
    if (s0_tmp < 0) warning("Test statistic is negative (", round(s0_tmp, 4), "); capped at 1e-10.")
    s0 <- max(s0_tmp, 1e-10)
    betasim_null <- c(mdl_alt$phi, mdl_alt$sigy, mdl_alt$sigv,
                      mdl_alt$v, rho_null)
    if (wt$use_hac) {
      sN <- .simnull_lev_t_Amat(betasim_null, rho_null, p, J, Tsize, N, burnin,
                                 rho_type, del, TRUE, wDecay = wDecay,
                                 trunc_lev = trunc_lev, logNu = logNu,
                                 sigvMethod = sigvMethod,
                                 winsorize_eps = winsorize_eps)
    } else {
      Amat_use <- if (is.matrix(wt$Amat)) wt$Amat else diag(n_mom)
      sN <- .simnull_lev_t(betasim_null, rho_null, p, J, Tsize, N, burnin,
                            Amat_use, rho_type, del, wDecay = wDecay,
                            trunc_lev = trunc_lev, logNu = logNu,
                            sigvMethod = sigvMethod,
                            winsorize_eps = winsorize_eps)
    }
  } else if (errorType == "GED") {
    # --- GED leverage test (p+4 moments) ---
    n_mom <- p + 4
    if (wt$use_hac) {
      s0_tmp <- Tsize * (LRT_moment_lev_ged_Amat(y_mat, mdl_null, rho_type, del, TRUE) -
                           LRT_moment_lev_ged_Amat(y_mat, mdl_alt, rho_type, del, TRUE))
    } else {
      Amat_use <- if (is.matrix(wt$Amat)) wt$Amat else diag(n_mom)
      s0_tmp <- Tsize * (LRT_moment_lev_ged(y_mat, mdl_null, Amat_use, rho_type, del) -
                           LRT_moment_lev_ged(y_mat, mdl_alt, Amat_use, rho_type, del))
    }
    if (is.na(s0_tmp)) stop("Test statistic is NA.")
    if (s0_tmp < 0) warning("Test statistic is negative (", round(s0_tmp, 4), "); capped at 1e-10.")
    s0 <- max(s0_tmp, 1e-10)
    betasim_null <- c(mdl_alt$phi, mdl_alt$sigy, mdl_alt$sigv,
                      mdl_alt$v, rho_null)
    if (wt$use_hac) {
      sN <- .simnull_lev_ged_Amat(betasim_null, rho_null, p, J, Tsize, N, burnin,
                                   rho_type, del, TRUE, wDecay = wDecay,
                                   trunc_lev = trunc_lev,
                                   sigvMethod = sigvMethod,
                                   winsorize_eps = winsorize_eps)
    } else {
      Amat_use <- if (is.matrix(wt$Amat)) wt$Amat else diag(n_mom)
      sN <- .simnull_lev_ged(betasim_null, rho_null, p, J, Tsize, N, burnin,
                              Amat_use, rho_type, del, wDecay = wDecay,
                              trunc_lev = trunc_lev,
                              sigvMethod = sigvMethod,
                              winsorize_eps = winsorize_eps)
    }
  }
  # Compute p-value
  pval <- (N + 1 - sum(s0 >= sN)) / (N + 1)
  test_label <- paste0("LMC Leverage (", errorType, ")")
  out <- list(s0 = s0, sN = as.numeric(sN), pval = pval,
              test_type = test_label,
              null_param = "rho", null_value = rho_null,
              call = cl)
  class(out) <- "svp_test"
  return(out)
}


#' MMC Test for Leverage in SV(p) Models
#'
#' Performs a Maximized Monte Carlo (MMC) test of the null hypothesis
#' \eqn{H_0: \rho = \rho_0} by maximizing the MC p-value over nuisance
#' parameters (phi, sigma_y, sigma_v).
#'
#' @inheritParams lmc_lev
#' @param eps Numeric vector. Half-width of the search region around the
#'   estimated nuisance parameters. For Gaussian: length \code{p+2}
#'   (phi, sigma_y, sigma_v). For Student-t/GED: length \code{p+2}
#'   (phi, sigma_y, sigma_v; nu bounds set proportionally at +/-30%) or
#'   length \code{p+3} (phi, sigma_y, sigma_v, nu). Default \code{NULL}
#'   which uses \code{rep(0.3, p+2)} with proportional nu bounds.
#' @param threshold Numeric. Target p-value (optimization stops if reached).
#'   Default 1.
#' @param method Character. Optimization method: \code{"pso"} (particle swarm),
#'   \code{"GenSA"} (generalized simulated annealing).
#'   Default \code{"pso"}.
#' @param maxit Integer or list. Maximum iterations/evaluations for the
#'   optimizer. Default depends on method.
#'
#' @return A list with the optimization output including:
#' \describe{
#'   \item{value}{Maximized p-value.}
#'   \item{par}{Nuisance parameter values at the maximum.}
#' }
#' Additional fields depend on the optimization method used.
#'
#' @examples
#' \donttest{
#' y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2, leverage = TRUE, rho = -0.3)$y
#' mmc <- mmc_lev(y, p = 1, J = 10, N = 19, method = "pso", maxit = 10)
#' mmc$value
#' }
#'
#' @export
mmc_lev <- function(y, p = 1, J = 10, N = 99, rho_null = 0,
                    burnin = 500, eps = NULL, threshold = 1,
                    method = "pso", maxit = NULL,
                    rho_type = "pearson", del = 1e-10,
                    trunc_lev = TRUE, wDecay = FALSE,
                    Bartlett = FALSE, Amat = NULL,
                    errorType = "Gaussian",
                    logNu = FALSE, sigvMethod = "factored",
                    winsorize_eps = 0) {
  cl <- match.call()
  y_vec <- as.numeric(y)
  .validate_test_common(y_vec, J, N, burnin, del)
  if (!is.numeric(p) || length(p) != 1L || p < 1L)
    stop("'p' must be a positive integer (>= 1).")
  y_mat <- as.matrix(y_vec)
  Tsize <- length(y_vec)
  errorType <- match.arg(errorType, c("Gaussian", "Student-t", "GED"))
  sigvMethod <- match.arg(sigvMethod, c("hybrid", "direct", "factored"))
  wt <- .resolve_weighting(Amat, Bartlett)
  # Estimate model under alternative
  mdl_alt <- svp(y_vec, p, J, leverage = TRUE, errorType = errorType,
                 rho_type = rho_type, del = del, trunc_lev = trunc_lev,
                 wDecay = wDecay, logNu = logNu, sigvMethod = sigvMethod,
                 winsorize_eps = winsorize_eps)
  # Determine nuisance parameters and bounds
  if (errorType == "Gaussian") {
    theta_0 <- c(mdl_alt$phi, mdl_alt$sigy, mdl_alt$sigv)
    n_nuisance <- p + 2
    n_mom <- p + 3
    if (is.null(eps)) {
      eps <- rep(0.3, n_nuisance)
      eps[p + 1] <- 0  # sigma_y: test stat is scale-invariant, fixing avoids spurious optimization
    }
    if (length(eps) != n_nuisance)
      stop("eps must have length ", n_nuisance,
           " (p+2: one entry per nuisance parameter phi_1,...,phi_p, sigma_y, sigma_v).")
    lower <- c(pmax(theta_0[1:p] - eps[1:p], rep(-0.999, p)),
               max(theta_0[p + 1] - eps[p + 1], 0.01),
               max(theta_0[p + 2] - eps[p + 2], 0.01))
    upper <- c(pmin(theta_0[1:p] + eps[1:p], rep(0.999, p)),
               theta_0[p + 1] + eps[p + 1],
               theta_0[p + 2] + eps[p + 2])
  } else {
    # Heavy-tail: nuisance includes nu
    theta_0 <- c(mdl_alt$phi, mdl_alt$sigy, mdl_alt$sigv, mdl_alt$v)
    n_nuisance <- p + 3
    n_mom <- p + 4
    nu_hat <- mdl_alt$v
    if (is.null(eps)) {
      eps <- rep(0.3, p + 2)  # default eps for phi, sigy, sigv only; nu gets proportional bounds
      eps[p + 1] <- 0  # sigma_y: test stat is scale-invariant, fixing avoids spurious optimization
    }
    if (length(eps) == n_nuisance) {
      # User provided eps for all nuisance params including nu
      eps_nu <- eps[n_nuisance]
      eps <- eps[1:(p + 2)]
      if (errorType == "Student-t") {
        nu_lo <- max(2.01, nu_hat - eps_nu)
        nu_hi <- min(500, nu_hat + eps_nu)
      } else {
        nu_lo <- max(0.1, nu_hat - eps_nu)
        nu_hi <- min(20, nu_hat + eps_nu)
      }
    } else if (length(eps) == p + 2) {
      # eps for phi, sigy, sigv only; nu bounds set proportionally
      if (errorType == "Student-t") {
        nu_lo <- max(2.01, nu_hat * 0.7)
        nu_hi <- min(500, nu_hat * 1.3)
      } else {
        nu_lo <- max(0.1, nu_hat * 0.7)
        nu_hi <- min(20, nu_hat * 1.3)
      }
    } else {
      stop("eps must have length ", p + 2, " (phi, sigy, sigv) or ", n_nuisance,
           " (phi, sigy, sigv, nu) for heavy-tail leverage tests.")
    }
    lower <- c(pmax(theta_0[1:p] - eps[1:p], rep(-0.999, p)),
               max(theta_0[p + 1] - eps[p + 1], 0.01),
               max(theta_0[p + 2] - eps[p + 2], 0.01),
               nu_lo)
    upper <- c(pmin(theta_0[1:p] + eps[1:p], rep(0.999, p)),
               theta_0[p + 1] + eps[p + 1],
               theta_0[p + 2] + eps[p + 2],
               nu_hi)
  }
  # Compute test statistic from observed data
  mdl_null <- mdl_alt
  mdl_null$rho <- rho_null
  if (errorType == "Gaussian") {
    if (wt$use_hac) {
      s0_tmp <- Tsize * (LRT_moment_lev_svp_Amat(y_mat, mdl_null, rho_type, del, TRUE) -
                           LRT_moment_lev_svp_Amat(y_mat, mdl_alt, rho_type, del, TRUE))
    } else {
      Amat_use <- if (is.matrix(wt$Amat)) wt$Amat else diag(n_mom)
      s0_tmp <- Tsize * (LRT_moment_lev_svp(y_mat, mdl_null, Amat_use, rho_type, del) -
                           LRT_moment_lev_svp(y_mat, mdl_alt, Amat_use, rho_type, del))
    }
    pval_fn <- if (wt$use_hac) .mmc_pval_lev_Amat else .mmc_pval_lev
  } else if (errorType == "Student-t") {
    if (wt$use_hac && !is.null(mdl_alt$v) && mdl_alt$v <= 4) {
      warning("HAC weighting may be unreliable for Student-t with nu <= 4.")
    }
    if (wt$use_hac) {
      s0_tmp <- Tsize * (LRT_moment_lev_t_Amat(y_mat, mdl_null, rho_type, del, TRUE) -
                           LRT_moment_lev_t_Amat(y_mat, mdl_alt, rho_type, del, TRUE))
    } else {
      Amat_use <- if (is.matrix(wt$Amat)) wt$Amat else diag(n_mom)
      s0_tmp <- Tsize * (LRT_moment_lev_t(y_mat, mdl_null, Amat_use, rho_type, del) -
                           LRT_moment_lev_t(y_mat, mdl_alt, Amat_use, rho_type, del))
    }
    pval_fn <- .mmc_pval_lev_t
  } else if (errorType == "GED") {
    if (wt$use_hac) {
      s0_tmp <- Tsize * (LRT_moment_lev_ged_Amat(y_mat, mdl_null, rho_type, del, TRUE) -
                           LRT_moment_lev_ged_Amat(y_mat, mdl_alt, rho_type, del, TRUE))
    } else {
      Amat_use <- if (is.matrix(wt$Amat)) wt$Amat else diag(n_mom)
      s0_tmp <- Tsize * (LRT_moment_lev_ged(y_mat, mdl_null, Amat_use, rho_type, del) -
                           LRT_moment_lev_ged(y_mat, mdl_alt, Amat_use, rho_type, del))
    }
    pval_fn <- .mmc_pval_lev_ged
  }
  if (is.na(s0_tmp)) stop("Test statistic is NA.")
  if (s0_tmp < 0) warning("Test statistic is negative (", round(s0_tmp, 4), "); capped at 1e-10.")
  s0_tmp <- max(s0_tmp, 1e-10)
  # Pre-draw innovations for fixed-error MMC (Dufour 2006)
  n_total <- burnin + Tsize
  N_draw <- ceiling(N * 1.5) + 10L
  if (errorType == "Student-t") {
    # Student-t leverage: PIT for chi2 (nu varies in nuisance)
    innov_lev <- list(
      zeta   = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw),
      aux    = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw),
      U_chi2 = matrix(runif(n_total * N_draw), nrow = n_total, ncol = N_draw)
    )
  } else {
    # Gaussian or GED leverage: just Gaussian primitives
    innov_lev <- list(
      zeta = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw),
      aux  = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw)
    )
  }
  # Optimize — pass Bartlett and Amat appropriately per error type
  opt_args <- list(method = method, theta_0 = theta_0, fn = pval_fn,
                   lower = lower, upper = upper,
                   threshold = threshold, maxit = maxit,
                   y = y_mat, j = J, N = N, s0 = s0_tmp,
                   mdl_alt = mdl_alt,
                   rho_null = rho_null, ini = burnin,
                   rho_type = rho_type, del = del,
                   wDecay = wDecay, trunc_lev = trunc_lev,
                   Bartlett = wt$use_hac,
                   innovations = innov_lev)
  opt_args$Amat <- if (is.matrix(wt$Amat)) wt$Amat else diag(n_mom)
  if (errorType == "Student-t") {
    opt_args$logNu <- logNu
  }
  opt_args$sigvMethod <- sigvMethod
  if (errorType != "Gaussian") {
    opt_args$winsorize_eps <- winsorize_eps
  }
  out <- do.call(.run_mmc_optimizer, opt_args)
  out$value <- -out$value
  out$s0 <- s0_tmp
  out$call <- cl
  return(out)
}


# =========================================================================== #
# Heavy-Tail Tests (Student-t and GED)
# =========================================================================== #

#' LMC Test for Student-t Tail Parameter
#'
#' Performs a Local Monte Carlo (LMC) test of the null hypothesis
#' \eqn{H_0: \nu = \nu_0} for the degrees of freedom parameter in an
#' SV(p) model with Student-t errors. Testing \eqn{\nu_0 = \infty}
#' (or a large value) corresponds to testing for normality.
#'
#' @param y Numeric vector. Observed returns.
#' @param p Integer. AR order of the volatility process. Default 1.
#' @param J Integer. Winsorizing parameter. Default 10.
#' @param N Integer. Number of Monte Carlo replications. Default 99.
#' @param nu_null Numeric. Value of \eqn{\nu} under the null hypothesis.
#' @param burnin Integer. Burn-in for simulation. Default 500.
#' @param del Numeric. Small constant for log transformation. Default \code{1e-10}.
#' @param wDecay Logical. Use decaying weights. Default \code{FALSE}.
#' @param Bartlett Logical. Use Bartlett kernel HAC for weighting matrix.
#'   Default \code{FALSE}.
#' @param Amat Weighting matrix specification. \code{NULL} (default) for identity
#'   weighting, \code{"Weighted"} for data-driven HAC, or a \code{(p+3)x(p+3)}
#'   matrix. Takes precedence over \code{Bartlett}.
#' @param logNu Logical. Use log-space for nu estimation. Default \code{TRUE}.
#' @param direction Character. Test direction: \code{"two-sided"} (default),
#'   \code{"less"} (H1: nu < nu_null), or \code{"greater"} (H1: nu > nu_null).
#'   Uses signed root of the LR statistic for one-sided tests.
#' @param sigvMethod Character. Method for \eqn{\sigma_v} estimation:
#'   \code{"factored"} (default), \code{"hybrid"}, or \code{"direct"}.
#' @param winsorize_eps Numeric. Winsorization threshold for moment conditions.
#'   Default 0 (no winsorization).
#'
#' @return An object of class \code{"svp_test"}.
#'
#' @examples
#' \donttest{
#' y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2, errorType = "Student-t", nu = 5)$y
#' test <- lmc_t(y, p = 1, J = 10, N = 49, nu_null = 5)
#' print(test)
#' }
#'
#' @export
lmc_t <- function(y, p = 1, J = 10, N = 99, nu_null, burnin = 500,
                  del = 1e-10, wDecay = FALSE, Bartlett = FALSE,
                  Amat = NULL, logNu = TRUE,
                  direction = c("two-sided", "less", "greater"),
                  sigvMethod = "factored", winsorize_eps = 0) {
  cl <- match.call()
  direction <- match.arg(direction)
  y_vec <- as.numeric(y)
  .validate_test_common(y_vec, J, N, burnin, del)
  if (!is.numeric(p) || length(p) != 1L || p < 1L)
    stop("'p' must be a positive integer (>= 1).")
  if (!is.numeric(nu_null) || length(nu_null) != 1L || nu_null <= 2)
    stop("'nu_null' must be > 2 for Student-t distribution.")
  sigvMethod <- match.arg(sigvMethod, c("hybrid", "direct", "factored"))
  y_mat <- as.matrix(y_vec)
  Tsize <- length(y_vec)
  # Resolve weighting: Amat takes precedence over Bartlett
  wt <- .resolve_weighting(Amat, Bartlett)
  if (wt$use_hac) {
    wa <- .parse_Amat("Weighted", p)
    Bartlett <- TRUE
  } else if (is.matrix(wt$Amat)) {
    wa <- list(Amat = wt$Amat, WAmat = FALSE)
    Bartlett <- FALSE
  } else {
    wa <- .parse_Amat(NULL, p)
    Bartlett <- FALSE
  }
  Amat_mat <- wa$Amat
  WAmat <- wa$WAmat
  # Estimate model under alternative
  mdl_alt <- svp(y_vec, p = p, J = J, errorType = "Student-t", del = del,
                 logNu = logNu, wDecay = wDecay, sigvMethod = sigvMethod,
                 winsorize_eps = winsorize_eps)
  mdl_null <- mdl_alt
  mdl_null$v <- nu_null
  # Compute test statistic
  s0_tmp <- Tsize * (LRT_moment_t(y_mat, mdl_null, Amat_mat, WAmat, del, Bartlett) -
                       LRT_moment_t(y_mat, mdl_alt, Amat_mat, WAmat, del, Bartlett))
  if (is.na(s0_tmp)) stop("Test statistic is NA.")
  if (s0_tmp < 0) warning("Test statistic is negative (", round(s0_tmp, 4), "); capped at 1e-10.")
  s0 <- max(s0_tmp, 1e-10)
  # Simulate null distribution
  betasim_null <- c(mdl_alt$phi, mdl_alt$sigy, mdl_alt$sigv, nu_null)

  if (direction == "two-sided") {
    sN <- .simnull_t(betasim_null, nu_null, J, Tsize, N, burnin,
                     Amat_mat, del, WAmat, Bartlett, logNu,
                     wDecay = wDecay, direction = "two-sided",
                     sigvMethod = sigvMethod, winsorize_eps = winsorize_eps)
    pval <- (N + 1 - sum(s0 >= sN)) / (N + 1)
    S_T <- NULL
  } else {
    S_T <- .signed_root(s0, mdl_alt$v, nu_null)
    sN <- .simnull_t(betasim_null, nu_null, J, Tsize, N, burnin,
                     Amat_mat, del, WAmat, Bartlett, logNu,
                     wDecay = wDecay, direction = direction,
                     sigvMethod = sigvMethod, winsorize_eps = winsorize_eps)
    pval <- .pvalue_directional(S_T, sN, direction)
  }

  out <- list(s0 = s0, sN = as.numeric(sN), pval = pval,
              test_type = "LMC Student-t",
              null_param = "nu", null_value = nu_null,
              direction = direction, S_T = S_T,
              call = cl)
  class(out) <- "svp_test"
  return(out)
}


#' LMC Test for GED Shape Parameter
#'
#' Performs a Local Monte Carlo (LMC) test of the null hypothesis
#' \eqn{H_0: \nu = \nu_0} for the shape parameter in an SV(p) model
#' with GED errors. Testing \eqn{\nu_0 = 2} corresponds to testing normality.
#'
#' @inheritParams lmc_t
#'
#' @return An object of class \code{"svp_test"}.
#'
#' @examples
#' \donttest{
#' y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2, errorType = "GED", nu = 1.5)$y
#' test <- lmc_ged(y, p = 1, J = 10, N = 49, nu_null = 2)
#' print(test)
#' }
#'
#' @export
lmc_ged <- function(y, p = 1, J = 10, N = 99, nu_null, burnin = 500,
                    del = 1e-10, wDecay = FALSE, Bartlett = FALSE,
                    Amat = NULL,
                    direction = c("two-sided", "less", "greater"),
                    sigvMethod = "factored", winsorize_eps = 0) {
  cl <- match.call()
  direction <- match.arg(direction)
  y_vec <- as.numeric(y)
  .validate_test_common(y_vec, J, N, burnin, del)
  if (!is.numeric(p) || length(p) != 1L || p < 1L)
    stop("'p' must be a positive integer (>= 1).")
  if (!is.numeric(nu_null) || length(nu_null) != 1L || nu_null <= 0)
    stop("'nu_null' must be > 0 for GED distribution.")
  sigvMethod <- match.arg(sigvMethod, c("hybrid", "direct", "factored"))
  y_mat <- as.matrix(y_vec)
  Tsize <- length(y_vec)
  # Resolve weighting: Amat takes precedence over Bartlett
  wt <- .resolve_weighting(Amat, Bartlett)
  if (wt$use_hac) {
    wa <- .parse_Amat("Weighted", p)
    Bartlett <- TRUE
  } else if (is.matrix(wt$Amat)) {
    wa <- list(Amat = wt$Amat, WAmat = FALSE)
    Bartlett <- FALSE
  } else {
    wa <- .parse_Amat(NULL, p)
    Bartlett <- FALSE
  }
  Amat_mat <- wa$Amat
  WAmat <- wa$WAmat
  mdl_alt <- svp(y_vec, p = p, J = J, errorType = "GED", del = del,
                 wDecay = wDecay, sigvMethod = sigvMethod,
                 winsorize_eps = winsorize_eps)
  mdl_null <- mdl_alt
  mdl_null$v <- nu_null
  s0_tmp <- Tsize * (LRT_moment_ged(y_mat, mdl_null, Amat_mat, WAmat, del, Bartlett) -
                       LRT_moment_ged(y_mat, mdl_alt, Amat_mat, WAmat, del, Bartlett))
  if (is.na(s0_tmp)) stop("Test statistic is NA.")
  if (s0_tmp < 0) warning("Test statistic is negative (", round(s0_tmp, 4), "); capped at 1e-10.")
  s0 <- max(s0_tmp, 1e-10)
  betasim_null <- c(mdl_alt$phi, mdl_alt$sigy, mdl_alt$sigv, nu_null)

  if (direction == "two-sided") {
    sN <- .simnull_ged(betasim_null, nu_null, J, Tsize, N, burnin,
                       Amat_mat, del, WAmat, Bartlett, wDecay = wDecay,
                       direction = "two-sided",
                       sigvMethod = sigvMethod, winsorize_eps = winsorize_eps)
    pval <- (N + 1 - sum(s0 >= sN)) / (N + 1)
    S_T <- NULL
  } else {
    S_T <- .signed_root(s0, mdl_alt$v, nu_null)
    sN <- .simnull_ged(betasim_null, nu_null, J, Tsize, N, burnin,
                       Amat_mat, del, WAmat, Bartlett, wDecay = wDecay,
                       direction = direction,
                       sigvMethod = sigvMethod, winsorize_eps = winsorize_eps)
    pval <- .pvalue_directional(S_T, sN, direction)
  }

  out <- list(s0 = s0, sN = as.numeric(sN), pval = pval,
              test_type = "LMC GED",
              null_param = "nu", null_value = nu_null,
              direction = direction, S_T = S_T,
              call = cl)
  class(out) <- "svp_test"
  return(out)
}


#' MMC Test for Student-t Tail Parameter
#'
#' Performs a Maximized Monte Carlo (MMC) test of \eqn{H_0: \nu = \nu_0}
#' by maximizing the MC p-value over nuisance parameters (phi, sigma_y, sigma_v).
#'
#' @inheritParams lmc_t
#' @param eps Numeric vector. Half-width of search region around estimated nuisance
#'   parameters. Must have length \code{p+2} (one entry per nuisance parameter:
#'   \eqn{\phi_1,\ldots,\phi_p, \sigma_y, \sigma_v}). Default \code{rep(0.3, p+2)}.
#' @param threshold Numeric. Target p-value. Default 1.
#' @param method Character. Optimization method: \code{"pso"} or \code{"GenSA"}.
#'   Default \code{"pso"}.
#' @param maxit Integer. Maximum iterations/evaluations. Default depends on method.
#'
#' @return A list with optimization output including \code{value} (maximized p-value)
#'   and \code{par} (nuisance parameters at the maximum).
#'
#' @examples
#' \donttest{
#' y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2, errorType = "Student-t", nu = 5)$y
#' mmc <- mmc_t(y, p = 1, J = 10, N = 19, nu_null = 5, method = "pso", maxit = 10)
#' mmc$value
#' }
#'
#' @export
mmc_t <- function(y, p = 1, J = 10, N = 99, nu_null, burnin = 500,
                  eps = NULL, threshold = 1, method = "pso", maxit = NULL,
                  del = 1e-10, wDecay = FALSE, Bartlett = FALSE,
                  Amat = NULL, logNu = TRUE,
                  direction = c("two-sided", "less", "greater"),
                  sigvMethod = "factored", winsorize_eps = 0) {
  direction <- match.arg(direction)
  cl <- match.call()
  y_vec <- as.numeric(y)
  .validate_test_common(y_vec, J, N, burnin, del)
  if (!is.numeric(p) || length(p) != 1L || p < 1L)
    stop("'p' must be a positive integer (>= 1).")
  if (!is.numeric(nu_null) || length(nu_null) != 1L || nu_null <= 2)
    stop("'nu_null' must be > 2 for Student-t distribution.")
  sigvMethod <- match.arg(sigvMethod, c("hybrid", "direct", "factored"))
  y_mat <- as.matrix(y_vec)
  Tsize <- length(y_vec)
  # Resolve weighting: Amat takes precedence over Bartlett
  wt <- .resolve_weighting(Amat, Bartlett)
  mdl_alt <- svp(y_vec, p = p, J = J, errorType = "Student-t", del = del,
                 logNu = logNu, wDecay = wDecay, sigvMethod = sigvMethod,
                 winsorize_eps = winsorize_eps)
  p <- length(mdl_alt$phi)
  if (wt$use_hac) {
    wa <- .parse_Amat("Weighted", p)
    Bartlett <- TRUE
  } else if (is.matrix(wt$Amat)) {
    wa <- list(Amat = wt$Amat, WAmat = FALSE)
    Bartlett <- FALSE
  } else {
    wa <- .parse_Amat(NULL, p)
    Bartlett <- FALSE
  }
  Amat_mat <- wa$Amat
  WAmat <- wa$WAmat
  theta_0 <- c(mdl_alt$phi, mdl_alt$sigy, mdl_alt$sigv)
  if (is.null(eps)) {
    eps <- rep(0.3, length(theta_0))
    eps[p + 1] <- 0  # sigma_y: test stat is scale-invariant, fixing avoids spurious optimization
  }
  if (length(eps) != length(theta_0))
    stop("eps must have length ", length(theta_0),
         " (p+2: one entry per nuisance parameter phi_1,...,phi_p, sigma_y, sigma_v).")
  lower <- c(pmax(theta_0[1:p] - eps[1:p], rep(-0.999, p)),
             max(theta_0[p + 1] - eps[p + 1], 0.01),
             max(theta_0[p + 2] - eps[p + 2], 0.01))
  upper <- c(pmin(theta_0[1:p] + eps[1:p], rep(0.999, p)),
             theta_0[p + 1] + eps[p + 1],
             theta_0[p + 2] + eps[p + 2])
  mdl_null <- mdl_alt
  mdl_null$v <- nu_null
  s0_tmp <- Tsize * (LRT_moment_t(y_mat, mdl_null, Amat_mat, WAmat, del, Bartlett) -
                       LRT_moment_t(y_mat, mdl_alt, Amat_mat, WAmat, del, Bartlett))
  if (!is.finite(s0_tmp)) stop("Test statistic is not finite.")
  if (s0_tmp < 0) warning("Test statistic is negative (", round(s0_tmp, 4), "); capped at 1e-10.")
  s0_tmp <- max(s0_tmp, 1e-10)
  # Pre-draw innovations ONCE for fixed-error MMC (Dufour 2006, eq 4.22)
  n_total <- burnin + Tsize
  N_draw <- ceiling(N * 1.5) + 10L
  innov_t <- list(
    eta = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw),
    eps = matrix(NA_real_, nrow = n_total, ncol = N_draw)
  )
  for (b in seq_len(N_draw)) {
    innov_t$eps[, b] <- rt(n_total, df = nu_null)
  }
  out <- .run_mmc_optimizer(method, theta_0, .mmc_pval_t, lower, upper,
                            threshold, maxit,
                            y = y_mat, j = J, N = N, s0 = s0_tmp,
                            mdl_alt = mdl_alt,
                            nu_null = nu_null, ini = burnin, Amat = Amat_mat,
                            WAmat = WAmat,
                            del = del, Bartlett = Bartlett, logNu = logNu,
                            wDecay = wDecay, direction = direction,
                            sigvMethod = sigvMethod,
                            winsorize_eps = winsorize_eps,
                            innovations = innov_t)
  out$value <- -out$value
  out$s0 <- s0_tmp
  out$direction <- direction
  out$call <- cl
  return(out)
}


#' MMC Test for GED Shape Parameter
#'
#' Performs a Maximized Monte Carlo (MMC) test of \eqn{H_0: \nu = \nu_0}
#' for the GED shape parameter.
#'
#' @inheritParams lmc_ged
#' @param eps Numeric vector. Half-width of search region around estimated nuisance
#'   parameters. Must have length \code{p+2} (one entry per nuisance parameter:
#'   \eqn{\phi_1,\ldots,\phi_p, \sigma_y, \sigma_v}). Default \code{rep(0.3, p+2)}.
#' @param threshold Numeric. Target p-value. Default 1.
#' @param method Character. Optimization method: \code{"pso"} or \code{"GenSA"}.
#'   Default \code{"pso"}.
#' @param maxit Integer. Maximum iterations/evaluations. Default depends on method.
#'
#' @return A list with optimization output including \code{value} (maximized p-value)
#'   and \code{par} (nuisance parameters at the maximum).
#'
#' @examples
#' \donttest{
#' y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2, errorType = "GED", nu = 1.5)$y
#' mmc <- mmc_ged(y, p = 1, J = 10, N = 19, nu_null = 2, method = "pso", maxit = 10)
#' mmc$value
#' }
#'
#' @export
mmc_ged <- function(y, p = 1, J = 10, N = 99, nu_null, burnin = 500,
                    eps = NULL, threshold = 1, method = "pso", maxit = NULL,
                    del = 1e-10, wDecay = FALSE, Bartlett = FALSE,
                    Amat = NULL,
                    direction = c("two-sided", "less", "greater"),
                    sigvMethod = "factored", winsorize_eps = 0) {
  direction <- match.arg(direction)
  cl <- match.call()
  y_vec <- as.numeric(y)
  .validate_test_common(y_vec, J, N, burnin, del)
  if (!is.numeric(p) || length(p) != 1L || p < 1L)
    stop("'p' must be a positive integer (>= 1).")
  if (!is.numeric(nu_null) || length(nu_null) != 1L || nu_null <= 0)
    stop("'nu_null' must be > 0 for GED distribution.")
  sigvMethod <- match.arg(sigvMethod, c("hybrid", "direct", "factored"))
  y_mat <- as.matrix(y_vec)
  Tsize <- length(y_vec)
  # Resolve weighting: Amat takes precedence over Bartlett
  wt <- .resolve_weighting(Amat, Bartlett)
  mdl_alt <- svp(y_vec, p = p, J = J, errorType = "GED", del = del,
                 wDecay = wDecay, sigvMethod = sigvMethod,
                 winsorize_eps = winsorize_eps)
  p <- length(mdl_alt$phi)
  if (wt$use_hac) {
    wa <- .parse_Amat("Weighted", p)
    Bartlett <- TRUE
  } else if (is.matrix(wt$Amat)) {
    wa <- list(Amat = wt$Amat, WAmat = FALSE)
    Bartlett <- FALSE
  } else {
    wa <- .parse_Amat(NULL, p)
    Bartlett <- FALSE
  }
  Amat_mat <- wa$Amat
  WAmat <- wa$WAmat
  theta_0 <- c(mdl_alt$phi, mdl_alt$sigy, mdl_alt$sigv)
  if (is.null(eps)) {
    eps <- rep(0.3, length(theta_0))
    eps[p + 1] <- 0  # sigma_y: test stat is scale-invariant, fixing avoids spurious optimization
  }
  if (length(eps) != length(theta_0))
    stop("eps must have length ", length(theta_0),
         " (p+2: one entry per nuisance parameter phi_1,...,phi_p, sigma_y, sigma_v).")
  lower <- c(pmax(theta_0[1:p] - eps[1:p], rep(-0.999, p)),
             max(theta_0[p + 1] - eps[p + 1], 0.01),
             max(theta_0[p + 2] - eps[p + 2], 0.01))
  upper <- c(pmin(theta_0[1:p] + eps[1:p], rep(0.999, p)),
             theta_0[p + 1] + eps[p + 1],
             theta_0[p + 2] + eps[p + 2])
  mdl_null <- mdl_alt
  mdl_null$v <- nu_null
  s0_tmp <- Tsize * (LRT_moment_ged(y_mat, mdl_null, Amat_mat, WAmat, del, Bartlett) -
                       LRT_moment_ged(y_mat, mdl_alt, Amat_mat, WAmat, del, Bartlett))
  if (!is.finite(s0_tmp)) stop("Test statistic is not finite.")
  if (s0_tmp < 0) warning("Test statistic is negative (", round(s0_tmp, 4), "); capped at 1e-10.")
  s0_tmp <- max(s0_tmp, 1e-10)
  # Pre-draw innovations ONCE for fixed-error MMC (Dufour 2006, eq 4.22)
  n_total <- burnin + Tsize
  N_draw <- ceiling(N * 1.5) + 10L
  a_ged <- exp(0.5 * (lgamma(1 / nu_null) - lgamma(3 / nu_null)))
  innov_ged <- list(
    eta = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw),
    eps = matrix(NA_real_, nrow = n_total, ncol = N_draw)
  )
  for (b in seq_len(N_draw)) {
    x_gam <- rgamma(n_total, shape = 1 / nu_null, rate = 1)
    innov_ged$eps[, b] <- sign(runif(n_total) - 0.5) * a_ged * x_gam^(1 / nu_null)
  }
  out <- .run_mmc_optimizer(method, theta_0, .mmc_pval_ged, lower, upper,
                            threshold, maxit,
                            y = y_mat, j = J, N = N, s0 = s0_tmp,
                            mdl_alt = mdl_alt,
                            nu_null = nu_null, ini = burnin, Amat = Amat_mat,
                            WAmat = WAmat,
                            del = del, Bartlett = Bartlett, wDecay = wDecay,
                            direction = direction,
                            sigvMethod = sigvMethod,
                            winsorize_eps = winsorize_eps,
                            innovations = innov_ged)
  out$value <- -out$value
  out$direction <- direction
  out$s0 <- s0_tmp
  out$call <- cl
  return(out)
}

test_that("lmc_ar runs and returns svp_test object", {
  set.seed(42)
  y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.3)$y
  test <- lmc_ar(y, p_null = 1, p_alt = 2, N = 19)
  expect_s3_class(test, "svp_test")
  expect_true("pval" %in% names(test))
  expect_true(test$pval >= 0 && test$pval <= 1)
})

test_that("mmc_ar runs and returns valid p-value", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.3)$y
  test <- mmc_ar(y, p_null = 1, p_alt = 2, N = 9,
                 method = "pso", maxit = 5)
  expect_true(test$value >= 0 && test$value <= 1)
})

test_that("lmc_lev runs and returns svp_test object", {
  set.seed(42)
  y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.3)$y
  test <- lmc_lev(y, p = 1, N = 19)
  expect_s3_class(test, "svp_test")
  expect_true(test$pval >= 0 && test$pval <= 1)
})

test_that("lmc_t runs and returns svp_test object", {
  set.seed(42)
  y <- sim_svp(1000, phi = 0.90, sigy = 1, sigv = 0.3,
               errorType = "Student-t", nu = 5)$y
  test <- suppressWarnings(lmc_t(y, nu_null = 5, N = 19))
  expect_s3_class(test, "svp_test")
  expect_true(test$pval >= 0 && test$pval <= 1)
})

test_that("lmc_ged runs and returns svp_test object", {
  set.seed(42)
  y <- sim_svp(1000, phi = 0.90, sigy = 1, sigv = 0.3,
               errorType = "GED", nu = 1.5)$y
  test <- suppressWarnings(lmc_ged(y, nu_null = 1.5, N = 19))
  expect_s3_class(test, "svp_test")
  expect_true(test$pval >= 0 && test$pval <= 1)
})

test_that("print.svp_test works", {
  set.seed(42)
  y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.3)$y
  test <- lmc_ar(y, p_null = 1, p_alt = 2, N = 19)
  expect_output(print(test))
})


# =========================================================================== #
# Directional testing
# =========================================================================== #

test_that(".signed_root computes correctly", {
  expect_equal(wARMASVp:::.signed_root(4, 5, 30), sign(5 - 30) * sqrt(4))  # -2
  expect_equal(wARMASVp:::.signed_root(4, 35, 30), sign(35 - 30) * sqrt(4))  # 2
  expect_equal(wARMASVp:::.signed_root(0, 5, 30), 0)
})

test_that(".pvalue_directional computes correctly", {
  S_sim <- c(-3, -1, 0, 1, 2)
  # H1: theta < theta_0 → count S_sim <= S_obs
  pval_less <- wARMASVp:::.pvalue_directional(-2, S_sim, "less")
  expect_equal(pval_less, (1 + sum(S_sim <= -2)) / (5 + 1))  # (1+1)/6 = 1/3

  # H1: theta > theta_0 → count S_sim >= S_obs
  pval_greater <- wARMASVp:::.pvalue_directional(2, S_sim, "greater")
  expect_equal(pval_greater, (1 + sum(S_sim >= 2)) / (5 + 1))  # (1+1)/6 = 1/3
})

test_that("lmc_t with direction='two-sided' gives same structure as before", {
  set.seed(42)
  y <- sim_svp(1000, phi = 0.90, sigy = 1, sigv = 0.3,
               errorType = "Student-t", nu = 5)$y
  test <- suppressWarnings(lmc_t(y, nu_null = 5, N = 19, direction = "two-sided"))
  expect_s3_class(test, "svp_test")
  expect_equal(test$direction, "two-sided")
  expect_null(test$S_T)
  expect_true(test$pval >= 0 && test$pval <= 1)
})

test_that("lmc_t with direction='less': p-value in [0,1] and S_T present", {
  set.seed(42)
  y <- sim_svp(1000, phi = 0.90, sigy = 1, sigv = 0.3,
               errorType = "Student-t", nu = 5)$y
  test <- suppressWarnings(lmc_t(y, nu_null = 5, N = 19, direction = "less"))
  expect_s3_class(test, "svp_test")
  expect_equal(test$direction, "less")
  expect_true(!is.null(test$S_T))
  expect_true(test$pval >= 0 && test$pval <= 1)
})

test_that("lmc_ged with direction='less': p-value in [0,1]", {
  set.seed(42)
  y <- sim_svp(1000, phi = 0.90, sigy = 1, sigv = 0.3,
               errorType = "GED", nu = 1.5)$y
  test <- suppressWarnings(lmc_ged(y, nu_null = 1.5, N = 19, direction = "less"))
  expect_s3_class(test, "svp_test")
  expect_equal(test$direction, "less")
  expect_true(test$pval >= 0 && test$pval <= 1)
})

test_that("print.svp_test shows direction for directional tests", {
  set.seed(42)
  y <- sim_svp(1000, phi = 0.90, sigy = 1, sigv = 0.3,
               errorType = "Student-t", nu = 5)$y
  test <- suppressWarnings(lmc_t(y, nu_null = 5, N = 19, direction = "less"))
  expect_output(print(test), "less")
  expect_output(print(test), "Signed root")
})


# =========================================================================== #
# SV(p>1) tests — tail distribution and leverage
# =========================================================================== #

test_that("lmc_t with p=2 runs and returns svp_test object", {
  set.seed(42)
  y <- sim_svp(1000, phi = c(0.20, 0.63), sigy = 1, sigv = 1,
               errorType = "Student-t", nu = 3)$y
  test <- suppressWarnings(lmc_t(y, p = 2, nu_null = 3, N = 19))
  expect_s3_class(test, "svp_test")
  expect_true(test$pval >= 0 && test$pval <= 1)
})

test_that("mmc_t with p=2 runs and returns valid p-value", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(500, phi = c(0.20, 0.63), sigy = 1, sigv = 1,
               errorType = "Student-t", nu = 3)$y
  test <- suppressWarnings(mmc_t(y, p = 2, nu_null = 3, N = 9,
                                  method = "pso", maxit = 5))
  expect_true(test$value >= 0 && test$value <= 1)
})

test_that("lmc_ged with p=2 runs and returns svp_test object", {
  set.seed(42)
  y <- sim_svp(1000, phi = c(0.20, 0.63), sigy = 1, sigv = 1,
               errorType = "GED", nu = 1.5)$y
  test <- suppressWarnings(lmc_ged(y, p = 2, nu_null = 1.5, N = 19))
  expect_s3_class(test, "svp_test")
  expect_true(test$pval >= 0 && test$pval <= 1)
})

test_that("mmc_ged with p=2 runs and returns valid p-value", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(500, phi = c(0.20, 0.63), sigy = 1, sigv = 1,
               errorType = "GED", nu = 1.5)$y
  test <- suppressWarnings(mmc_ged(y, p = 2, nu_null = 1.5, N = 9,
                                    method = "pso", maxit = 5))
  expect_true(test$value >= 0 && test$value <= 1)
})

test_that("lmc_lev with p=2 runs and returns svp_test object", {
  set.seed(42)
  y <- sim_svp(1000, phi = c(0.20, 0.63), sigy = 1, sigv = 1)$y
  test <- suppressWarnings(lmc_lev(y, p = 2, N = 19))
  expect_s3_class(test, "svp_test")
  expect_true(test$pval >= 0 && test$pval <= 1)
})

test_that("mmc_lev with p=2 runs and returns svp_test object", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(500, phi = c(0.20, 0.63), sigy = 1, sigv = 1)$y
  test <- suppressWarnings(mmc_lev(y, p = 2, N = 9,
                                    method = "pso", maxit = 5))
  expect_true(test$value >= 0 && test$value <= 1)
})

# =========================================================================== #
# Bartlett/WAmat forwarding tests (Bugs A-D fix, 2026-04-03)
# =========================================================================== #

test_that("mmc_t with Amat='Weighted' (WAmat) runs without error", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(500, phi = 0.90, sigy = 1, sigv = 0.3,
               errorType = "Student-t", nu = 5)$y
  test <- suppressWarnings(mmc_t(y, nu_null = 5, N = 9,
                                  Amat = "Weighted",
                                  method = "pso", maxit = 5))
  expect_true(test$value >= 0 && test$value <= 1)
})

test_that("mmc_ged with Amat='Weighted' (WAmat) runs without error", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(500, phi = 0.90, sigy = 1, sigv = 0.3,
               errorType = "GED", nu = 1.5)$y
  test <- suppressWarnings(mmc_ged(y, nu_null = 1.5, N = 9,
                                    Amat = "Weighted",
                                    method = "pso", maxit = 5))
  expect_true(test$value >= 0 && test$value <= 1)
})

test_that("mmc_lev Gaussian Amat='Weighted' runs without error", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.3)$y
  test <- suppressWarnings(mmc_lev(y, N = 9, Amat = "Weighted",
                                    method = "pso", maxit = 5))
  expect_true(test$value >= 0 && test$value <= 1)
})

test_that("mmc_lev Student-t Amat='Weighted' runs without error", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(500, phi = 0.90, sigy = 1, sigv = 0.3,
               errorType = "Student-t", nu = 5)$y
  test <- suppressWarnings(mmc_lev(y, N = 9, Amat = "Weighted",
                                    errorType = "Student-t",
                                    method = "pso", maxit = 5))
  expect_true(test$value >= 0 && test$value <= 1)
})

test_that("mmc_lev GED Amat='Weighted' runs without error", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(500, phi = 0.90, sigy = 1, sigv = 0.3,
               errorType = "GED", nu = 1.5)$y
  test <- suppressWarnings(mmc_lev(y, N = 9, Amat = "Weighted",
                                    errorType = "GED",
                                    method = "pso", maxit = 5))
  expect_true(test$value >= 0 && test$value <= 1)
})

test_that("mmc_t eps length validation works", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(500, phi = c(0.20, 0.63), sigy = 1, sigv = 1,
               errorType = "Student-t", nu = 3)$y
  expect_error(
    mmc_t(y, p = 2, nu_null = 3, N = 9, eps = c(0.3, 0.3, 0.3)),
    "eps must have length 4"
  )
})

test_that("mmc_ged eps length validation works", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(500, phi = c(0.20, 0.63), sigy = 1, sigv = 1,
               errorType = "GED", nu = 1.5)$y
  expect_error(
    mmc_ged(y, p = 2, nu_null = 1.5, N = 9, eps = c(0.3, 0.3, 0.3)),
    "eps must have length 4"
  )
})

# ---- mmc_lev heavy-tail eps tests ----

test_that("mmc_lev Student-t accepts eps of length p+3 (includes nu)", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(500, phi = 0.90, sigy = 1, sigv = 0.3,
               errorType = "Student-t", nu = 5, leverage = TRUE, rho = -0.3)$y
  test <- suppressWarnings(mmc_lev(y, N = 9, errorType = "Student-t",
                                    eps = c(0.3, 0.3, 0.3, 2.0),
                                    method = "pso", maxit = 5))
  expect_true(test$value >= 0 && test$value <= 1)
  expect_length(test$par, 4)  # phi, sigy, sigv, nu
})

test_that("mmc_lev GED accepts eps of length p+3 (includes nu)", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(500, phi = 0.90, sigy = 1, sigv = 0.3,
               errorType = "GED", nu = 1.5, leverage = TRUE, rho = -0.3)$y
  test <- suppressWarnings(mmc_lev(y, N = 9, errorType = "GED",
                                    eps = c(0.3, 0.3, 0.3, 0.5),
                                    method = "pso", maxit = 5))
  expect_true(test$value >= 0 && test$value <= 1)
  expect_length(test$par, 4)  # phi, sigy, sigv, nu
})

test_that("mmc_lev Student-t eps length p+2 still works (backward compat)", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(500, phi = 0.90, sigy = 1, sigv = 0.3,
               errorType = "Student-t", nu = 5, leverage = TRUE, rho = -0.3)$y
  test <- suppressWarnings(mmc_lev(y, N = 9, errorType = "Student-t",
                                    eps = c(0.3, 0.3, 0.3),
                                    method = "pso", maxit = 5))
  expect_true(test$value >= 0 && test$value <= 1)
})

test_that("mmc_lev heavy-tail eps length validation rejects wrong lengths", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(500, phi = 0.90, sigy = 1, sigv = 0.3,
               errorType = "Student-t", nu = 5, leverage = TRUE, rho = -0.3)$y
  # length 1 — wrong
  expect_error(
    mmc_lev(y, N = 9, errorType = "Student-t", eps = 0.3,
            method = "pso", maxit = 5),
    "eps must have length"
  )
  # length p+4 = 5 — wrong
  expect_error(
    mmc_lev(y, N = 9, errorType = "Student-t", eps = rep(0.3, 5),
            method = "pso", maxit = 5),
    "eps must have length"
  )
})

test_that("mmc_lev Student-t p=2 eps of length p+3=5 works", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(500, phi = c(0.20, 0.63), sigy = 1, sigv = 1.0,
               errorType = "Student-t", nu = 5, leverage = TRUE, rho = -0.3)$y
  test <- suppressWarnings(mmc_lev(y, p = 2, N = 9, errorType = "Student-t",
                                    eps = c(0.3, 0.3, 0.3, 0.3, 2.0),
                                    method = "pso", maxit = 5))
  expect_true(test$value >= 0 && test$value <= 1)
  expect_length(test$par, 5)  # phi1, phi2, sigy, sigv, nu
})


# =========================================================================== #
# Unified Amat interface tests (2026-04-15)
# =========================================================================== #

test_that("lmc_ar with Amat='Weighted' runs (HAC via Amat)", {
  set.seed(42)
  y <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.2)$y
  test <- lmc_ar(y, p_null = 1, p_alt = 2, N = 19, Amat = "Weighted")
  expect_s3_class(test, "svp_test")
  expect_true(test$pval >= 0 && test$pval <= 1)
  expect_true(grepl("Bartlett", test$test_type))
})

test_that("mmc_ar with Amat='Weighted' runs (HAC via Amat)", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.2)$y
  test <- mmc_ar(y, p_null = 1, p_alt = 2, N = 9, Amat = "Weighted",
                 method = "pso", maxit = 5)
  expect_true(test$value >= 0 && test$value <= 1)
})

test_that("lmc_ar rejects user-supplied weighting matrix", {
  set.seed(42)
  y <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.2)$y
  expect_error(
    lmc_ar(y, p_null = 1, p_alt = 2, N = 19, Amat = diag(4)),
    "not supported"
  )
})

test_that("lmc_lev with Amat='Weighted' runs (HAC via Amat)", {
  set.seed(42)
  y <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.2,
               leverage = TRUE, rho = -0.3)$y
  test <- suppressWarnings(lmc_lev(y, p = 1, N = 19, Amat = "Weighted"))
  expect_s3_class(test, "svp_test")
  expect_true(test$pval >= 0 && test$pval <= 1)
})

test_that("mmc_lev with Amat='Weighted' runs (HAC via Amat)", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.2,
               leverage = TRUE, rho = -0.3)$y
  test <- suppressWarnings(mmc_lev(y, p = 1, N = 9, Amat = "Weighted",
                                    method = "pso", maxit = 5))
  expect_true(test$value >= 0 && test$value <= 1)
})

test_that("Amat takes precedence over Bartlett", {
  set.seed(42)
  y <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.2)$y
  # Amat = "Weighted" should override Bartlett = FALSE
  test <- lmc_ar(y, p_null = 1, p_alt = 2, N = 19,
                 Bartlett = FALSE, Amat = "Weighted")
  expect_true(grepl("Bartlett", test$test_type))
})

test_that("lmc_t default is now identity weighting (Bartlett=FALSE)", {
  set.seed(42)
  y <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.2,
               errorType = "Student-t", nu = 5)$y
  # Default call — should use identity (no HAC), same as Bartlett=FALSE
  test_default <- suppressWarnings(lmc_t(y, nu_null = 5, N = 19))
  test_explicit <- suppressWarnings(lmc_t(y, nu_null = 5, N = 19,
                                           Bartlett = FALSE, Amat = NULL))
  expect_equal(test_default$s0, test_explicit$s0)
})

test_that("lmc_t Amat='Weighted' gives HAC weighting", {
  set.seed(42)
  y <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.2,
               errorType = "Student-t", nu = 5)$y
  # Amat="Weighted" should produce same result as Bartlett=TRUE
  test_amat <- suppressWarnings(lmc_t(y, nu_null = 5, N = 19,
                                       Amat = "Weighted"))
  test_bart <- suppressWarnings(lmc_t(y, nu_null = 5, N = 19,
                                       Bartlett = TRUE))
  expect_equal(test_amat$s0, test_bart$s0)
})

# =========================================================================== #
# Information criteria for SV(p) AR-order selection
#
# Six criteria computed against an svp() fit:
#   1. AIC_YW       Yule-Walker projection-error AIC on log(y^2+del)
#   2. BIC_YW       Yule-Walker projection-error BIC on log(y^2+del)
#   3. BIC_Whittle  Whittle log-likelihood BIC at SV(p) signal-plus-noise spectrum
#   4. AIC_Kalman   Quasi-likelihood AIC using filter_svp() log-likelihood
#   5. BIC_Kalman   Quasi-likelihood BIC using filter_svp() log-likelihood
#   6. AICc_Kalman  Hurvich-Tsai (1989) small-sample-corrected AIC_Kalman
# =========================================================================== #

# Number of free parameters in an SV(p) model object
.svp_n_params <- function(p, errorType, leverage) {
  p + 2L +
    as.integer(errorType != "Gaussian") +
    as.integer(isTRUE(leverage))
}

# sigma_eps^2 = Var(log z_t^2) by error type. Used in spectral density.
.svp_sigma_eps2 <- function(errorType, nu = NULL) {
  switch(errorType,
    "Gaussian"  = pi^2 / 2,
    "Student-t" = trigamma(0.5) + trigamma(nu / 2),
    "GED"       = (2 / nu)^2 * trigamma(1 / nu),
    stop("Unknown errorType: ", errorType))
}

# Sample autocovariances of y*_t = log(y^2 + del) - mean
# Returns numeric vector of length max_lag + 1: gamma_y*(0..max_lag).
# Divisor 1/T (consistent with FFT periodogram / Whittle convention).
.svp_acov_ystar <- function(y, max_lag, del = 1e-10) {
  ystar <- log(y^2 + del)
  ystar <- ystar - mean(ystar)
  Tn <- length(ystar)
  vapply(0:max_lag,
         function(k) sum(ystar[(1L + k):Tn] * ystar[1:(Tn - k)]) / Tn,
         numeric(1))
}

# Yule-Walker projection-error variance for the AR(p) projection of y*_t
# onto its own past at lags 1..p, evaluated at the W-ARMA-SV phi estimate.
# sigma_pred^2 = gamma_y*(0) - phi' gamma_y*(1:p).
# Diagnostic; consistency in the SV(p) setting is demonstrated by simulation
# (Ahsan, Dufour & Rodriguez-Rondon, 2026) rather than analytically.
.svp_residual_var <- function(fit, y, del = 1e-10) {
  phi <- as.numeric(fit$phi)
  p <- length(phi)
  gv <- .svp_acov_ystar(y, p, del)
  v <- gv[1] - sum(phi * gv[2:(p + 1L)])
  if (!is.finite(v) || v <= 0) NA_real_ else v
}

# Hannan-Rissanen (1982, Biometrika) two-stage ARMA(p,p) residual variance.
# y*_t = log(y^2 + del) - mean has the form h_t + epsilon_t under SV(p), which
# is ARMA(p, p) (Harvey, Ruiz & Shephard 1994).  Stage 1 fits a long AR(L)
# pre-whitening regression; Stage 2 is OLS of y* on its lags 1..p plus the
# stage-1 residuals at lags 1..p.  Residual variance from Stage 2 is the
# Hannan-Rissanen estimator of the ARMA innovation variance.
#
# L_const: scaling for stage-1 AR length; default 1.5 (so L = floor(1.5*T^{1/3})).
# Returns scalar sigma_hr^2, T_eff (rows used in Stage 2), or NA if any guard fails.
.svp_hr_stage1 <- function(ystar, L_const = 1.5) {
  Tn <- length(ystar)
  L  <- max(5L, as.integer(floor(L_const * Tn^(1/3))))
  L  <- min(L, as.integer(floor(Tn / 4L)))
  if (Tn - L < 30L) return(list(eps = NULL, L = L))
  # Build long-AR design at lags 1..L
  X <- matrix(NA_real_, Tn - L, L)
  for (k in 1:L) X[, k] <- ystar[(L - k + 1L):(Tn - k)]
  y <- ystar[(L + 1L):Tn]
  X <- cbind(1, X)
  fit <- tryCatch(qr.coef(qr(X), y), error = function(e) NULL)
  if (is.null(fit) || any(!is.finite(fit))) return(list(eps = NULL, L = L))
  fitted <- as.numeric(X %*% fit)
  eps <- y - fitted
  list(eps = eps, L = L)
}

.svp_hr_stage2 <- function(ystar, eps_hat, L, p) {
  Tn <- length(ystar)
  start <- L + p + 1L  # need ystar lags 1..p AND eps_hat lags 1..p available
  if (start >= Tn) return(NA_real_)
  T_eff <- Tn - start + 1L
  if (T_eff < 2L * p + 5L) return(NA_real_)
  # Build the design: intercept, p AR lags of ystar, p MA lags of eps_hat
  yvec <- ystar[start:Tn]
  Xmat <- matrix(NA_real_, T_eff, 1L + 2L * p)
  Xmat[, 1L] <- 1.0
  for (k in 1:p) Xmat[, 1L + k]      <- ystar[(start - k):(Tn - k)]
  for (k in 1:p) Xmat[, 1L + p + k]  <- eps_hat[(start - L - k):(Tn - L - k)]
  beta <- tryCatch(qr.coef(qr(Xmat), yvec), error = function(e) NULL)
  if (is.null(beta) || any(!is.finite(beta))) return(NA_real_)
  resid <- yvec - as.numeric(Xmat %*% beta)
  ss <- sum(resid * resid)
  df <- T_eff - (2L * p + 1L)
  if (df <= 0L || !is.finite(ss) || ss <= 0) return(NA_real_)
  ss / df
}

# Hannan-Rissanen ARMA(p,p) residual variance — wrapper.
.svp_hr_residual_var <- function(fit, y, del = 1e-10, L_const = 1.5) {
  ystar <- log(y^2 + del); ystar <- ystar - mean(ystar)
  s1 <- .svp_hr_stage1(ystar, L_const)
  if (is.null(s1$eps)) return(list(sigma2 = NA_real_, T_eff = NA_integer_))
  p <- length(fit$phi)
  sigma2 <- .svp_hr_stage2(ystar, s1$eps, s1$L, p)
  if (is.na(sigma2)) return(list(sigma2 = NA_real_, T_eff = NA_integer_))
  T_eff <- length(ystar) - s1$L - p
  list(sigma2 = sigma2, T_eff = as.integer(T_eff))
}

# Whittle log-likelihood of y*_t at the SV(p) signal-plus-noise spectrum.
# Periodogram convention (no 1/(2*pi) factors): for consistency between
# periodogram and spectral density, both omit the 1/(2*pi). This is a
# constant additive shift -m * log(2*pi)/2 from the Brockwell-Davis form
# and does not affect ranking.
#
#   I(omega_j) = |FFT(y*)|^2 / T,   omega_j = 2*pi*j/T
#   f_y*(omega) = sigma_v^2 / |1 - sum phi_j e^{-i j omega}|^2 + sigma_eps^2(nu)
#   ell_W = -0.5 * sum_j [log f(omega_j) + I(omega_j) / f(omega_j)]
.svp_whittle_loglik <- function(fit, y, errorType, del = 1e-10) {
  ystar <- log(y^2 + del)
  ystar <- ystar - mean(ystar)
  Tn <- length(ystar)
  m  <- floor((Tn - 1L) / 2L)
  if (m < 1L) return(NA_real_)
  freqs <- 2 * pi * seq_len(m) / Tn

  Y <- stats::fft(ystar)
  I <- (Mod(Y)^2 / Tn)[2:(m + 1L)]

  phi <- as.numeric(fit$phi)
  sigv2 <- as.numeric(fit$sigv)^2
  nu <- if (errorType == "Gaussian") NA_real_ else as.numeric(fit$v)
  sigeps2 <- .svp_sigma_eps2(errorType, nu)

  # |phi(e^{-iw})|^2 with phi(z) = 1 - sum phi_j z^j
  poly_mag2 <- vapply(freqs, function(om) {
    z <- exp(-1i * om)
    Mod(1 - sum(phi * z^seq_along(phi)))^2
  }, numeric(1))

  f_y <- sigv2 / poly_mag2 + sigeps2

  if (any(!is.finite(f_y)) || any(f_y <= 0)) return(NA_real_)
  -0.5 * sum(log(f_y) + I / f_y)
}


#' Information criteria for SV(p) AR-order selection
#'
#' Computes information criteria for an \code{\link{svp}} fit to support
#' AR-order selection. Eight criteria are computable; \strong{four are
#' returned by default} — \code{BIC_Kalman}, \code{AIC_Kalman},
#' \code{BIC_HR}, \code{AIC_HR}. These span two families (state-space QML
#' and Hannan--Rissanen two-stage ARMA) and two penalty philosophies
#' (Schwarz-consistent BIC / Shibata-efficient AIC), and were selected as
#' the most informative criteria across the simulation grid of the
#' SVHT methodology paper (Ahsan, Dufour and Rodriguez-Rondon 2026).
#' The remaining four are available on request via the \code{criteria}
#' argument.
#'
#' \strong{Default criteria (returned by \code{svp_IC(fit)}):}
#' \itemize{
#'   \item \code{BIC_Kalman}, \code{AIC_Kalman}: \eqn{-2\,\hat\ell_K + k\log T}
#'         and \eqn{-2\,\hat\ell_K + 2k} where \eqn{\hat\ell_K} is the
#'         (quasi-)log-likelihood from \code{\link{filter_svp}}; default
#'         \code{filter_method = "mixture"} uses the Gaussian mixture
#'         Kalman filter (Kim, Shephard and Chib 1998). \code{BIC_Kalman}
#'         is the primary recommended criterion: Schwarz-consistent under
#'         the Bayes-optimal leverage proxy (see \code{proxy} argument)
#'         and strong finite-sample performance across the simulation
#'         grid (Ahsan, Dufour and Rodriguez-Rondon 2026).
#'         \code{AIC_Kalman} is Shibata-efficient
#'         and often selects larger \eqn{p} sooner at \eqn{p_{\mathrm{true}}
#'         \ge 2}.
#'   \item \code{BIC_HR}, \code{AIC_HR}: Hannan--Rissanen (1982) two-stage
#'         ARMA(\eqn{p,p}) criteria. Stage 1: long-AR pre-whitening at
#'         order \eqn{L = \lfloor 1.5\, T^{1/3}\rfloor} produces residuals
#'         \eqn{\hat\varepsilon_t}. Stage 2: OLS regression of \eqn{y_t^*}
#'         on AR lags \eqn{1{:}p} of \eqn{y_t^*} and MA lags \eqn{1{:}p}
#'         of \eqn{\hat\varepsilon_t} gives \eqn{\hat\sigma_u^2}. Then
#'         \eqn{T_{\mathrm{eff}} \log \hat\sigma_u^2 + \{2(2p{+}1),
#'         (2p{+}1)\log T_{\mathrm{eff}}\}}. Filter-free anchor, robust to
#'         mis-specification of the GMKF mixture. \code{BIC_HR} is
#'         Schwarz-consistent for ARMA(\eqn{p,p}) (Hannan & Rissanen
#'         1982; Pötscher 1989).
#' }
#'
#' \strong{Opt-in criteria (request via \code{criteria = ...}):}
#' \itemize{
#'   \item \code{AICc_Kalman}: \code{AIC_Kalman} with the Hurvich--Tsai
#'         (1989) small-sample correction \eqn{2k(k+1)/(T-k-1)}.
#'         Numerically equivalent to \code{AIC_Kalman} at \eqn{T \ge 500};
#'         use when \eqn{T < 500}.
#'   \item \code{BIC_Whittle}: \eqn{-2\,\hat\ell_W + k\log T} where
#'         \eqn{\hat\ell_W} is the Whittle log-likelihood evaluated at the
#'         SV(\eqn{p}) signal-plus-noise spectral density
#'         \eqn{f(\omega) = \sigma_v^2 / |1 - \sum_j \phi_j e^{-ij\omega}|^2
#'                        + \sigma_\varepsilon^2(\nu)}.
#'         Schwarz-consistent but collapses to \eqn{\hat p = 1} in 98--100\%
#'         of cells at \eqn{p_{\mathrm{true}} \ge 2} under near-unit-root
#'         persistence (Ahsan, Dufour and Rodriguez-Rondon 2026). Useful
#'         as a conservative
#'         diagnostic: a Whittle selection of \eqn{p > 1} is strong
#'         evidence against \eqn{p = 1}.
#'   \item \code{AIC_YW}, \code{BIC_YW}: \emph{Legacy / not recommended.}
#'         Yule--Walker projection-error criteria on
#'         \eqn{y_t^* = \log(y_t^2 + \delta) - \mu}, computed as
#'         \eqn{T \log \hat\sigma_{\mathrm{pred}}^2 + \{2k, k\log T\}}
#'         with the AR(\eqn{p}) projection-error variance. Under
#'         SV(\eqn{p}), \eqn{y_t^*} is ARMA(\eqn{p,p}) (not AR(\eqn{p})),
#'         so the AR projection error does not saturate at
#'         \eqn{p_{\mathrm{true}}} and the criteria are
#'         \emph{inconsistent}: the AR(\eqn{p}) projection-error variance
#'         keeps decreasing past \eqn{p_{\mathrm{true}}}, producing
#'         non-monotone (sometimes anti-Schwarz) behaviour in \eqn{T}.
#'         Simulation evidence: 0--29\% correct selection at
#'         \eqn{p_{\mathrm{true}} = 2} across all DGP cells and
#'         \eqn{T \le 10{,}000} (Ahsan, Dufour and Rodriguez-Rondon
#'         2026). Retained for paper-reproducibility of the
#'         documented failure-case results; \strong{use \code{BIC_HR} /
#'         \code{AIC_HR} for theoretically consistent AR-order selection.}
#' }
#'
#' Lower is better; \code{argmin} over a grid of candidate \code{p} (see
#' \code{\link{svp_AR_order}}) selects the AR order.
#'
#' @section Leverage invariance of non-Kalman criteria:
#' Leverage does not affect \code{AIC_YW}, \code{BIC_YW}, or
#' \code{BIC_Whittle}: under the W-ARMA-SV parameterization
#' \eqn{\mathrm{Cov}(v_t, \varepsilon_{t-1}) = 0} for all three error
#' distributions (odd-times-even moment symmetry), so the autocovariance
#' structure of \eqn{y_t^*} is invariant to the leverage parameter. The
#' \code{*_HR} and \code{*_Kalman} criteria do incorporate leverage
#' through the estimated \eqn{\delta_p} and the conditional state
#' innovation variance.
#'
#' @param fit Output of \code{\link{svp}}. Must carry the original \code{y}
#'   series (which \code{svp()} stores by default), \code{errorType}, and
#'   \code{leverage} fields.
#' @param criteria Character vector. Subset of
#'   \code{c("BIC_Kalman", "AIC_Kalman", "AICc_Kalman", "BIC_Whittle",
#'   "BIC_HR", "AIC_HR", "BIC_YW", "AIC_YW")}. Default returns the four
#'   recommended criteria: \code{c("BIC_Kalman", "AIC_Kalman", "BIC_HR",
#'   "AIC_HR")}. See the description for the rationale for each
#'   opt-in criterion.
#' @param filter_method Character. Filter method passed to
#'   \code{\link{filter_svp}} for \code{*_Kalman} criteria. One of
#'   \code{"mixture"} (default, recommended), \code{"corrected"}, or
#'   \code{"particle"}.
#' @param proxy Character. Leverage proxy passed to \code{\link{filter_svp}}.
#'   \code{"bayes_optimal"} (default here, unlike \code{filter_svp})
#'   removes the \eqn{O(T)} log-likelihood bias of the û-proxy under
#'   Student-t leverage and restores Schwarz consistency of \code{BIC_Kalman}.
#'   \code{"u"} reproduces the paper-faithful (Remark 3.5) Kalman likelihood;
#'   set this if you need IC values that match the filter's default behavior.
#' @param K Integer. Number of mixture components for
#'   \code{filter_method = "mixture"}. Default 7 (KSC).
#' @param M Integer. Number of particles for
#'   \code{filter_method = "particle"}. Default 1000. Ignored for other
#'   filter methods.
#' @param seed Integer. Random seed for the bootstrap particle filter.
#'   Default 42. Ignored for non-particle filters.
#' @param del Numeric. Small constant added inside \eqn{\log} to avoid
#'   \eqn{\log 0}. Default \code{1e-10}.
#'
#' @return Named numeric vector of the requested criteria. Lower is better.
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' y <- sim_svp(2000, phi = 0.95, sigy = 1, sigv = 0.5)$y
#' fit1 <- svp(y, p = 1)
#' fit2 <- svp(y, p = 2)
#' svp_IC(fit1)
#' svp_IC(fit2)
#' }
#'
#' @references
#' Akaike, H. (1974). A new look at the statistical model identification.
#' \emph{IEEE Transactions on Automatic Control} 19(6), 716--723.
#' \doi{10.1109/TAC.1974.1100705}
#'
#' Schwarz, G. (1978). Estimating the dimension of a model.
#' \emph{Annals of Statistics} 6(2), 461--464.
#' \doi{10.1214/aos/1176344136}
#'
#' Shibata, R. (1976). Selection of the order of an autoregressive model
#' by Akaike's information criterion. \emph{Biometrika} 63(1), 117--126.
#' \doi{10.1093/biomet/63.1.117}
#'
#' Hannan, E. J. (1980). The estimation of the order of an ARMA process.
#' \emph{Annals of Statistics} 8(5), 1071--1081.
#' \doi{10.1214/aos/1176345144}
#'
#' Hannan, E. J., and Rissanen, J. (1982). Recursive estimation of mixed
#' autoregressive-moving average order. \emph{Biometrika} 69(1), 81--94.
#' \doi{10.1093/biomet/69.1.81}
#'
#' Pötscher, B. M. (1989). Model selection under nonstationarity:
#' Autoregressive models and stochastic linear regression models.
#' \emph{Annals of Statistics} 17(3), 1257--1274.
#' \doi{10.1214/aos/1176347267}
#'
#' Whittle, P. (1953). Estimation and information in stationary time series.
#' \emph{Arkiv f\"or Matematik} 2, 423--434. \doi{10.1007/BF02590998}
#'
#' Dunsmuir, W. (1979). A central limit theorem for parameter estimation in
#' stationary vector time series and its application to models for a signal
#' observed with noise. \emph{Annals of Statistics} 7(3), 490--506.
#' \doi{10.1214/aos/1176344671}
#'
#' Hurvich, C. M., and Tsai, C.-L. (1989). Regression and time series model
#' selection in small samples. \emph{Biometrika} 76(2), 297--307.
#' \doi{10.1093/biomet/76.2.297}
#'
#' Kim, S., Shephard, N., and Chib, S. (1998). Stochastic volatility:
#' likelihood inference and comparison with ARCH models.
#' \emph{Review of Economic Studies} 65(3), 361--393.
#' \doi{10.1111/1467-937X.00050}
#'
#' White, H. (1982). Maximum likelihood estimation of misspecified models.
#' \emph{Econometrica} 50(1), 1--25. \doi{10.2307/1912526}
#'
#' Ahsan, M. N., Dufour, J.-M., and Rodriguez-Rondon, G. (2026). Estimation and
#' inference for stochastic volatility models with heavy-tailed distributions.
#' Bank of Canada Staff Working Paper 2026-8. \doi{10.34989/swp-2026-8}
#'
#' @seealso \code{\link{svp_AR_order}}, \code{\link{svp}}, \code{\link{filter_svp}}
#' @export
svp_IC <- function(fit,
                   criteria = c("BIC_Kalman", "AIC_Kalman",
                                "BIC_HR", "AIC_HR"),
                   filter_method = c("mixture", "corrected", "particle"),
                   proxy = c("bayes_optimal", "u"),
                   K = 7L, M = 1000L, seed = 42L,
                   del = 1e-10) {
  if (!inherits(fit, c("svp", "svp_t", "svp_ged")))
    stop("'fit' must be an svp() output (class 'svp', 'svp_t' or 'svp_ged').")

  all_crit <- c("BIC_Kalman", "AIC_Kalman", "AICc_Kalman",
                "BIC_Whittle",
                "BIC_HR", "AIC_HR",
                "BIC_YW", "AIC_YW")
  criteria <- match.arg(criteria, choices = all_crit, several.ok = TRUE)
  filter_method <- match.arg(filter_method)
  proxy <- match.arg(proxy)

  errorType <- fit$errorType
  if (is.null(errorType))
    stop("'fit$errorType' is missing; refit with the current package version.")
  leverage <- isTRUE(fit$leverage)

  y <- as.numeric(fit$y)
  if (length(y) < 4L) stop("Series too short for IC computation.")

  Tn <- length(y)
  p  <- length(fit$phi)
  k  <- .svp_n_params(p, errorType, leverage)

  out <- setNames(rep(NA_real_, length(criteria)), criteria)

  # YW projection-error criteria (legacy — see roxygen note)
  if (any(c("AIC_YW", "BIC_YW") %in% criteria)) {
    sig2 <- .svp_residual_var(fit, y, del)
    if (is.finite(sig2)) {
      if ("AIC_YW" %in% criteria) out["AIC_YW"] <- Tn * log(sig2) + 2 * k
      if ("BIC_YW" %in% criteria) out["BIC_YW"] <- Tn * log(sig2) + k * log(Tn)
    }
  }

  # Hannan-Rissanen ARMA(p,p) criteria — recommended over YW for SV(p)
  if (any(c("AIC_HR", "BIC_HR") %in% criteria)) {
    hr <- .svp_hr_residual_var(fit, y, del)
    if (!is.na(hr$sigma2) && hr$sigma2 > 0 && !is.na(hr$T_eff) && hr$T_eff > 0) {
      Te <- hr$T_eff
      kHR <- 2L * length(fit$phi) + 1L  # AR(p) coefs + MA(p) coefs + intercept
      logS <- log(hr$sigma2)
      if ("AIC_HR" %in% criteria) out["AIC_HR"] <- Te * logS + 2 * kHR
      if ("BIC_HR" %in% criteria) out["BIC_HR"] <- Te * logS + kHR * log(Te)
    }
  }

  # Whittle BIC
  if ("BIC_Whittle" %in% criteria) {
    ell_W <- .svp_whittle_loglik(fit, y, errorType, del)
    if (is.finite(ell_W))
      out["BIC_Whittle"] <- -2 * ell_W + k * log(Tn)
  }

  # Kalman QML criteria
  if (any(c("AIC_Kalman", "BIC_Kalman", "AICc_Kalman") %in% criteria)) {
    filt <- tryCatch(
      filter_svp(fit, method = filter_method, proxy = proxy,
                 K = K, M = M, seed = seed, del = del),
      error = function(e) NULL
    )
    if (!is.null(filt) && is.finite(filt$loglik)) {
      m2L <- -2 * filt$loglik
      if ("AIC_Kalman"  %in% criteria) out["AIC_Kalman"]  <- m2L + 2 * k
      if ("BIC_Kalman"  %in% criteria) out["BIC_Kalman"]  <- m2L + k * log(Tn)
      if ("AICc_Kalman" %in% criteria) {
        if (Tn - k - 1L > 0L)
          out["AICc_Kalman"] <- m2L + 2 * k + 2 * k * (k + 1L) / (Tn - k - 1L)
      }
    }
  }

  out
}


#' AR-order selection sweep for SV(p)
#'
#' Convenience wrapper around \code{\link{svp_IC}}: fits \code{\link{svp}}
#' at each \code{p = 1, ..., pmax} and returns a matrix of information
#' criteria along with the argmin per criterion.
#'
#' @param y Numeric vector. Observed returns.
#' @param pmax Integer. Maximum AR order to consider. Default 6.
#' @param J Integer. Winsorizing parameter passed to \code{\link{svp}}.
#'   Default 10.
#' @param leverage Logical. Whether to estimate leverage. Default
#'   \code{FALSE}.
#' @param errorType Character. \code{"Gaussian"}, \code{"Student-t"}, or
#'   \code{"GED"}. Default \code{"Gaussian"}.
#' @param rho_type,del,trunc_lev,wDecay,logNu,sigvMethod,winsorize_eps Other
#'   arguments passed to \code{\link{svp}}.
#' @param filter_method Character. Filter method for \code{*_Kalman}
#'   criteria. Default \code{"mixture"}.
#' @param proxy Character. Leverage proxy. Default \code{"bayes_optimal"}
#'   for IC consistency under Student-t leverage. See \code{\link{svp_IC}}.
#' @param K,M,seed Filter arguments passed to \code{\link{filter_svp}}.
#' @param criteria Character vector of criteria to compute. Default
#'   returns the four recommended criteria: \code{c("BIC_Kalman",
#'   "AIC_Kalman", "BIC_HR", "AIC_HR")}. See \code{\link{svp_IC}} for the
#'   full set of eight valid names and the rationale for each opt-in
#'   criterion.
#'
#' @return A list with components:
#' \describe{
#'   \item{IC}{Numeric matrix, one row per criterion, one column per
#'     candidate \code{p} in \code{1:pmax}.}
#'   \item{argmin}{Named integer vector, one entry per criterion, giving the
#'     selected \code{p}. \code{NA_integer_} if all entries for that
#'     criterion are \code{NA}.}
#'   \item{fits}{List of length \code{pmax} containing the fitted
#'     \code{svp()} objects (or \code{NULL} if a fit failed).}
#' }
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' y <- sim_svp(2000, phi = 0.95, sigy = 1, sigv = 0.5)$y
#' res <- svp_AR_order(y, pmax = 4)
#' res$IC
#' res$argmin
#' }
#'
#' @seealso \code{\link{svp_IC}}, \code{\link{svp}}, \code{\link{filter_svp}}
#' @export
svp_AR_order <- function(y, pmax = 6L,
                         J = 10L, leverage = FALSE, errorType = "Gaussian",
                         rho_type = "pearson", del = 1e-10, trunc_lev = TRUE,
                         wDecay = FALSE, logNu = FALSE,
                         sigvMethod = "factored", winsorize_eps = 0L,
                         filter_method = "mixture",
                         proxy = c("bayes_optimal", "u"),
                         K = 7L, M = 1000L, seed = 42L,
                         criteria = c("BIC_Kalman", "AIC_Kalman",
                                      "BIC_HR", "AIC_HR")) {
  pmax <- as.integer(pmax)
  if (pmax < 1L) stop("'pmax' must be a positive integer.")

  all_crit <- c("BIC_Kalman", "AIC_Kalman", "AICc_Kalman",
                "BIC_Whittle",
                "BIC_HR", "AIC_HR",
                "BIC_YW", "AIC_YW")
  criteria <- match.arg(criteria, choices = all_crit, several.ok = TRUE)
  filter_method <- match.arg(filter_method, c("mixture", "corrected", "particle"))
  proxy <- match.arg(proxy)

  # Pre-allocate IC_mat in the order requested by the user (criteria arg).
  # Note: row names follow the user-specified ordering.
  IC_mat <- matrix(NA_real_, nrow = length(criteria), ncol = pmax,
                   dimnames = list(criteria, paste0("p=", seq_len(pmax))))
  fits <- vector("list", pmax)

  for (p in seq_len(pmax)) {
    fit <- tryCatch(
      svp(y, p = p, J = J, leverage = leverage, errorType = errorType,
          rho_type = rho_type, del = del, trunc_lev = trunc_lev,
          wDecay = wDecay, logNu = logNu,
          sigvMethod = sigvMethod, winsorize_eps = winsorize_eps),
      error = function(e) NULL
    )
    fits[[p]] <- fit
    if (!is.null(fit)) {
      IC_mat[, p] <- svp_IC(fit, criteria = criteria,
                             filter_method = filter_method,
                             proxy = proxy,
                             K = K, M = M, seed = seed, del = del)
    }
  }

  argmin <- vapply(seq_len(nrow(IC_mat)), function(i) {
    row <- IC_mat[i, ]
    if (all(is.na(row))) NA_integer_ else as.integer(which.min(row))
  }, integer(1))
  names(argmin) <- rownames(IC_mat)

  list(IC = IC_mat, argmin = argmin, fits = fits)
}

test_that("svp_IC returns the 4 default criteria for Gaussian SV(1) without leverage", {
  set.seed(101)
  y <- sim_svp(800, phi = 0.95, sigy = 1, sigv = 0.4)$y
  fit <- svp(y, p = 1)
  ic <- svp_IC(fit)
  expect_named(ic, c("BIC_Kalman", "AIC_Kalman", "BIC_HR", "AIC_HR"),
               ignore.order = TRUE)
  expect_true(all(is.finite(ic)))
})

test_that("svp_IC handles all errorType x leverage combinations", {
  combos <- expand.grid(
    errorType = c("Gaussian", "Student-t", "GED"),
    leverage = c(FALSE, TRUE),
    stringsAsFactors = FALSE
  )
  for (i in seq_len(nrow(combos))) {
    set.seed(200 + i)
    args <- list(n = 800, phi = 0.95, sigy = 1, sigv = 0.4,
                 errorType = combos$errorType[i],
                 leverage = combos$leverage[i])
    if (combos$errorType[i] == "Student-t") args$nu <- 5
    if (combos$errorType[i] == "GED")       args$nu <- 1.5
    if (combos$leverage[i])                 args$rho <- -0.4
    y <- do.call(sim_svp, args)$y
    fit <- suppressWarnings(svp(y, p = 1,
                                 errorType = combos$errorType[i],
                                 leverage = combos$leverage[i]))
    ic <- svp_IC(fit)
    expect_length(ic, 4L)
    # at least one finite criterion
    expect_true(any(is.finite(ic)),
                info = sprintf("Row %d: %s, leverage=%s",
                               i, combos$errorType[i], combos$leverage[i]))
  }
})

test_that("svp_IC criteria subset selection works", {
  set.seed(102)
  y <- sim_svp(800, phi = 0.95, sigy = 1, sigv = 0.4)$y
  fit <- svp(y, p = 1)

  ic_one <- svp_IC(fit, criteria = "BIC_Kalman")
  expect_named(ic_one, "BIC_Kalman")
  expect_length(ic_one, 1L)

  ic_two <- svp_IC(fit, criteria = c("BIC_YW", "AIC_YW"))
  expect_named(ic_two, c("BIC_YW", "AIC_YW"), ignore.order = TRUE)
  expect_length(ic_two, 2L)
})

test_that("svp_IC computes all 8 criteria when explicitly requested", {
  set.seed(106)
  y <- sim_svp(800, phi = 0.95, sigy = 1, sigv = 0.4)$y
  fit <- svp(y, p = 1)
  all_crit <- c("BIC_Kalman", "AIC_Kalman", "AICc_Kalman",
                "BIC_Whittle",
                "BIC_HR", "AIC_HR",
                "BIC_YW", "AIC_YW")
  ic_all <- svp_IC(fit, criteria = all_crit)
  expect_length(ic_all, 8L)
  expect_named(ic_all, all_crit, ignore.order = TRUE)
  # at least one finite criterion in each opt-in family
  expect_true(is.finite(ic_all["AICc_Kalman"]))
  expect_true(is.finite(ic_all["BIC_Whittle"]))
  expect_true(is.finite(ic_all["BIC_YW"]))
  expect_true(is.finite(ic_all["AIC_YW"]))
})

test_that("svp_IC errors on non-svp object", {
  expect_error(svp_IC(list(phi = 0.5)),
               "must be an svp")
})

test_that("svp_AR_order at SV(1) DGP selects p=1 by Kalman criteria", {
  skip_on_cran()
  set.seed(103)
  y <- sim_svp(2000, phi = 0.95, sigy = 1, sigv = 0.5)$y
  res <- svp_AR_order(y, pmax = 3L)
  expect_true(is.matrix(res$IC))
  expect_equal(dim(res$IC), c(4L, 3L))
  expect_named(res$argmin,
               c("BIC_Kalman", "AIC_Kalman", "BIC_HR", "AIC_HR"),
               ignore.order = TRUE)
  # Kalman criteria should select p=1 for SV(1) truth
  expect_equal(unname(res$argmin["BIC_Kalman"]), 1L)
  expect_equal(unname(res$argmin["AIC_Kalman"]), 1L)
})

test_that("svp_AR_order pmax=1 returns 1-column IC matrix", {
  set.seed(104)
  y <- sim_svp(500, phi = 0.95, sigy = 1, sigv = 0.4)$y
  res <- svp_AR_order(y, pmax = 1L)
  expect_equal(ncol(res$IC), 1L)
  expect_equal(length(res$fits), 1L)
  expect_true(all(res$argmin == 1L | is.na(res$argmin)))
})

test_that("internal helpers produce sensible values", {
  set.seed(105)
  y <- sim_svp(1000, phi = 0.9, sigy = 1, sigv = 0.5)$y
  fit <- svp(y, p = 1)

  # n_params: SV(1) Gaussian no-lev = 1 + 2 = 3
  expect_equal(wARMASVp:::.svp_n_params(1L, "Gaussian", FALSE), 3L)
  # SV(2) Student-t with leverage = 2 + 2 + 1 + 1 = 6
  expect_equal(wARMASVp:::.svp_n_params(2L, "Student-t", TRUE), 6L)

  # sigma_eps^2 for Gaussian
  expect_equal(wARMASVp:::.svp_sigma_eps2("Gaussian"), pi^2 / 2)
  # for Student-t at nu=5
  expect_equal(wARMASVp:::.svp_sigma_eps2("Student-t", nu = 5),
               trigamma(0.5) + trigamma(2.5))

  # acov returns max_lag+1 finite values
  acov <- wARMASVp:::.svp_acov_ystar(y, max_lag = 3L)
  expect_length(acov, 4L)
  expect_true(all(is.finite(acov)))
  expect_true(acov[1] > 0)  # variance positive

  # residual variance is finite and positive for a real fit
  rv <- wARMASVp:::.svp_residual_var(fit, y)
  expect_true(is.finite(rv) && rv > 0)
})


test_that("Hannan-Rissanen helpers run and produce sensible variance", {
  skip_on_cran()
  set.seed(1234)
  y <- sim_svp(1500, phi = 0.95, sigy = 1, sigv = 0.5)$y
  fit <- svp(y, p = 1)

  # Direct call to the wrapper
  hr <- wARMASVp:::.svp_hr_residual_var(fit, y)
  expect_true(is.finite(hr$sigma2))
  expect_true(hr$sigma2 > 0)
  expect_true(hr$T_eff > 100L)

  # Stage 1 alone: should produce residuals
  ystar <- log(y^2 + 1e-10); ystar <- ystar - mean(ystar)
  s1 <- wARMASVp:::.svp_hr_stage1(ystar)
  expect_true(!is.null(s1$eps))
  expect_true(s1$L > 0L)
  expect_true(length(s1$eps) > 0L)
})

test_that("BIC_HR / AIC_HR appear in svp_IC output and are finite", {
  skip_on_cran()
  set.seed(2345)
  y <- sim_svp(1500, phi = 0.95, sigy = 1, sigv = 0.5)$y
  fit <- svp(y, p = 2)
  ic <- svp_IC(fit, criteria = c("BIC_HR", "AIC_HR"))
  expect_named(ic, c("BIC_HR", "AIC_HR"), ignore.order = TRUE)
  expect_true(all(is.finite(ic)))
})

# =========================================================================== #
# Internal estimation helpers called by svp() in estim.R
# =========================================================================== #


# =========================================================================== #
# Correction factors and helper functions for leverage under heavy tails
# =========================================================================== #

#' Correction factor C_t(nu) for Student-t leverage estimation
#'
#' Under scale mixture \eqn{u_t = z_t \lambda_t^{-1/2}},
#' \eqn{C_t(\nu) = [E[\lambda^{-1/2}]]^2 = (\nu/2) [\Gamma((\nu-1)/2) / \Gamma(\nu/2)]^2}.
#' Exact, parameter-free.
#' @param nu Degrees of freedom (nu > 1)
#' @return C_t(nu), always > 1 for finite nu (approaches 1 as nu -> Inf)
#' @keywords internal
correction_factor_t <- function(nu) {
  exp(log(nu / 2) + 2 * (lgamma((nu - 1) / 2) - lgamma(nu / 2)))
}

#' E[|u|] for standardized GED(nu) with Var = 1
#' Closed form: sqrt(Gamma(1/nu)/Gamma(3/nu)) * Gamma(2/nu) / Gamma(1/nu)
#' @param nu GED shape parameter (nu > 0)
#' @return E[|u|]
#' @keywords internal
ged_E_abs_u <- function(nu) {
  exp(0.5 * (lgamma(1 / nu) - lgamma(3 / nu)) + lgamma(2 / nu) - lgamma(1 / nu))
}

#' Quantile function for standardized GED(nu) with Var = 1
#' Uses the relationship between GED CDF and incomplete gamma function.
#' @param p Probability (0 < p < 1). Clamped to [1e-15, 1-1e-15] internally.
#' @param nu GED shape parameter (nu > 0)
#' @return Quantile value
#' @keywords internal
qged_std <- function(p, nu) {
  p <- pmax(1e-15, pmin(1 - 1e-15, p))
  a <- exp(0.5 * (lgamma(1 / nu) - lgamma(3 / nu)))
  # Handle p = 0.5 separately (returns 0) to avoid qgamma(0, ...) NaN warnings
  result <- numeric(length(p))
  hi <- p > 0.5
  lo <- p < 0.5
  if (any(hi)) {
    result[hi] <- a * qgamma(2 * (p[hi] - 0.5), shape = 1 / nu)^(1 / nu)
  }
  if (any(lo)) {
    result[lo] <- -a * qgamma(2 * (0.5 - p[lo]), shape = 1 / nu)^(1 / nu)
  }
  result
}

#' Gauss-Hermite quadrature nodes and weights for N(0,1) integration
#' Computes nodes z_i and weights w_i such that sum(w_i * f(z_i)) approximates E[f(Z)]
#' where Z ~ N(0,1). Uses the Golub-Welsch algorithm.
#' @param n Number of quadrature points (default 200)
#' @return List with components nodes and weights
#' @keywords internal
.gauss_hermite_normal <- function(n = 200L) {
  # Golub-Welsch: eigenvalues of tridiagonal Jacobi matrix for Hermite polynomials
  i <- seq_len(n)
  # Recurrence coefficients for (physicist's) Hermite: a_i = 0, b_i = sqrt(i/2)
  b <- sqrt(i[-n] / 2)
  # Tridiagonal matrix
  J <- matrix(0, n, n)
  diag(J) <- 0
  for (k in seq_len(n - 1L)) {
    J[k, k + 1L] <- b[k]
    J[k + 1L, k] <- b[k]
  }
  eig <- eigen(J, symmetric = TRUE)
  # Physicist's Hermite nodes: multiply by sqrt(2) for N(0,1)
  nodes <- eig$values * sqrt(2)
  # Weights: first component squared, normalized for N(0,1)
  weights <- eig$vectors[1, ]^2
  # Sort by nodes
  ord <- order(nodes)
  list(nodes = nodes[ord], weights = weights[ord])
}

# Package-level cached GH quadrature (lazy initialization)
.gh_cache <- new.env(parent = emptyenv())

#' Get cached Gauss-Hermite nodes/weights for N(0,1)
#' @param n Number of nodes (default 200)
#' @return List with nodes and weights
#' @keywords internal
.get_gh <- function(n = 200L) {
  key <- paste0("gh_", n)
  if (is.null(.gh_cache[[key]])) {
    .gh_cache[[key]] <- .gauss_hermite_normal(n)
  }
  .gh_cache[[key]]
}

#' Approximate correction factor C_g(nu) for GED leverage (diagnostic use only)
#'
#' \eqn{C_g(\nu) = E[|u|] \cdot \textrm{Cov}(z, F_{GED}^{-1}(\Phi(z))) / \sqrt{2/\pi}}.
#' This is a first-order approximation; estimation uses the exact implicit equation.
#' @param nu GED shape parameter (nu > 0)
#' @param n_nodes Number of GH quadrature nodes (default 200)
#' @return Approximate C_g(nu)
#' @keywords internal
correction_factor_ged_approx <- function(nu, n_nodes = 200L) {
  E_abs_u <- ged_E_abs_u(nu)
  gh <- .get_gh(n_nodes)
  u_vals <- qged_std(stats::pnorm(gh$nodes), nu)
  cov_zu <- sum(gh$weights * gh$nodes * u_vals)
  E_abs_u * cov_zu / sqrt(2 / pi)
}


# =========================================================================== #
# Unified leverage estimation — works for all distributions
# =========================================================================== #

#' Compute EH cross-moment for leverage estimation
#'
#' EH = demeaned sample cross-moment \eqn{(1/(T-2)) \sum(|y_t| - \bar{|y|})(y_{t-1} - \bar{y})}.
#' @param y Numeric vector of observations
#' @param rho_type "pearson" or "kendall"
#' @return EH value
#' @keywords internal
.compute_EH <- function(y, rho_type = "pearson") {
  N <- length(y)
  yabs <- abs(y)
  muu <- mean(y[1:(N - 1)])
  mua <- mean(yabs[2:N])
  if (rho_type == "kendall") {
    tau <- kendall_corr(yabs[2:N] - mua, y[1:(N - 1)] - muu)
    EH <- tau * sqrt(stats::var(yabs[2:N]) * stats::var(y[1:(N - 1)]))
  } else {
    EH <- sum((yabs[2:N] - mua) * (y[1:(N - 1)] - muu)) / (N - 2)
  }
  as.numeric(EH)
}

#' Add leverage estimation to a fitted SV(p) model (post-processing)
#' Works for all error distributions: Gaussian, Student-t, GED.
#' - Gaussian: closed form (C_F = 1)
#' - Student-t: closed form with C_t(nu) correction
#' - GED: exact 1D root-finding via uniroot + Gauss-Hermite quadrature
#' @param out Model object from .svp_gaussian/.svp_t/.svp_ged (without leverage)
#' @param y Numeric vector of observations
#' @param p AR order
#' @param rho_type "pearson", "kendall", or "both"
#' @param del Small constant for log transform
#' @param trunc_lev Logical: truncate rho to [-0.999, 0.999]
#' @param wDecay Logical: decaying weights (passed from original estimation)
#' @param errorType "Gaussian", "Student-t", or "GED"
#' @return Updated model object with leverage fields added
#' @keywords internal
.add_leverage <- function(out, y, p, rho_type, del, trunc_lev, wDecay, errorType) {
  y <- as.numeric(y)
  phi <- as.numeric(out$phi)
  sigy <- out$sigy
  sigv <- out$sigv
  nu <- if (!is.null(out$v)) out$v else NULL

  # --- Compute gammatilde (general-p, distribution-free) ---
  rho_w <- as.numeric(stats::ARMAacf(ar = phi, lag.max = p)[-1])
  gamma_w0 <- sigv^2 / (1 - sum(phi * rho_w))
  gammatilde <- gamma_w0 * (1 + rho_w[1])

  # --- Guard against degenerate cases ---
  if (sigv < 1e-10 || sigy < 1e-10) {
    warning("Leverage estimation failed: sigma_v or sigma_y near zero.")
    out$rho <- NA_real_
    out$gammatilde <- gammatilde
    out$leverage <- TRUE
    out$rho_type <- rho_type
    out$trunc_lev <- trunc_lev
    out$theta <- c(out$theta, NA_real_)
    return(out)
  }

  # --- Helper to compute delta from EH ---
  .compute_delta <- function(EH_val) {
    if (errorType == "Gaussian") {
      # Closed form: delta = sqrt(2pi) * EH / (sigv * sigy^2) * exp(-gammatilde/4)
      delta <- (sqrt(2 * pi) * EH_val) / (sigv * sigy^2) * exp(-0.25 * gammatilde)

    } else if (errorType == "Student-t") {
      # Closed form with C_t(nu) correction
      Ct <- correction_factor_t(nu)
      delta <- (sqrt(2 * pi) * EH_val) / (sigv * sigy^2 * Ct) * exp(-0.25 * gammatilde)

    } else if (errorType == "GED") {
      # Exact 1D root-finding via C++:
      # Solve E[g(z + delta*sigv/2)] = EH / (sigy^2 * E[|u|] * exp(gammatilde/4))
      E_abs_u <- ged_E_abs_u(nu)
      target <- EH_val / (sigy^2 * E_abs_u * exp(gammatilde / 4))
      gh <- .get_gh(200L)
      delta <- find_delta_ged_cpp(target, sigv, nu, gh$nodes, gh$weights)
      if (is.na(delta)) {
        warning("GED leverage root-finding failed.")
      }
    }
    delta
  }

  # --- Compute leverage for requested rho_type(s) ---
  if (rho_type == "pearson") {
    EH <- .compute_EH(y, "pearson")
    rho_val <- .compute_delta(EH)
    if (trunc_lev && !is.na(rho_val)) rho_val <- max(-0.999, min(0.999, rho_val))
  } else if (rho_type == "kendall") {
    EH <- .compute_EH(y, "kendall")
    rho_val <- .compute_delta(EH)
    if (trunc_lev && !is.na(rho_val)) rho_val <- max(-0.999, min(0.999, rho_val))
  } else if (rho_type == "both") {
    EH_pea <- .compute_EH(y, "pearson")
    EH_ken <- .compute_EH(y, "kendall")
    rho_val <- .compute_delta(EH_pea)
    rho_ken <- .compute_delta(EH_ken)
    if (trunc_lev && !is.na(rho_val)) rho_val <- max(-0.999, min(0.999, rho_val))
    if (trunc_lev && !is.na(rho_ken)) rho_ken <- max(-0.999, min(0.999, rho_ken))
    out$rho_kendall <- rho_ken
    out$rho_pearson <- rho_val
  } else {
    rho_val <- NA_real_
  }

  # --- Update model object ---
  out$rho <- rho_val
  out$gammatilde <- gammatilde
  out$leverage <- TRUE
  out$rho_type <- rho_type
  out$trunc_lev <- trunc_lev
  out$theta <- c(out$theta, rho_val)

  # Store correction factor for diagnostics
  if (errorType == "Student-t") {
    out$CF <- correction_factor_t(nu)
  } else if (errorType == "GED") {
    out$CF <- correction_factor_ged_approx(nu)  # approximate, for diagnostic reporting
  } else {
    out$CF <- 1
  }

  return(out)
}


# --- Gaussian SV(p) estimation (with optional leverage) ---
.svp_gaussian <- function(y, p, J, leverage, rho_type, del, trunc_lev, wDecay,
                          sigvMethod = "factored") {
  y <- as.numeric(y)
  if (length(y) < 2 * p + J) {
    stop("Time series too short for the given p and J.")
  }
  if (!rho_type %in% c("pearson", "kendall", "both", "none")) {
    stop("rho_type must be one of 'pearson', 'kendall', 'both', 'none'.")
  }
  # Step 1: phi estimation (C++, distribution-free)
  para <- svpCpp_nolev(y, p, J, del, wDecay)
  # Step 2: sigv override by method
  # C++ computes hybrid (factored for p=1, direct for p>=2) by default
  ly2 <- log(y^2 + del)
  ys_g <- ly2 - mean(ly2)
  N_g <- length(ly2)
  var_ly2 <- stats::var(as.numeric(ly2))
  sig_eps2 <- (pi^2) / 2
  phi_g <- as.numeric(para$phi)
  if (sigvMethod == "direct") {
    gam_vec_g <- numeric(p)
    for (k in seq_len(p)) {
      gam_vec_g[k] <- (1 / (N_g - 1)) * sum(ys_g[(k + 1):N_g] * ys_g[1:(N_g - k)])
    }
    sv2 <- var_ly2 - sum(phi_g * gam_vec_g) - sig_eps2
    para$sigv <- sqrt(abs(sv2))
  } else if (sigvMethod == "factored") {
    rho_w_p <- as.numeric(stats::ARMAacf(ar = phi_g, lag.max = p)[-1])
    sv2 <- (var_ly2 - sig_eps2) * (1 - sum(phi_g * rho_w_p))
    para$sigv <- sqrt(abs(sv2))
  }
  # else "hybrid": keep C++ result as-is
  para$theta <- c(para$phi, para$sigy, para$sigv)
  # Metadata
  para$rho <- NA_real_
  para$gammatilde <- NA_real_
  para$y <- y
  para$p <- p
  para$J <- J
  para$leverage <- FALSE
  para$del <- del
  para$wDecay <- wDecay
  para$trunc_lev <- trunc_lev
  para$rho_type <- NA_character_
  para$sigvMethod <- sigvMethod
  class(para) <- "svp"
  # Step 3: leverage post-processing (if requested)
  if (leverage) {
    para <- .add_leverage(para, y, p, rho_type, del, trunc_lev, wDecay, "Gaussian")
  }
  return(para)
}

# --- Student-t SV(p) estimation ---
.svp_t <- function(y, p, J, del, wDecay, logNu,
                   sigvMethod = "factored", winsorize_eps = 0) {
  y <- as.matrix(as.numeric(y))
  N <- nrow(y)
  ly2 <- log(y^2 + del)
  mu <- mean(ly2)
  ys <- ly2 - mu
  # Estimate phi via W-ARMA-SV (distribution-free)
  para_base <- svpCpp_nolev(as.numeric(y), p, J, del, wDecay)
  phi_reg <- as.numeric(para_base$phi)
  var_ly2 <- stats::var(as.numeric(ly2))
  # Estimate sigma_eps^2
  # winsorize_eps: 0 or FALSE = off; TRUE = use J; integer > 0 = use that as J_w
  J_w <- 0L
  if (is.logical(winsorize_eps)) {
    if (winsorize_eps) J_w <- J
  } else if (is.numeric(winsorize_eps) && winsorize_eps > 0) {
    J_w <- as.integer(winsorize_eps)
  }
  if (J_w > 0L) {
    # OLS winsorized estimator (SVHT Eq. 69): average gamma_w(0) across J_w lags
    rho_w_all <- as.numeric(stats::ARMAacf(ar = phi_reg, lag.max = J_w)[-1])
    gam_all <- numeric(J_w)
    for (k in seq_len(J_w)) {
      gam_all[k] <- (1 / (N - 1)) * as.numeric(
        t(ys[(k + 1):N, , drop = FALSE]) %*% ys[1:(N - k), , drop = FALSE])
    }
    gamma_w0_ols <- sum(gam_all * rho_w_all) / sum(rho_w_all^2)
    se2 <- var_ly2 - gamma_w0_ols
  } else {
    # Single-lag estimator (SVHT Eq. 18): sigma_eps^2 = gamma(0) - gamma(1)/rho_w(1)
    rho_w1 <- as.numeric(stats::ARMAacf(ar = phi_reg, lag.max = 1)[2])
    gam1 <- (1 / (N - 1)) * as.numeric(
      t(ys[2:N, , drop = FALSE]) %*% ys[1:(N - 1), , drop = FALSE])
    se2 <- var_ly2 - gam1 / rho_w1
  }
  se2b <- as.numeric(se2) - psigamma(0.5, 1)
  # Estimate nu via root-finding on trigamma: psigamma(nu/2, 1) = se2b (C++)
  nu_lower <- 2.01
  nu_upper <- 500
  nuh <- find_nu_t_cpp(se2b, nu_lower, nu_upper, logNu)
  if (nuh >= nu_upper) {
    warning("Estimated nu at upper boundary (", nu_upper,
            "); tails indistinguishable from Gaussian.")
  } else if (nuh <= nu_lower) {
    warning("Estimated nu at lower boundary (", nu_lower,
            "); extremely heavy tails.")
  }
  # Estimate sigv
  var_log_sq_t <- psigamma(0.5, 1) + psigamma(nuh / 2, 1)
  # Compute sample autocovariances at lags 1,...,p (needed by direct/hybrid for p>=2)
  gam_vec <- numeric(p)
  for (k in seq_len(p)) {
    gam_vec[k] <- (1 / (N - 1)) * as.numeric(
      t(ys[(k + 1):N, , drop = FALSE]) %*% ys[1:(N - k), , drop = FALSE])
  }
  if (sigvMethod == "direct") {
    sv_reg <- sqrt(abs(var_ly2 - sum(phi_reg * gam_vec) - var_log_sq_t))
  } else if (sigvMethod == "factored") {
    rho_w_p <- as.numeric(stats::ARMAacf(ar = phi_reg, lag.max = p)[-1])
    sv_reg <- sqrt(abs((var_ly2 - var_log_sq_t) * (1 - sum(phi_reg * rho_w_p))))
  } else {
    # "hybrid": AD2021 for p=1, direct for p>=2
    if (p == 1L) {
      sv_reg <- sqrt(abs((1 - phi_reg^2) * (var_ly2 - var_log_sq_t)))
    } else {
      sv_reg <- sqrt(abs(var_ly2 - sum(phi_reg * gam_vec) - var_log_sq_t))
    }
  }
  # Estimate sigy
  mu_log_sq_t <- psigamma(0.5, 0) - psigamma(nuh / 2, 0) + log(nuh)
  if (p == 1L) {
    sigy <- sqrt(exp(mean(log(y^2)) - mu_log_sq_t))
  } else {
    sigy <- sqrt(exp(mean(log(y^2 + del)) - mu_log_sq_t))
  }
  theta <- c(phi_reg, sigy, sv_reg, nuh)
  out <- list(mu = mu, phi = phi_reg, sigv = as.numeric(sv_reg),
              sigy = as.numeric(sigy), v = nuh, theta = theta,
              y = as.numeric(y), J = J, p = as.integer(p), del = del,
              wDecay = wDecay, logNu = logNu,
              sigvMethod = sigvMethod, winsorize_eps = winsorize_eps,
              rho = NA_real_, gammatilde = NA_real_,
              leverage = FALSE, rho_type = NA_character_,
              trunc_lev = TRUE,
              nonstationary_ind = isTRUE(para_base$nonstationary_ind))
  class(out) <- "svp_t"
  return(out)
}

# --- GED SV(p) estimation ---
.svp_ged <- function(y, p, J, del, wDecay,
                     sigvMethod = "factored", winsorize_eps = 0) {
  y <- as.matrix(as.numeric(y))
  N <- nrow(y)
  ly2 <- log(y^2 + del)
  mu <- mean(ly2)
  ys <- ly2 - mu
  # Estimate phi (distribution-free)
  para_base <- svpCpp_nolev(as.numeric(y), p, J, del, wDecay)
  phi_reg <- as.numeric(para_base$phi)
  var_ly2 <- stats::var(as.numeric(ly2))
  # Estimate sigma_eps^2
  # winsorize_eps: 0 or FALSE = off; TRUE = use J; integer > 0 = use that as J_w
  J_w <- 0L
  if (is.logical(winsorize_eps)) {
    if (winsorize_eps) J_w <- J
  } else if (is.numeric(winsorize_eps) && winsorize_eps > 0) {
    J_w <- as.integer(winsorize_eps)
  }
  if (J_w > 0L) {
    # OLS winsorized estimator (SVHT Eq. 69)
    rho_w_all <- as.numeric(stats::ARMAacf(ar = phi_reg, lag.max = J_w)[-1])
    gam_all <- numeric(J_w)
    for (k in seq_len(J_w)) {
      gam_all[k] <- (1 / (N - 1)) * as.numeric(
        t(ys[(k + 1):N, , drop = FALSE]) %*% ys[1:(N - k), , drop = FALSE])
    }
    gamma_w0_ols <- sum(gam_all * rho_w_all) / sum(rho_w_all^2)
    se2 <- var_ly2 - gamma_w0_ols
  } else {
    # Single-lag estimator (SVHT Eq. 18)
    rho_w1 <- as.numeric(stats::ARMAacf(ar = phi_reg, lag.max = 1)[2])
    gam1 <- (1 / (N - 1)) * as.numeric(
      t(ys[2:N, , drop = FALSE]) %*% ys[1:(N - 1), , drop = FALSE])
    se2 <- as.numeric(var_ly2 - gam1 / rho_w1)
  }
  # Estimate nu: solve (2/nu)^2 * psigamma(1/nu, 1) = se2 (C++)
  m1 <- sum(abs(ly2)) / N
  m2 <- sqrt(sum((abs(ly2) - m1)^2) / N)
  x0 <- m1 / m2
  lower <- max(1e-6, x0 / 5)
  upper <- x0 * 5
  ged_nuh <- find_nu_ged_cpp(as.numeric(se2), lower, upper)
  if (ged_nuh >= 20) {
    warning("Estimated GED nu at upper boundary (20); tails indistinguishable from Gaussian.")
  } else if (ged_nuh <= 0.1) {
    warning("Estimated GED nu at lower boundary (0.1); extremely heavy tails.")
  }
  # Estimate sigv
  var_log_sq_ged <- ((2 / ged_nuh)^2) * psigamma(1 / ged_nuh, 1)
  gam_vec <- numeric(p)
  for (k in seq_len(p)) {
    gam_vec[k] <- (1 / (N - 1)) * as.numeric(
      t(ys[(k + 1):N, , drop = FALSE]) %*% ys[1:(N - k), , drop = FALSE])
  }
  if (sigvMethod == "direct") {
    sv_reg <- sqrt(abs(var_ly2 - sum(phi_reg * gam_vec) - var_log_sq_ged))
  } else if (sigvMethod == "factored") {
    rho_w_p <- as.numeric(stats::ARMAacf(ar = phi_reg, lag.max = p)[-1])
    sv_reg <- sqrt(abs((var_ly2 - var_log_sq_ged) * (1 - sum(phi_reg * rho_w_p))))
  } else {
    if (p == 1L) {
      sv_reg <- sqrt(abs((1 - phi_reg^2) * (var_ly2 - var_log_sq_ged)))
    } else {
      sv_reg <- sqrt(abs(var_ly2 - sum(phi_reg * gam_vec) - var_log_sq_ged))
    }
  }
  # Estimate sigy
  mu_log_sq_ged <- (2 / ged_nuh) * psigamma(1 / ged_nuh, 0) +
    log(gamma(1 / ged_nuh)) - log(gamma(3 / ged_nuh))
  if (p == 1L) {
    sigy <- sqrt(exp(mean(log(y^2)) - mu_log_sq_ged))
  } else {
    sigy <- sqrt(exp(mean(log(y^2 + del)) - mu_log_sq_ged))
  }
  theta <- c(phi_reg, sigy, sv_reg, ged_nuh)
  out <- list(mu = mu, phi = phi_reg, sigv = as.numeric(sv_reg),
              sigy = as.numeric(sigy), v = ged_nuh, theta = theta,
              y = as.numeric(y), J = J, p = as.integer(p), del = del,
              wDecay = wDecay,
              sigvMethod = sigvMethod, winsorize_eps = winsorize_eps,
              rho = NA_real_, gammatilde = NA_real_,
              leverage = FALSE, rho_type = NA_character_,
              trunc_lev = TRUE,
              nonstationary_ind = isTRUE(para_base$nonstationary_ind))
  class(out) <- "svp_ged"
  return(out)
}








# --- Internal SE helpers ---

.svpSE_gaussian <- function(object, n_sim, alpha, burnin) {
  p <- object$p
  Tsize <- length(object$y)
  has_lev <- isTRUE(object$leverage) && !is.na(object$rho)
  wDecay <- if (is.null(object$wDecay)) FALSE else object$wDecay
  trunc_lev <- if (is.null(object$trunc_lev)) TRUE else object$trunc_lev
  n_params <- if (has_lev) p + 3 else p + 2
  betamat <- matrix(0, n_sim, n_params)
  if (has_lev) {
    betasim <- c(object$phi, object$sigy, object$sigv, object$rho)
    rho_type <- if (is.null(object$rho_type)) "pearson" else object$rho_type
    if (rho_type == "both") rho_type <- "pearson"
  } else {
    betasim <- c(object$phi, object$sigy, object$sigv)
    rho_type <- "pearson"
  }
  xn <- 1
  while (xn <= n_sim) {
    if (has_lev) {
      u_out <- sim_svp(Tsize, phi = object$phi, sigy = object$sigy,
                       sigv = object$sigv, leverage = TRUE,
                       rho = object$rho, burnin = burnin)
      u <- u_out$y
    } else {
      u <- sim_svp(Tsize, phi = object$phi, sigy = object$sigy,
                   sigv = object$sigv, burnin = burnin)$y
    }
    sigvMethod_g <- if (is.null(object$sigvMethod)) "factored" else object$sigvMethod
    out_tmp <- tryCatch(
      svp(u, p = p, J = object$J, leverage = has_lev,
          rho_type = rho_type, del = object$del, trunc_lev = trunc_lev,
          wDecay = wDecay, sigvMethod = sigvMethod_g),
      error = function(e) NULL
    )
    if (!is.null(out_tmp) && !isTRUE(out_tmp$nonstationary_ind)) {
      if (has_lev) {
        betamat[xn, ] <- c(out_tmp$phi, out_tmp$sigy, out_tmp$sigv, out_tmp$rho)
      } else {
        betamat[xn, ] <- c(out_tmp$phi, out_tmp$sigy, out_tmp$sigv)
      }
      xn <- xn + 1
    }
  }
  .compute_se_ci(betamat, betasim, n_sim, n_params, alpha)
}

.svpSE_t <- function(object, n_sim, alpha, burnin, logNu) {
  Tsize <- length(object$y)
  p <- object$p
  has_lev <- isTRUE(object$leverage) && !is.na(object$rho)
  wDecay <- if (is.null(object$wDecay)) FALSE else object$wDecay
  trunc_lev <- if (is.null(object$trunc_lev)) TRUE else object$trunc_lev
  sigvMethod_t <- if (is.null(object$sigvMethod)) "factored" else object$sigvMethod
  winsorize_eps_t <- if (is.null(object$winsorize_eps)) FALSE else object$winsorize_eps
  rho_type <- if (is.null(object$rho_type) || is.na(object$rho_type)) "pearson" else object$rho_type
  if (rho_type == "both") rho_type <- "pearson"
  n_params <- if (has_lev) length(object$phi) + 4 else length(object$phi) + 3
  betamat <- matrix(0, n_sim, n_params)
  if (has_lev) {
    betasim <- c(object$phi, object$sigy, object$sigv, object$v, object$rho)
  } else {
    betasim <- c(object$phi, object$sigy, object$sigv, object$v)
  }
  xn <- 1
  while (xn <= n_sim) {
    if (has_lev) {
      u_out <- sim_svp(Tsize, phi = object$phi, sigy = object$sigy,
                       sigv = object$sigv, errorType = "Student-t",
                       leverage = TRUE, rho = object$rho,
                       nu = object$v, burnin = burnin)
      u <- u_out$y
    } else {
      u <- sim_svp(Tsize, phi = object$phi, sigy = object$sigy,
                   sigv = object$sigv, errorType = "Student-t",
                   nu = object$v, burnin = burnin)$y
    }
    out_tmp <- tryCatch(
      svp(as.numeric(u), p = p, J = object$J,
          errorType = "Student-t", leverage = has_lev,
          rho_type = rho_type, del = object$del, logNu = logNu,
          trunc_lev = trunc_lev, wDecay = wDecay,
          sigvMethod = sigvMethod_t, winsorize_eps = winsorize_eps_t),
      error = function(e) NULL
    )
    if (!is.null(out_tmp) && is.finite(out_tmp$v)) {
      if (has_lev && (is.na(out_tmp$rho) || !is.finite(out_tmp$rho))) next
      if (has_lev) {
        betamat[xn, ] <- c(out_tmp$phi, out_tmp$sigy, out_tmp$sigv,
                           out_tmp$v, out_tmp$rho)
      } else {
        betamat[xn, ] <- c(out_tmp$phi, out_tmp$sigy, out_tmp$sigv, out_tmp$v)
      }
      xn <- xn + 1
    }
  }
  .compute_se_ci(betamat, betasim, n_sim, n_params, alpha)
}

.svpSE_ged <- function(object, n_sim, alpha, burnin) {
  Tsize <- length(object$y)
  p <- object$p
  has_lev <- isTRUE(object$leverage) && !is.na(object$rho)
  wDecay <- if (is.null(object$wDecay)) FALSE else object$wDecay
  trunc_lev <- if (is.null(object$trunc_lev)) TRUE else object$trunc_lev
  sigvMethod_ged <- if (is.null(object$sigvMethod)) "factored" else object$sigvMethod
  winsorize_eps_ged <- if (is.null(object$winsorize_eps)) FALSE else object$winsorize_eps
  rho_type <- if (is.null(object$rho_type) || is.na(object$rho_type)) "pearson" else object$rho_type
  if (rho_type == "both") rho_type <- "pearson"
  n_params <- if (has_lev) length(object$phi) + 4 else length(object$phi) + 3
  betamat <- matrix(0, n_sim, n_params)
  if (has_lev) {
    betasim <- c(object$phi, object$sigy, object$sigv, object$v, object$rho)
  } else {
    betasim <- c(object$phi, object$sigy, object$sigv, object$v)
  }
  xn <- 1
  while (xn <= n_sim) {
    if (has_lev) {
      u_out <- sim_svp(Tsize, phi = object$phi, sigy = object$sigy,
                       sigv = object$sigv, errorType = "GED",
                       leverage = TRUE, rho = object$rho,
                       nu = object$v, burnin = burnin)
      u <- u_out$y
    } else {
      u <- sim_svp(Tsize, phi = object$phi, sigy = object$sigy,
                   sigv = object$sigv, errorType = "GED",
                   nu = object$v, burnin = burnin)$y
    }
    out_tmp <- tryCatch(
      svp(as.numeric(u), p = p, J = object$J,
          errorType = "GED", leverage = has_lev,
          rho_type = rho_type, del = object$del,
          trunc_lev = trunc_lev, wDecay = wDecay,
          sigvMethod = sigvMethod_ged, winsorize_eps = winsorize_eps_ged),
      error = function(e) NULL
    )
    if (!is.null(out_tmp) && is.finite(out_tmp$v)) {
      if (has_lev && (is.na(out_tmp$rho) || !is.finite(out_tmp$rho))) next
      if (has_lev) {
        betamat[xn, ] <- c(out_tmp$phi, out_tmp$sigy, out_tmp$sigv,
                           out_tmp$v, out_tmp$rho)
      } else {
        betamat[xn, ] <- c(out_tmp$phi, out_tmp$sigy, out_tmp$sigv, out_tmp$v)
      }
      xn <- xn + 1
    }
  }
  .compute_se_ci(betamat, betasim, n_sim, n_params, alpha)
}

# Internal helper for SE/CI computation
#' @keywords internal
.compute_se_ci <- function(betamat, betasim, n_sim, n_params, alpha) {
  betasim_mat <- matrix(betasim, nrow = n_sim, ncol = n_params, byrow = TRUE)
  SEsim0 <- sqrt(colSums((betamat - betasim_mat)^2) / (n_sim - n_params))
  SEsim <- sqrt(colSums((betamat - matrix(colMeans(betamat), n_sim, n_params, byrow = TRUE))^2) / (n_sim - n_params))
  ISEconservative <- numeric(n_params)
  ISEliberal <- numeric(n_params)
  CI <- matrix(0, n_params, 2)
  for (xp in seq_len(n_params)) {
    sorted <- sort(betamat[, xp])
    cl <- sorted[max(1, round(n_sim * (alpha / 2)))]
    ch <- sorted[round(n_sim * (1 - alpha / 2))]
    z_alpha <- stats::qnorm(1 - alpha / 2)
    sig_L <- (betasim[xp] - cl) / z_alpha
    sig_H <- (ch - betasim[xp]) / z_alpha
    ISEconservative[xp] <- min(abs(sig_L), abs(sig_H))
    ISEliberal[xp] <- max(abs(sig_L), abs(sig_H))
    CI[xp, 1] <- cl
    CI[xp, 2] <- ch
  }
  list(CI = t(CI), SEsim0 = SEsim0, SEsim = SEsim,
       ISEconservative = ISEconservative, ISEliberal = ISEliberal,
       thetamat = betamat)
}


# =========================================================================== #
# Internal helpers for hypothesis testing
# =========================================================================== #

# --- MMC p-value functions (return negative for minimization) ---

# MMC p-value for AR test
.mmc_pval_ar <- function(theta, y, p_null, p_alt, j, N, s0, ini,
                         del = 1e-10, wDecay = FALSE, Bartlett = FALSE,
                         sigvMethod = "factored",
                         errorType = "Gaussian",
                         nu_fixed = NA_real_,
                         logNu = TRUE,
                         winsorize_eps = 0,
                         innovations = NULL) {
  Tsize <- length(y)
  phi_null <- theta[1:p_null]
  sigy_null <- theta[p_null + 1]
  sigv_null <- theta[p_null + 2]
  stationary <- all(Mod(polyroot(c(1, -phi_null))) > 1)
  if (!stationary || sigy_null <= 0 || sigv_null <= 0) {
    return(9999999999999)
  }
  betasim_null <- c(phi_null, sigy_null, sigv_null)
  sN <- .simnull_ar(betasim_null, p_null, p_alt, j, Tsize, N, ini,
                    del, wDecay, Bartlett, sigvMethod = sigvMethod,
                    errorType = errorType, nu_fixed = nu_fixed,
                    logNu = logNu, winsorize_eps = winsorize_eps,
                    innovations = innovations)
  pval <- -((N + 1 - sum(s0 >= sN)) / (N + 1))
  return(pval)
}

# MMC p-value for leverage test (identity Amat)
# S0 is kept fixed per Dufour (2006, eq 4.22)
.mmc_pval_lev <- function(theta, y, j, N, s0, mdl_alt, rho_null, ini,
                          Amat, rho_type, del = 1e-10, Bartlett = FALSE,
                          wDecay = FALSE, trunc_lev = TRUE,
                          logNu = FALSE, sigvMethod = "factored",
                          winsorize_eps = 0,
                          innovations = NULL) {
  p <- length(mdl_alt$phi)
  Tsize <- length(as.numeric(y))
  stationary <- all(Mod(polyroot(c(1, -theta[1:p]))) > 1)
  if (!stationary || theta[p + 1] <= 0 || theta[p + 2] <= 0) {
    return(9999999999999)
  }
  betasim_null <- c(theta[1:p], theta[p + 1], theta[p + 2], rho_null)
  sN <- .simnull(betasim_null, rho_null, p, j, Tsize, N, ini,
                 Amat, rho_type, del, wDecay = wDecay,
                 trunc_lev = trunc_lev, sigvMethod = sigvMethod,
                 innovations = innovations)
  pval <- -((N + 1 - sum(s0 >= sN)) / (N + 1))
  return(pval)
}

# MMC p-value for leverage test (Bartlett kernel Amat)
# S0 is kept fixed per Dufour (2006, eq 4.22)
.mmc_pval_lev_Amat <- function(theta, y, j, N, s0, mdl_alt, rho_null, ini, innovations = NULL,
                               Amat = NULL, rho_type, del = 1e-10, Bartlett = TRUE,
                               wDecay = FALSE, trunc_lev = TRUE,
                               logNu = FALSE, sigvMethod = "factored",
                               winsorize_eps = 0) {
  p <- length(mdl_alt$phi)
  Tsize <- length(as.numeric(y))
  stationary <- all(Mod(polyroot(c(1, -theta[1:p]))) > 1)
  if (!stationary || theta[p + 1] <= 0 || theta[p + 2] <= 0) {
    return(9999999999999)
  }
  betasim_null <- c(theta[1:p], theta[p + 1], theta[p + 2], rho_null)
  sN <- .simnull_Amat(betasim_null, rho_null, p, j, Tsize, N, ini,
                      rho_type, del, Bartlett, wDecay = wDecay,
                      trunc_lev = trunc_lev, sigvMethod = sigvMethod,
                      innovations = innovations)
  pval <- -((N + 1 - sum(s0 >= sN)) / (N + 1))
  return(pval)
}

# MMC p-value for Student-t leverage test (returns negative for minimization)
# theta = (phi_1,...,phi_p, sigy, sigv, nu) — nuisance under H0: rho = rho_null
# S0 is kept fixed per Dufour (2006, eq 4.22)
.mmc_pval_lev_t <- function(theta, y, j, N, s0, mdl_alt, rho_null, ini, innovations = NULL,
                             Amat, rho_type, del = 1e-10, Bartlett = FALSE,
                             wDecay = FALSE, trunc_lev = TRUE,
                             logNu = FALSE, sigvMethod = "factored",
                             winsorize_eps = FALSE) {
  p <- length(mdl_alt$phi)
  Tsize <- length(as.numeric(y))
  phi_cand <- theta[1:p]
  sigy_cand <- theta[p + 1]
  sigv_cand <- theta[p + 2]
  nu_cand <- theta[p + 3]
  stationary <- all(Mod(polyroot(c(1, -phi_cand))) > 1)
  if (!stationary || sigy_cand <= 0 || sigv_cand <= 0 || nu_cand <= 2) {
    return(9999999999999)
  }
  betasim_null <- c(phi_cand, sigy_cand, sigv_cand, nu_cand, rho_null)
  if (isTRUE(Bartlett)) {
    sN <- .simnull_lev_t_Amat(betasim_null, rho_null, p, j, Tsize, N, ini,
                               rho_type, del, TRUE, wDecay = wDecay,
                               trunc_lev = trunc_lev, logNu = logNu,
                               sigvMethod = sigvMethod,
                               winsorize_eps = winsorize_eps,
                               innovations = innovations)
  } else {
    sN <- .simnull_lev_t(betasim_null, rho_null, p, j, Tsize, N, ini,
                          Amat, rho_type, del, wDecay = wDecay,
                          trunc_lev = trunc_lev, logNu = logNu,
                          sigvMethod = sigvMethod,
                          winsorize_eps = winsorize_eps,
                          innovations = innovations)
  }
  pval <- -((N + 1 - sum(s0 >= sN)) / (N + 1))
  return(pval)
}

# MMC p-value for GED leverage test (returns negative for minimization)
# theta = (phi_1,...,phi_p, sigy, sigv, nu) — nuisance under H0: rho = rho_null
# S0 is kept fixed per Dufour (2006, eq 4.22)
.mmc_pval_lev_ged <- function(theta, y, j, N, s0, mdl_alt, rho_null, ini, innovations = NULL,
                               Amat, rho_type, del = 1e-10, Bartlett = FALSE,
                               wDecay = FALSE, trunc_lev = TRUE,
                               sigvMethod = "factored",
                               winsorize_eps = FALSE) {
  p <- length(mdl_alt$phi)
  phi_cand <- theta[1:p]
  sigy_cand <- theta[p + 1]
  sigv_cand <- theta[p + 2]
  nu_cand <- theta[p + 3]
  stationary <- all(Mod(polyroot(c(1, -phi_cand))) > 1)
  if (!stationary || sigy_cand <= 0 || sigv_cand <= 0 || nu_cand <= 0) {
    return(9999999999999)
  }
  Tsize <- length(as.numeric(y))
  betasim_null <- c(phi_cand, sigy_cand, sigv_cand, nu_cand, rho_null)
  if (isTRUE(Bartlett)) {
    sN <- .simnull_lev_ged_Amat(betasim_null, rho_null, p, j, Tsize, N, ini,
                                 rho_type, del, TRUE, wDecay = wDecay,
                                 trunc_lev = trunc_lev,
                                 sigvMethod = sigvMethod,
                                 winsorize_eps = winsorize_eps,
                                 innovations = innovations)
  } else {
    sN <- .simnull_lev_ged(betasim_null, rho_null, p, j, Tsize, N, ini,
                            Amat, rho_type, del, wDecay = wDecay,
                            trunc_lev = trunc_lev,
                            sigvMethod = sigvMethod,
                            winsorize_eps = winsorize_eps,
                            innovations = innovations)
  }
  pval <- -((N + 1 - sum(s0 >= sN)) / (N + 1))
  return(pval)
}

# MMC p-value for Student-t (returns negative for minimization)
# S0 is kept fixed per Dufour (2006, eq 4.22)
.mmc_pval_t <- function(theta, y, j, N, s0, mdl_alt, nu_null, ini,
                        Amat, WAmat = FALSE, del = 1e-10, Bartlett = TRUE,
                        logNu = TRUE, wDecay = FALSE,
                        sigvMethod = "factored", winsorize_eps = 0,
                        direction = "two-sided",
                        innovations = NULL) {
  p <- length(mdl_alt$phi)
  Tsize <- length(as.numeric(y))
  stationary <- all(Mod(polyroot(c(1, -theta[1:p]))) > 1)
  if (!stationary || theta[p + 1] <= 0 || theta[p + 2] <= 0) {
    return(9999999999999)
  }
  betasim_null <- c(theta[1:p], theta[p + 1], theta[p + 2], nu_null)
  sN <- .simnull_t(betasim_null, nu_null, j, Tsize, N, ini,
                   Amat, del, WAmat, Bartlett, logNu,
                   wDecay = wDecay, sigvMethod = sigvMethod,
                   winsorize_eps = winsorize_eps,
                   direction = direction,
                   innovations = innovations)
  if (direction == "two-sided") {
    pval <- -((N + 1 - sum(s0 >= sN)) / (N + 1))
  } else {
    S_obs <- .signed_root(s0, mdl_alt$v, nu_null)
    pval <- -.pvalue_directional(S_obs, sN, direction)
  }
  return(pval)
}

# MMC p-value for GED (returns negative for minimization)
# S0 is kept fixed per Dufour (2006, eq 4.22)
.mmc_pval_ged <- function(theta, y, j, N, s0, mdl_alt, nu_null, ini,
                          Amat, WAmat = FALSE, del = 1e-10, Bartlett = TRUE,
                          wDecay = FALSE, sigvMethod = "factored",
                          winsorize_eps = 0,
                          innovations = NULL,
                          direction = "two-sided") {
  p <- length(mdl_alt$phi)
  Tsize <- length(as.numeric(y))
  stationary <- all(Mod(polyroot(c(1, -theta[1:p]))) > 1)
  if (!stationary || theta[p + 1] <= 0 || theta[p + 2] <= 0) {
    return(9999999999999)
  }
  betasim_null <- c(theta[1:p], theta[p + 1], theta[p + 2], nu_null)
  sN <- .simnull_ged(betasim_null, nu_null, j, Tsize, N, ini,
                     Amat, del, WAmat, Bartlett, wDecay = wDecay,
                     sigvMethod = sigvMethod,
                     winsorize_eps = winsorize_eps,
                     direction = direction,
                     innovations = innovations)
  if (direction == "two-sided") {
    pval <- -((N + 1 - sum(s0 >= sN)) / (N + 1))
  } else {
    S_obs <- .signed_root(s0, mdl_alt$v, nu_null)
    pval <- -.pvalue_directional(S_obs, sN, direction)
  }
  return(pval)
}

# --- Amat parsing and MMC optimizer dispatch ---

# Resolve Amat + Bartlett into a unified weighting specification.
# Amat takes precedence: "Weighted" -> HAC, <matrix> -> user-supplied, NULL -> check Bartlett.
# Bartlett = TRUE without Amat is backward-compat shorthand for Amat = "Weighted".
.resolve_weighting <- function(Amat, Bartlett) {
  if (!is.null(Amat)) {
    if (identical(Amat, "Weighted")) {
      return(list(Amat = "Weighted", use_hac = TRUE))
    } else if (is.matrix(Amat)) {
      return(list(Amat = Amat, use_hac = FALSE))
    } else {
      stop("Amat must be NULL, 'Weighted', or a numeric matrix.")
    }
  }
  if (isTRUE(Bartlett)) {
    return(list(Amat = "Weighted", use_hac = TRUE))
  }
  list(Amat = NULL, use_hac = FALSE)
}

# Parse Amat argument (shared by t, GED, and leverage tests)
# n_mom: number of moment conditions (p+3 for non-leverage heavy-tail, p+4 for leverage heavy-tail)
.parse_Amat <- function(Amat, p, n_mom = NULL) {
  if (is.null(n_mom)) n_mom <- p + 3
  WAmat <- FALSE
  if (is.null(Amat)) {
    Amat <- diag(n_mom)
  } else if (identical(Amat, "Weighted")) {
    WAmat <- TRUE
    Amat <- diag(n_mom)  # placeholder; overridden inside moment function
  } else if (!is.matrix(Amat) || !all(dim(Amat) == c(n_mom, n_mom))) {
    stop("Amat must be NULL, 'Weighted', or a (", n_mom, "x", n_mom, ") matrix.")
  }
  list(Amat = Amat, WAmat = WAmat)
}

# Generic MMC optimizer dispatch
.run_mmc_optimizer <- function(method, theta_0, fn, lower, upper,
                               threshold, maxit, ...) {
  if (method == "GenSA") {
    if (!requireNamespace("GenSA", quietly = TRUE)) {
      stop("Package 'GenSA' required. Install it or use method='pso'.")
    }
    if (is.null(maxit)) maxit <- 100
    out <- GenSA::GenSA(theta_0, fn, lower = lower, upper = upper,
                        control = list(threshold.stop = -threshold,
                                       max.call = maxit, verbose = TRUE),
                        ...)
  } else if (method == "pso") {
    if (!requireNamespace("pso", quietly = TRUE)) {
      stop("Package 'pso' required. Install it or use method='GenSA'.")
    }
    if (is.null(maxit)) maxit <- 100
    out <- pso::psoptim(theta_0, fn, lower = lower, upper = upper,
                        control = list(abstol = -threshold,
                                       maxf = maxit, trace = 1,
                                       REPORT = 1, trace.stats = TRUE),
                        ...)
  } else {
    stop("method must be 'pso' or 'GenSA'.")
  }
  return(out)
}

# --- GMM moment functions ---

# Pseudo-inverse via SVD
.pinv <- function(A, tol = .Machine$double.eps^0.5) {
  s <- svd(A)
  d <- s$d
  d[d > tol] <- 1 / d[d > tol]
  d[d <= tol] <- 0
  s$v %*% diag(d, nrow = length(d)) %*% t(s$u)
}

#' @keywords internal
LRT_moment_lev_svp <- function(y, mdl_out, Amat, rho_type, del = 1e-10) {
  LRT_moment_lev_svp_cpp(as.numeric(y), mdl_out, Amat, rho_type, del)
}

#' @keywords internal
LRT_moment_lev_svp_Amat <- function(y, mdl_out, rho_type, del = 1e-10,
                                     Bartlett = TRUE) {
  LRT_moment_lev_svp_Amat_cpp(as.numeric(y), mdl_out, rho_type, del,
                                Bartlett, pinv_fn = .pinv)
}

#' @keywords internal
LRT_moment_ar_Amat <- function(y, mdl_out, del = 1e-10, Bartlett = TRUE) {
  LRT_moment_ar_Amat_cpp(as.numeric(y), mdl_out, del, Bartlett, pinv_fn = .pinv)
}

#' @keywords internal
LRT_moment_t <- function(y, mdl_out, Amat, WAmat = FALSE, del = 1e-10,
                         Bartlett = TRUE) {
  LRT_moment_t_cpp(as.numeric(y), mdl_out, Amat, WAmat, del, Bartlett,
                    pinv_fn = .pinv)
}

#' @keywords internal
LRT_moment_ged <- function(y, mdl_out, Amat, WAmat = FALSE, del = 1e-10,
                           Bartlett = TRUE) {
  LRT_moment_ged_cpp(as.numeric(y), mdl_out, Amat, WAmat, del, Bartlett,
                      pinv_fn = .pinv)
}


# =========================================================================== #
# GMM moment functions for leverage + heavy-tail testing (p+4 conditions)
# =========================================================================== #

#' GMM moments for SVL(p)-Student-t with fixed A matrix (p+4 conditions)
#' @keywords internal
LRT_moment_lev_t <- function(y, mdl_out, Amat, rho_type, del = 1e-10) {
  LRT_moment_lev_t_cpp(as.numeric(y), mdl_out, Amat, rho_type, del)
}

#' GMM moments for SVL(p)-Student-t with HAC weighting (p+4 conditions)
#' @keywords internal
LRT_moment_lev_t_Amat <- function(y, mdl_out, rho_type, del = 1e-10,
                                   Bartlett = TRUE) {
  LRT_moment_lev_t_Amat_cpp(as.numeric(y), mdl_out, rho_type, del,
                              Bartlett, pinv_fn = .pinv)
}

#' GMM moments for SVL(p)-GED with fixed A matrix (p+4 conditions)
#' Uses exact GED leverage moment.
#' @keywords internal
LRT_moment_lev_ged <- function(y, mdl_out, Amat, rho_type, del = 1e-10) {
  gh <- .get_gh(200L)
  LRT_moment_lev_ged_cpp(as.numeric(y), mdl_out, Amat, rho_type, del,
                          gh_nodes = gh$nodes, gh_weights = gh$weights)
}

#' GMM moments for SVL(p)-GED with HAC weighting (p+4 conditions)
#' @keywords internal
LRT_moment_lev_ged_Amat <- function(y, mdl_out, rho_type, del = 1e-10,
                                     Bartlett = TRUE) {
  gh <- .get_gh(200L)
  LRT_moment_lev_ged_Amat_cpp(as.numeric(y), mdl_out, rho_type, del,
                                Bartlett, pinv_fn = .pinv,
                                gh_nodes = gh$nodes, gh_weights = gh$weights)
}



# =========================================================================== #
# Simulation-under-null helpers
# =========================================================================== #

# Simulate null distribution for AR test
.simnull_ar <- function(betasim_null, p_null, p_alt, j, Tsize, N, ini,
                        del = 1e-10, wDecay = FALSE, Bartlett = FALSE,
                        sigvMethod = "factored",
                        errorType = "Gaussian",
                        nu_fixed = NA_real_,
                        logNu = TRUE,
                        winsorize_eps = 0,
                        innovations = NULL) {
  phi_sim <- betasim_null[seq_len(p_null)]
  sigy_sim <- betasim_null[p_null + 1]
  sigv_sim <- betasim_null[p_null + 2]
  n_total <- ini + Tsize

  # Pre-draw innovations if not supplied (errorType-specific eps distribution)
  if (is.null(innovations)) {
    N_draw <- ceiling(N * 1.5) + 10L
    innovations <- list(
      eta = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw),
      eps = if (errorType == "Gaussian") {
        matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw)
      } else if (errorType == "Student-t") {
        matrix(rt(n_total * N_draw, df = nu_fixed), nrow = n_total, ncol = N_draw)
      } else {  # GED
        matrix(rged_cpp(n_total * N_draw, 0, 1, nu_fixed), nrow = n_total, ncol = N_draw)
      }
    )
  }

  # Helper closures for distribution-specific simulation, fitting, and stat computation
  sim_one <- function(eta_vec, eps_vec) {
    if (errorType == "Gaussian") {
      sim_from_innov_gaussian_cpp(phi = phi_sim, sigma_y = sigy_sim,
                                  sigma_v = sigv_sim,
                                  eta_vec = eta_vec, eps_vec = eps_vec,
                                  p = p_null, T_out = Tsize, burnin = ini)
    } else if (errorType == "Student-t") {
      sim_from_innov_t_cpp(phi = phi_sim, sigma_y = sigy_sim,
                           sigma_v = sigv_sim,
                           eta_vec = eta_vec, eps_vec = eps_vec,
                           p = p_null, T_out = Tsize, burnin = ini)
    } else {
      sim_from_innov_ged_cpp(phi = phi_sim, sigma_y = sigy_sim,
                             sigma_v = sigv_sim,
                             eta_vec = eta_vec, eps_vec = eps_vec,
                             p = p_null, T_out = Tsize, burnin = ini)
    }
  }
  fit_one <- function(u, p_use) {
    if (errorType == "Gaussian") {
      svp(u, p = p_use, J = j, leverage = FALSE, del = del, wDecay = wDecay,
          sigvMethod = sigvMethod)
    } else {
      svp(u, p = p_use, J = j, leverage = FALSE, errorType = errorType,
          del = del, wDecay = wDecay, logNu = logNu, sigvMethod = sigvMethod,
          winsorize_eps = winsorize_eps)
    }
  }
  stat_bartlett <- function(u_vec, mdl_null_tmp, mdl_alt_tmp) {
    if (errorType == "Gaussian") {
      M_n <- LRT_moment_ar_Amat(u_vec, mdl_null_tmp, del = del, Bartlett = TRUE)
      M_a <- LRT_moment_ar_Amat(u_vec, mdl_alt_tmp, del = del, Bartlett = TRUE)
    } else {
      u_mat <- as.matrix(u_vec)
      wa_n <- .parse_Amat("Weighted", p_null)
      wa_a <- .parse_Amat("Weighted", p_alt)
      if (errorType == "Student-t") {
        M_n <- LRT_moment_t(u_mat, mdl_null_tmp, wa_n$Amat, wa_n$WAmat, del, TRUE)
        M_a <- LRT_moment_t(u_mat, mdl_alt_tmp, wa_a$Amat, wa_a$WAmat, del, TRUE)
      } else {  # GED
        M_n <- LRT_moment_ged(u_mat, mdl_null_tmp, wa_n$Amat, wa_n$WAmat, del, TRUE)
        M_a <- LRT_moment_ged(u_mat, mdl_alt_tmp, wa_a$Amat, wa_a$WAmat, del, TRUE)
      }
    }
    Tsize * (M_n - M_a)
  }

  sN <- numeric(N)
  xn <- 1
  b <- 1
  while (xn <= N && b <= ncol(innovations$eta)) {
    u <- as.numeric(sim_one(innovations$eta[, b], innovations$eps[, b]))
    mdl_alt_tmp <- tryCatch(fit_one(u, p_alt), error = function(e) NULL)
    b <- b + 1
    if (is.null(mdl_alt_tmp) || isTRUE(mdl_alt_tmp$nonstationary_ind)) next
    if (isTRUE(Bartlett)) {
      mdl_null_tmp <- tryCatch(fit_one(u, p_null), error = function(e) NULL)
      if (is.null(mdl_null_tmp) || isTRUE(mdl_null_tmp$nonstationary_ind)) next
      stat <- tryCatch(stat_bartlett(u, mdl_null_tmp, mdl_alt_tmp),
                       error = function(e) NA_real_)
      if (is.na(stat)) next
      sN[xn] <- max(stat, 1e-10)
    } else {
      phi_extra <- mdl_alt_tmp$phi[(p_null + 1):p_alt]
      sN[xn] <- Tsize * sum(phi_extra^2)
    }
    xn <- xn + 1
  }
  # Fallback: draw fresh innovations if pre-drawn pool exhausted
  while (xn <= N) {
    eta_extra <- rnorm(n_total)
    eps_extra <- if (errorType == "Gaussian") rnorm(n_total)
                 else if (errorType == "Student-t") rt(n_total, df = nu_fixed)
                 else rged_cpp(n_total, 0, 1, nu_fixed)
    u <- as.numeric(sim_one(eta_extra, eps_extra))
    mdl_alt_tmp <- tryCatch(fit_one(u, p_alt), error = function(e) NULL)
    if (is.null(mdl_alt_tmp) || isTRUE(mdl_alt_tmp$nonstationary_ind)) next
    if (isTRUE(Bartlett)) {
      mdl_null_tmp <- tryCatch(fit_one(u, p_null), error = function(e) NULL)
      if (is.null(mdl_null_tmp) || isTRUE(mdl_null_tmp$nonstationary_ind)) next
      stat <- tryCatch(stat_bartlett(u, mdl_null_tmp, mdl_alt_tmp),
                       error = function(e) NA_real_)
      if (is.na(stat)) next
      sN[xn] <- max(stat, 1e-10)
    } else {
      phi_extra <- mdl_alt_tmp$phi[(p_null + 1):p_alt]
      sN[xn] <- Tsize * sum(phi_extra^2)
    }
    xn <- xn + 1
  }
  attr(sN, "innovations") <- innovations
  return(sN)
}

# Simulate null distribution for leverage test (identity Amat)
.simnull <- function(betasim_null, rho_null, p, j, Tsize, N, ini,
                     Amat, rho_type, del = 1e-10,
                     wDecay = FALSE, trunc_lev = TRUE,
                     sigvMethod = "factored",
                     innovations = NULL) {
  phi_sim <- betasim_null[seq_len(p)]
  sigy_sim <- betasim_null[p + 1]
  sigv_sim <- betasim_null[p + 2]
  rho_sim <- betasim_null[p + 3]
  n_total <- ini + Tsize

  if (is.null(innovations)) {
    N_draw <- ceiling(N * 1.5) + 10L
    innovations <- list(
      zeta = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw),
      aux  = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw)
    )
  }

  sN <- numeric(N)
  xn <- 1
  b <- 1
  max_iter <- ncol(innovations$zeta)
  while (xn <= N && b <= max_iter) {
    u <- as.numeric(sim_from_innov_gaussian_lev_cpp(
      phi = phi_sim, sigma_y = sigy_sim, sigma_v = sigv_sim, rho = rho_sim,
      zeta_vec = innovations$zeta[, b], aux_vec = innovations$aux[, b],
      p = p, T_out = Tsize, burnin = ini
    ))
    b <- b + 1
    out_tmp <- tryCatch(
      svp(u, p, j, leverage = TRUE, rho_type = rho_type, del = del,
          trunc_lev = trunc_lev, wDecay = wDecay, sigvMethod = sigvMethod),
      error = function(e) NULL
    )
    if (!is.null(out_tmp) && abs(out_tmp$rho) <= 1 &&
        !isTRUE(out_tmp$nonstationary_ind)) {
      out_null_tmp <- out_tmp
      out_null_tmp$rho <- rho_null
      u_mat <- as.matrix(u)
      sN_tmp <- Tsize * (LRT_moment_lev_svp(u_mat, out_null_tmp, Amat, rho_type, del) -
                           LRT_moment_lev_svp(u_mat, out_tmp, Amat, rho_type, del))
      if (!is.na(sN_tmp)) {
        sN[xn] <- max(sN_tmp, 1e-10)
        xn <- xn + 1
      }
    }
  }
  # Fallback
  while (xn <= N) {
    u <- as.numeric(sim_from_innov_gaussian_lev_cpp(
      phi = phi_sim, sigma_y = sigy_sim, sigma_v = sigv_sim, rho = rho_sim,
      zeta_vec = rnorm(n_total), aux_vec = rnorm(n_total),
      p = p, T_out = Tsize, burnin = ini
    ))
    out_tmp <- tryCatch(
      svp(u, p, j, leverage = TRUE, rho_type = rho_type, del = del,
          trunc_lev = trunc_lev, wDecay = wDecay, sigvMethod = sigvMethod),
      error = function(e) NULL
    )
    if (!is.null(out_tmp) && abs(out_tmp$rho) <= 1 &&
        !isTRUE(out_tmp$nonstationary_ind)) {
      out_null_tmp <- out_tmp
      out_null_tmp$rho <- rho_null
      u_mat <- as.matrix(u)
      sN_tmp <- Tsize * (LRT_moment_lev_svp(u_mat, out_null_tmp, Amat, rho_type, del) -
                           LRT_moment_lev_svp(u_mat, out_tmp, Amat, rho_type, del))
      if (!is.na(sN_tmp)) {
        sN[xn] <- max(sN_tmp, 1e-10)
        xn <- xn + 1
      }
    }
  }
  attr(sN, "innovations") <- innovations
  return(sN)
}

# Simulate null distribution for leverage test (Bartlett kernel Amat)
.simnull_Amat <- function(betasim_null, rho_null, p, j, Tsize, N, ini,
                          rho_type, del = 1e-10, Bartlett = TRUE,
                          wDecay = FALSE, trunc_lev = TRUE,
                          sigvMethod = "factored",
                          innovations = NULL) {
  phi_sim <- betasim_null[seq_len(p)]
  sigy_sim <- betasim_null[p + 1]
  sigv_sim <- betasim_null[p + 2]
  rho_sim <- betasim_null[p + 3]
  n_total <- ini + Tsize

  if (is.null(innovations)) {
    N_draw <- ceiling(N * 1.5) + 10L
    innovations <- list(
      zeta = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw),
      aux  = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw)
    )
  }

  sN <- numeric(N)
  xn <- 1
  b <- 1
  max_iter <- ncol(innovations$zeta)
  while (xn <= N && b <= max_iter) {
    u <- as.numeric(sim_from_innov_gaussian_lev_cpp(
      phi = phi_sim, sigma_y = sigy_sim, sigma_v = sigv_sim, rho = rho_sim,
      zeta_vec = innovations$zeta[, b], aux_vec = innovations$aux[, b],
      p = p, T_out = Tsize, burnin = ini
    ))
    b <- b + 1
    out_tmp <- tryCatch(
      svp(u, p, j, leverage = TRUE, rho_type = rho_type, del = del,
          trunc_lev = trunc_lev, wDecay = wDecay, sigvMethod = sigvMethod),
      error = function(e) NULL
    )
    if (!is.null(out_tmp) && abs(out_tmp$rho) <= 1 &&
        !isTRUE(out_tmp$nonstationary_ind)) {
      out_null_tmp <- out_tmp
      out_null_tmp$rho <- rho_null
      u_mat <- as.matrix(u)
      sN_tmp <- Tsize * (LRT_moment_lev_svp_Amat(u_mat, out_null_tmp, rho_type, del, Bartlett) -
                           LRT_moment_lev_svp_Amat(u_mat, out_tmp, rho_type, del, Bartlett))
      if (!is.na(sN_tmp)) {
        sN[xn] <- max(sN_tmp, 1e-10)
        xn <- xn + 1
      }
    }
  }
  while (xn <= N) {
    u <- as.numeric(sim_from_innov_gaussian_lev_cpp(
      phi = phi_sim, sigma_y = sigy_sim, sigma_v = sigv_sim, rho = rho_sim,
      zeta_vec = rnorm(n_total), aux_vec = rnorm(n_total),
      p = p, T_out = Tsize, burnin = ini
    ))
    out_tmp <- tryCatch(
      svp(u, p, j, leverage = TRUE, rho_type = rho_type, del = del,
          trunc_lev = trunc_lev, wDecay = wDecay, sigvMethod = sigvMethod),
      error = function(e) NULL
    )
    if (!is.null(out_tmp) && abs(out_tmp$rho) <= 1 &&
        !isTRUE(out_tmp$nonstationary_ind)) {
      out_null_tmp <- out_tmp
      out_null_tmp$rho <- rho_null
      u_mat <- as.matrix(u)
      sN_tmp <- Tsize * (LRT_moment_lev_svp_Amat(u_mat, out_null_tmp, rho_type, del, Bartlett) -
                           LRT_moment_lev_svp_Amat(u_mat, out_tmp, rho_type, del, Bartlett))
      if (!is.na(sN_tmp)) {
        sN[xn] <- max(sN_tmp, 1e-10)
        xn <- xn + 1
      }
    }
  }
  attr(sN, "innovations") <- innovations
  return(sN)
}

# Simulate null distribution for Student-t test
# Signed root: S = sign(theta_hat - theta_0) * sqrt(max(LR, 0))
.signed_root <- function(LR_T, theta_hat, theta_0) {
  sign(theta_hat - theta_0) * sqrt(pmax(LR_T, 0))
}

# Directional p-value
.pvalue_directional <- function(S_obs, S_sim, direction) {
  N <- length(S_sim)
  switch(direction,
    "less"    = (1 + sum(S_sim <= S_obs)) / (N + 1),
    "greater" = (1 + sum(S_sim >= S_obs)) / (N + 1)
  )
}

.simnull_t <- function(betasim_null, nu_null, j, Tsize, N, ini,
                       Amat, del = 1e-10, WAmat = FALSE,
                       Bartlett = TRUE, logNu = TRUE,
                       wDecay = FALSE, sigvMethod = "factored",
                       winsorize_eps = 0,
                       direction = "two-sided",
                       innovations = NULL) {
  p <- length(betasim_null) - 3
  phi_sim <- betasim_null[1:p]
  sigy_sim <- betasim_null[p + 1]
  sigv_sim <- betasim_null[p + 2]
  nu_sim <- betasim_null[p + 3]
  n_total <- ini + Tsize

  # Pre-draw innovations if not provided
  # When innovations is NULL: draw fresh (standard LMC or standalone MMC)
  # When innovations is provided: reuse (fixed-error MMC, Dufour 2006 eq 4.22)
  if (is.null(innovations)) {
    # Draw extra columns to handle estimation failures (need >= N valid)
    N_draw <- ceiling(N * 1.5) + 10L
    innovations <- list(
      eta = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw),
      eps = matrix(rt(n_total * N_draw, df = nu_sim), nrow = n_total, ncol = N_draw)
    )
  }

  sN <- numeric(N)
  sign_vec <- numeric(N)
  xn <- 1
  b <- 1  # innovation index
  while (xn <= N && b <= ncol(innovations$eta)) {
    # Simulate from pre-drawn innovations
    u <- as.matrix(sim_from_innov_t_cpp(
      phi = phi_sim, sigma_y = sigy_sim, sigma_v = sigv_sim,
      eta_vec = innovations$eta[, b], eps_vec = innovations$eps[, b],
      p = p, T_out = Tsize, burnin = ini
    ))
    out_tmp <- tryCatch(
      svp(as.numeric(u), p = p, J = j, errorType = "Student-t", del = del,
          logNu = logNu, wDecay = wDecay, sigvMethod = sigvMethod,
          winsorize_eps = winsorize_eps),
      error = function(e) NULL
    )
    if (!is.null(out_tmp)) {
      out_null_tmp <- out_tmp
      out_null_tmp$v <- nu_null
      sN_tmp <- tryCatch(
        Tsize * (LRT_moment_t(u, out_null_tmp, Amat, WAmat, del, Bartlett) -
                   LRT_moment_t(u, out_tmp, Amat, WAmat, del, Bartlett)),
        error = function(e) NULL
      )
      if (!is.null(sN_tmp) && !is.na(sN_tmp)) {
        sN[xn] <- max(sN_tmp, 1e-10)
        sign_vec[xn] <- sign(out_tmp$v - nu_null)
        xn <- xn + 1
      }
    }
    b <- b + 1
  }
  # If we ran out of innovations, draw more (fallback)
  while (xn <= N) {
    eta_extra <- rnorm(n_total)
    eps_extra <- rt(n_total, df = nu_sim)
    u <- as.matrix(sim_from_innov_t_cpp(
      phi = phi_sim, sigma_y = sigy_sim, sigma_v = sigv_sim,
      eta_vec = eta_extra, eps_vec = eps_extra,
      p = p, T_out = Tsize, burnin = ini
    ))
    out_tmp <- tryCatch(
      svp(as.numeric(u), p = p, J = j, errorType = "Student-t", del = del,
          logNu = logNu, wDecay = wDecay, sigvMethod = sigvMethod,
          winsorize_eps = winsorize_eps),
      error = function(e) NULL
    )
    if (!is.null(out_tmp)) {
      out_null_tmp <- out_tmp
      out_null_tmp$v <- nu_null
      sN_tmp <- tryCatch(
        Tsize * (LRT_moment_t(u, out_null_tmp, Amat, WAmat, del, Bartlett) -
                   LRT_moment_t(u, out_tmp, Amat, WAmat, del, Bartlett)),
        error = function(e) NULL
      )
      if (!is.null(sN_tmp) && !is.na(sN_tmp)) {
        sN[xn] <- max(sN_tmp, 1e-10)
        sign_vec[xn] <- sign(out_tmp$v - nu_null)
        xn <- xn + 1
      }
    }
  }

  out <- if (direction != "two-sided") sign_vec * sqrt(sN) else sN
  attr(out, "innovations") <- innovations
  return(out)
}

# Simulate null distribution for GED test
.simnull_ged <- function(betasim_null, nu_null, j, Tsize, N, ini,
                         Amat, del = 1e-10, WAmat = FALSE,
                         Bartlett = TRUE, wDecay = FALSE,
                         sigvMethod = "factored", winsorize_eps = 0,
                         direction = "two-sided",
                         innovations = NULL) {
  p <- length(betasim_null) - 3
  phi_sim <- betasim_null[1:p]
  sigy_sim <- betasim_null[p + 1]
  sigv_sim <- betasim_null[p + 2]
  nu_sim <- betasim_null[p + 3]
  n_total <- ini + Tsize

  # Pre-draw innovations if not provided
  if (is.null(innovations)) {
    N_draw <- ceiling(N * 1.5) + 10L
    a_ged <- exp(0.5 * (lgamma(1 / nu_sim) - lgamma(3 / nu_sim)))
    total_n <- n_total * N_draw
    x_gam_all <- rgamma(total_n, shape = 1 / nu_sim, rate = 1)
    eps_all <- sign(runif(total_n) - 0.5) * a_ged * x_gam_all^(1 / nu_sim)
    innovations <- list(
      eta = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw),
      eps = matrix(eps_all, nrow = n_total, ncol = N_draw)
    )
  }

  sN <- numeric(N)
  sign_vec <- numeric(N)
  xn <- 1
  b <- 1
  while (xn <= N && b <= ncol(innovations$eta)) {
    u <- as.matrix(sim_from_innov_ged_cpp(
      phi = phi_sim, sigma_y = sigy_sim, sigma_v = sigv_sim,
      eta_vec = innovations$eta[, b], eps_vec = innovations$eps[, b],
      p = p, T_out = Tsize, burnin = ini
    ))
    out_tmp <- tryCatch(
      svp(as.numeric(u), p = p, J = j, errorType = "GED", del = del,
          wDecay = wDecay, sigvMethod = sigvMethod,
          winsorize_eps = winsorize_eps),
      error = function(e) NULL
    )
    if (!is.null(out_tmp)) {
      out_null_tmp <- out_tmp
      out_null_tmp$v <- nu_null
      sN_tmp <- tryCatch(
        Tsize * (LRT_moment_ged(u, out_null_tmp, Amat, WAmat, del, Bartlett) -
                   LRT_moment_ged(u, out_tmp, Amat, WAmat, del, Bartlett)),
        error = function(e) NULL
      )
      if (!is.null(sN_tmp) && !is.na(sN_tmp)) {
        sN[xn] <- max(sN_tmp, 1e-10)
        sign_vec[xn] <- sign(out_tmp$v - nu_null)
        xn <- xn + 1
      }
    }
    b <- b + 1
  }
  # Fallback: draw more if needed
  while (xn <= N) {
    eta_extra <- rnorm(n_total)
    a_ged <- exp(0.5 * (lgamma(1 / nu_sim) - lgamma(3 / nu_sim)))
    x_gam <- rgamma(n_total, shape = 1 / nu_sim, rate = 1)
    eps_extra <- sign(runif(n_total) - 0.5) * a_ged * x_gam^(1 / nu_sim)
    u <- as.matrix(sim_from_innov_ged_cpp(
      phi = phi_sim, sigma_y = sigy_sim, sigma_v = sigv_sim,
      eta_vec = eta_extra, eps_vec = eps_extra,
      p = p, T_out = Tsize, burnin = ini
    ))
    out_tmp <- tryCatch(
      svp(as.numeric(u), p = p, J = j, errorType = "GED", del = del,
          wDecay = wDecay, sigvMethod = sigvMethod,
          winsorize_eps = winsorize_eps),
      error = function(e) NULL
    )
    if (!is.null(out_tmp)) {
      out_null_tmp <- out_tmp
      out_null_tmp$v <- nu_null
      sN_tmp <- tryCatch(
        Tsize * (LRT_moment_ged(u, out_null_tmp, Amat, WAmat, del, Bartlett) -
                   LRT_moment_ged(u, out_tmp, Amat, WAmat, del, Bartlett)),
        error = function(e) NULL
      )
      if (!is.null(sN_tmp) && !is.na(sN_tmp)) {
        sN[xn] <- max(sN_tmp, 1e-10)
        sign_vec[xn] <- sign(out_tmp$v - nu_null)
        xn <- xn + 1
      }
    }
  }

  out <- if (direction != "two-sided") sign_vec * sqrt(sN) else sN
  attr(out, "innovations") <- innovations
  return(out)
}

# =========================================================================== #
# Null simulation helpers for leverage tests under heavy tails
# =========================================================================== #

.simnull_lev_t <- function(betasim_null, rho_null, p, j, Tsize, N, ini,
                            Amat, rho_type, del = 1e-10,
                            wDecay = FALSE, trunc_lev = TRUE,
                            logNu = FALSE, sigvMethod = "factored",
                            winsorize_eps = FALSE,
                            innovations = NULL) {
  phi_sim <- betasim_null[seq_len(p)]
  sigy_sim <- betasim_null[p + 1]
  sigv_sim <- betasim_null[p + 2]
  nu_sim <- betasim_null[p + 3]
  rho_sim <- betasim_null[p + 4]
  n_total <- ini + Tsize

  # Student-t leverage: nu varies in nuisance → PIT for chi2
  if (is.null(innovations)) {
    N_draw <- ceiling(N * 1.5) + 10L
    innovations <- list(
      zeta    = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw),
      aux     = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw),
      U_chi2  = matrix(runif(n_total * N_draw), nrow = n_total, ncol = N_draw)
    )
  }

  sN <- numeric(N)
  xn <- 1
  b <- 1
  max_iter <- ncol(innovations$zeta)
  while (xn <= N && b <= max_iter) {
    u <- as.numeric(sim_from_innov_t_lev_cpp(
      phi = phi_sim, sigma_y = sigy_sim, sigma_v = sigv_sim,
      nu = nu_sim, rho = rho_sim,
      zeta_vec = innovations$zeta[, b], aux_vec = innovations$aux[, b],
      U_chi2_vec = innovations$U_chi2[, b],
      p = p, T_out = Tsize, burnin = ini
    ))
    b <- b + 1
    out_tmp <- tryCatch(
      svp(u, p, j, leverage = TRUE, errorType = "Student-t",
          rho_type = rho_type, del = del, trunc_lev = trunc_lev,
          wDecay = wDecay, logNu = logNu, sigvMethod = sigvMethod,
          winsorize_eps = winsorize_eps),
      error = function(e) NULL
    )
    if (!is.null(out_tmp) && is.finite(out_tmp$v) &&
        !is.na(out_tmp$rho) && abs(out_tmp$rho) <= 1) {
      out_null_tmp <- out_tmp
      out_null_tmp$rho <- rho_null
      u_mat <- as.matrix(u)
      sN_tmp <- tryCatch(
        Tsize * (LRT_moment_lev_t(u_mat, out_null_tmp, Amat, rho_type, del) -
                   LRT_moment_lev_t(u_mat, out_tmp, Amat, rho_type, del)),
        error = function(e) NA
      )
      if (!is.na(sN_tmp)) {
        sN[xn] <- max(sN_tmp, 1e-10)
        xn <- xn + 1
      }
    }
  }
  if (xn <= N) warning("simnull_lev_t: only ", xn - 1, " of ", N, " valid stats.")
  attr(sN, "innovations") <- innovations
  return(sN)
}

.simnull_lev_t_Amat <- function(betasim_null, rho_null, p, j, Tsize, N, ini,
                                 rho_type, del = 1e-10, Bartlett = TRUE,
                                 wDecay = FALSE, trunc_lev = TRUE,
                                 logNu = FALSE, sigvMethod = "factored",
                                 winsorize_eps = FALSE,
                                 innovations = NULL) {
  phi_sim <- betasim_null[seq_len(p)]
  sigy_sim <- betasim_null[p + 1]
  sigv_sim <- betasim_null[p + 2]
  nu_sim <- betasim_null[p + 3]
  rho_sim <- betasim_null[p + 4]
  n_total <- ini + Tsize

  if (is.null(innovations)) {
    N_draw <- ceiling(N * 1.5) + 10L
    innovations <- list(
      zeta    = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw),
      aux     = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw),
      U_chi2  = matrix(runif(n_total * N_draw), nrow = n_total, ncol = N_draw)
    )
  }

  sN <- numeric(N)
  xn <- 1
  b <- 1
  max_iter <- ncol(innovations$zeta)
  while (xn <= N && b <= max_iter) {
    u <- as.numeric(sim_from_innov_t_lev_cpp(
      phi = phi_sim, sigma_y = sigy_sim, sigma_v = sigv_sim,
      nu = nu_sim, rho = rho_sim,
      zeta_vec = innovations$zeta[, b], aux_vec = innovations$aux[, b],
      U_chi2_vec = innovations$U_chi2[, b],
      p = p, T_out = Tsize, burnin = ini
    ))
    b <- b + 1
    out_tmp <- tryCatch(
      svp(u, p, j, leverage = TRUE, errorType = "Student-t",
          rho_type = rho_type, del = del, trunc_lev = trunc_lev,
          wDecay = wDecay, logNu = logNu, sigvMethod = sigvMethod,
          winsorize_eps = winsorize_eps),
      error = function(e) NULL
    )
    if (!is.null(out_tmp) && is.finite(out_tmp$v) &&
        !is.na(out_tmp$rho) && abs(out_tmp$rho) <= 1) {
      out_null_tmp <- out_tmp
      out_null_tmp$rho <- rho_null
      u_mat <- as.matrix(u)
      sN_tmp <- tryCatch(
        Tsize * (LRT_moment_lev_t_Amat(u_mat, out_null_tmp, rho_type, del, Bartlett) -
                   LRT_moment_lev_t_Amat(u_mat, out_tmp, rho_type, del, Bartlett)),
        error = function(e) NA
      )
      if (!is.na(sN_tmp)) {
        sN[xn] <- max(sN_tmp, 1e-10)
        xn <- xn + 1
      }
    }
  }
  if (xn <= N) warning("simnull_lev_t_Amat: only ", xn - 1, " of ", N, " valid stats.")
  attr(sN, "innovations") <- innovations
  return(sN)
}

.simnull_lev_ged <- function(betasim_null, rho_null, p, j, Tsize, N, ini,
                              Amat, rho_type, del = 1e-10,
                              wDecay = FALSE, trunc_lev = TRUE,
                              sigvMethod = "factored", winsorize_eps = FALSE,
                              innovations = NULL) {
  phi_sim <- betasim_null[seq_len(p)]
  sigy_sim <- betasim_null[p + 1]
  sigv_sim <- betasim_null[p + 2]
  nu_sim <- betasim_null[p + 3]
  rho_sim <- betasim_null[p + 4]
  n_total <- ini + Tsize

  # GED leverage: Gaussian copula, just zeta+aux (no chi2 needed)
  if (is.null(innovations)) {
    N_draw <- ceiling(N * 1.5) + 10L
    innovations <- list(
      zeta = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw),
      aux  = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw)
    )
  }

  sN <- numeric(N)
  xn <- 1
  b <- 1
  max_iter <- ncol(innovations$zeta)
  while (xn <= N && b <= max_iter) {
    u <- as.numeric(sim_from_innov_ged_lev_cpp(
      phi = phi_sim, sigma_y = sigy_sim, sigma_v = sigv_sim,
      nu = nu_sim, rho = rho_sim,
      zeta_vec = innovations$zeta[, b], aux_vec = innovations$aux[, b],
      p = p, T_out = Tsize, burnin = ini
    ))
    b <- b + 1
    out_tmp <- tryCatch(
      svp(u, p, j, leverage = TRUE, errorType = "GED",
          rho_type = rho_type, del = del, trunc_lev = trunc_lev,
          wDecay = wDecay, sigvMethod = sigvMethod,
          winsorize_eps = winsorize_eps),
      error = function(e) NULL
    )
    if (!is.null(out_tmp) && is.finite(out_tmp$v) &&
        !is.na(out_tmp$rho) && abs(out_tmp$rho) <= 1) {
      out_null_tmp <- out_tmp
      out_null_tmp$rho <- rho_null
      u_mat <- as.matrix(u)
      sN_tmp <- tryCatch(
        Tsize * (LRT_moment_lev_ged(u_mat, out_null_tmp, Amat, rho_type, del) -
                   LRT_moment_lev_ged(u_mat, out_tmp, Amat, rho_type, del)),
        error = function(e) NA
      )
      if (!is.na(sN_tmp)) {
        sN[xn] <- max(sN_tmp, 1e-10)
        xn <- xn + 1
      }
    }
  }
  if (xn <= N) warning("simnull_lev_ged: only ", xn - 1, " of ", N, " valid stats.")
  attr(sN, "innovations") <- innovations
  return(sN)
}

.simnull_lev_ged_Amat <- function(betasim_null, rho_null, p, j, Tsize, N, ini,
                                   rho_type, del = 1e-10, Bartlett = TRUE,
                                   wDecay = FALSE, trunc_lev = TRUE,
                                   sigvMethod = "factored", winsorize_eps = FALSE,
                                   innovations = NULL) {
  phi_sim <- betasim_null[seq_len(p)]
  sigy_sim <- betasim_null[p + 1]
  sigv_sim <- betasim_null[p + 2]
  nu_sim <- betasim_null[p + 3]
  rho_sim <- betasim_null[p + 4]
  n_total <- ini + Tsize

  if (is.null(innovations)) {
    N_draw <- ceiling(N * 1.5) + 10L
    innovations <- list(
      zeta = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw),
      aux  = matrix(rnorm(n_total * N_draw), nrow = n_total, ncol = N_draw)
    )
  }

  sN <- numeric(N)
  xn <- 1
  b <- 1
  max_iter <- ncol(innovations$zeta)
  while (xn <= N && b <= max_iter) {
    u <- as.numeric(sim_from_innov_ged_lev_cpp(
      phi = phi_sim, sigma_y = sigy_sim, sigma_v = sigv_sim,
      nu = nu_sim, rho = rho_sim,
      zeta_vec = innovations$zeta[, b], aux_vec = innovations$aux[, b],
      p = p, T_out = Tsize, burnin = ini
    ))
    b <- b + 1
    out_tmp <- tryCatch(
      svp(u, p, j, leverage = TRUE, errorType = "GED",
          rho_type = rho_type, del = del, trunc_lev = trunc_lev,
          wDecay = wDecay, sigvMethod = sigvMethod,
          winsorize_eps = winsorize_eps),
      error = function(e) NULL
    )
    if (!is.null(out_tmp) && is.finite(out_tmp$v) &&
        !is.na(out_tmp$rho) && abs(out_tmp$rho) <= 1) {
      out_null_tmp <- out_tmp
      out_null_tmp$rho <- rho_null
      u_mat <- as.matrix(u)
      sN_tmp <- tryCatch(
        Tsize * (LRT_moment_lev_ged_Amat(u_mat, out_null_tmp, rho_type, del, Bartlett) -
                   LRT_moment_lev_ged_Amat(u_mat, out_tmp, rho_type, del, Bartlett)),
        error = function(e) NA
      )
      if (!is.na(sN_tmp)) {
        sN[xn] <- max(sN_tmp, 1e-10)
        xn <- xn + 1
      }
    }
  }
  if (xn <= N) warning("simnull_lev_ged_Amat: only ", xn - 1, " of ", N, " valid stats.")
  attr(sN, "innovations") <- innovations
  return(sN)
}

# =========================================================================== #
# Tests for leverage estimation under heavy-tailed distributions
# =========================================================================== #

# --- Simulation Tests ---

test_that("SVL(1) Student-t leverage simulation produces correct output", {
  set.seed(42)
  out <- sim_svp(500, phi = 0.9, sigy = 1, sigv = 0.5,
                 errorType = "Student-t", leverage = TRUE, rho = -0.5, nu = 5)
  expect_type(out, "list")
  expect_true(all(c("y", "h", "z", "v") %in% names(out)))
  expect_length(out$y, 500)
  expect_length(out$h, 500)
  expect_length(out$z, 500)
  expect_length(out$v, 500)
})

test_that("SVL(1) GED leverage simulation produces correct output", {
  set.seed(42)
  out <- sim_svp(500, phi = 0.9, sigy = 1, sigv = 0.5,
                 errorType = "GED", leverage = TRUE, rho = -0.5, nu = 1.5)
  expect_type(out, "list")
  expect_true(all(c("y", "h", "z", "v") %in% names(out)))
  expect_length(out$y, 500)
})

test_that("SVL(1) Student-t with rho=0 matches non-leverage marginal variance", {
  set.seed(123)
  y_lev <- sim_svp(5000, phi = 0.9, sigy = 1, sigv = 0.5,
                   errorType = "Student-t", leverage = TRUE, rho = 0, nu = 5)$y
  set.seed(456)
  y_nolev <- sim_svp(5000, phi = 0.9, sigy = 1, sigv = 0.5,
                     errorType = "Student-t", leverage = FALSE, nu = 5)$y
  expect_true(abs(var(y_lev) - var(y_nolev)) / var(y_nolev) < 0.15)
})

test_that("SVL(2) Student-t leverage simulation works", {
  set.seed(42)
  out <- sim_svp(500, phi = c(0.2, 0.63), sigy = 1, sigv = 1,
                 errorType = "Student-t", leverage = TRUE, rho = -0.3, nu = 5)
  expect_type(out, "list")
  expect_length(out$y, 500)
})

# --- Correction Factor Tests ---

test_that("C_t(nu) matches known values", {
  expect_equal(correction_factor_t(3), 6 / pi, tolerance = 1e-6)
  expect_equal(correction_factor_t(5), 40 / (9 * pi), tolerance = 1e-4)
  expect_true(correction_factor_t(100) > 1 && correction_factor_t(100) < 1.02)
  expect_true(correction_factor_t(1000) > 1 && correction_factor_t(1000) < 1.003)
})

test_that("C_g(2) = 1 (Gaussian limit)", {
  expect_equal(correction_factor_ged_approx(2), 1, tolerance = 1e-6)
})

test_that("C_g(1.5) matches paper Table 8", {
  expect_equal(correction_factor_ged_approx(1.5), 0.959, tolerance = 0.002)
})

test_that("qged_std reproduces qnorm for nu=2", {
  p_vals <- c(0.025, 0.1, 0.25, 0.5, 0.75, 0.9, 0.975)
  expect_equal(qged_std(p_vals, 2), qnorm(p_vals), tolerance = 1e-6)
})

test_that("E[|u|] for GED(2) equals sqrt(2/pi)", {
  expect_equal(ged_E_abs_u(2), sqrt(2 / pi), tolerance = 1e-10)
})

# --- Estimation Tests ---

test_that("SVL(1) Student-t estimation returns leverage parameters", {
  set.seed(42)
  y <- sim_svp(2000, phi = 0.9, sigy = 1, sigv = 0.5,
               errorType = "Student-t", leverage = TRUE, rho = -0.5, nu = 5)$y
  fit <- svp(y, p = 1, J = 50, leverage = TRUE, errorType = "Student-t")
  expect_true(!is.na(fit$rho))
  expect_true(abs(fit$rho) < 1)
  expect_true(fit$v > 2)
  expect_true(isTRUE(fit$leverage))
  expect_true(!is.na(fit$gammatilde))
  expect_true(!is.null(fit$CF))
  expect_equal(length(fit$theta), 5)  # phi, sigy, sigv, nu, rho
})

test_that("SVL(1) GED estimation returns leverage parameters", {
  set.seed(42)
  y <- sim_svp(2000, phi = 0.9, sigy = 1, sigv = 0.5,
               errorType = "GED", leverage = TRUE, rho = -0.5, nu = 1.5)$y
  fit <- svp(y, p = 1, J = 50, leverage = TRUE, errorType = "GED")
  expect_true(!is.na(fit$rho))
  expect_true(abs(fit$rho) < 1)
  expect_true(fit$v > 0)
  expect_true(isTRUE(fit$leverage))
  expect_equal(length(fit$theta), 5)
})

test_that("SVL(1) GED with nu=2 gives leverage close to Gaussian", {
  set.seed(123)
  y <- sim_svp(3000, phi = 0.9, sigy = 1, sigv = 0.3, leverage = TRUE, rho = -0.3)$y
  fit_g <- svp(y, p = 1, J = 50, leverage = TRUE, errorType = "Gaussian")
  suppressWarnings(
    fit_ged <- svp(y, p = 1, J = 50, leverage = TRUE, errorType = "GED")
  )
  # GED nu=2 should give similar rho to Gaussian
  if (fit_ged$v > 1.8 && fit_ged$v < 2.2) {
    expect_true(abs(fit_g$rho - fit_ged$rho) < 0.1)
  }
})

test_that("Student-t nu=500 gives leverage close to Gaussian", {
  set.seed(123)
  y <- sim_svp(3000, phi = 0.9, sigy = 1, sigv = 0.3, leverage = TRUE, rho = -0.3)$y
  fit_g <- svp(y, p = 1, J = 50, leverage = TRUE, errorType = "Gaussian")
  suppressWarnings(
    fit_t <- svp(y, p = 1, J = 50, leverage = TRUE, errorType = "Student-t")
  )
  # nu should be large (near Gaussian), rho should be similar
  if (fit_t$v > 30) {
    expect_true(abs(fit_g$rho - fit_t$rho) < 0.05)
  }
})

test_that("SVL(2) Student-t estimation works", {
  set.seed(42)
  y <- sim_svp(2000, phi = c(0.2, 0.63), sigy = 1, sigv = 1,
               errorType = "Student-t", leverage = TRUE, rho = -0.3, nu = 5)$y
  fit <- svp(y, p = 2, J = 50, leverage = TRUE, errorType = "Student-t")
  expect_true(!is.na(fit$rho))
  expect_equal(length(fit$phi), 2)
  expect_equal(length(fit$theta), 6)  # phi1, phi2, sigy, sigv, nu, rho
})

test_that("Model object has all required leverage fields", {
  set.seed(42)
  y <- sim_svp(1000, phi = 0.9, sigy = 1, sigv = 0.5,
               errorType = "Student-t", leverage = TRUE, rho = -0.5, nu = 5)$y
  fit <- svp(y, p = 1, J = 50, leverage = TRUE, errorType = "Student-t")
  expect_true("rho" %in% names(fit))
  expect_true("gammatilde" %in% names(fit))
  expect_true("leverage" %in% names(fit))
  expect_true("rho_type" %in% names(fit))
  expect_true("trunc_lev" %in% names(fit))
  expect_true("CF" %in% names(fit))
  expect_equal(fit$rho_type, "pearson")
  expect_true(isTRUE(fit$leverage))
})

# --- LMC Test Tests ---

test_that("lmc_lev with Student-t runs and returns svp_test", {
  set.seed(42)
  y <- sim_svp(500, phi = 0.9, sigy = 1, sigv = 0.5,
               errorType = "Student-t", leverage = TRUE, rho = -0.3, nu = 5)$y
  suppressWarnings(
    result <- lmc_lev(y, p = 1, J = 50, N = 9, rho_null = 0,
                      errorType = "Student-t")
  )
  expect_s3_class(result, "svp_test")
  expect_true(result$pval >= 0 && result$pval <= 1)
  expect_equal(length(result$sN), 9)
})

test_that("lmc_lev with GED runs and returns svp_test", {
  set.seed(42)
  y <- sim_svp(500, phi = 0.9, sigy = 1, sigv = 0.5,
               errorType = "GED", leverage = TRUE, rho = -0.3, nu = 1.5)$y
  suppressWarnings(
    result <- lmc_lev(y, p = 1, J = 50, N = 9, rho_null = 0,
                      errorType = "GED")
  )
  expect_s3_class(result, "svp_test")
  expect_true(result$pval >= 0 && result$pval <= 1)
})

# --- SE Tests ---

test_that("svpSE for SVL(1) Student-t returns correct dimensions", {
  set.seed(42)
  y <- sim_svp(1000, phi = 0.9, sigy = 1, sigv = 0.5,
               errorType = "Student-t", leverage = TRUE, rho = -0.5, nu = 5)$y
  fit <- svp(y, p = 1, J = 50, leverage = TRUE, errorType = "Student-t")
  suppressWarnings(
    se <- svpSE(fit, n_sim = 9, burnin = 200)
  )
  expect_equal(ncol(se$CI), 5)  # phi, sigy, sigv, nu, rho
  expect_equal(nrow(se$CI), 2)  # lower, upper
  expect_equal(length(se$SEsim), 5)
})

test_that("svpSE for SVL(1) GED returns correct dimensions", {
  set.seed(42)
  y <- sim_svp(1000, phi = 0.9, sigy = 1, sigv = 0.5,
               errorType = "GED", leverage = TRUE, rho = -0.5, nu = 1.5)$y
  fit <- svp(y, p = 1, J = 50, leverage = TRUE, errorType = "GED")
  suppressWarnings(
    se <- svpSE(fit, n_sim = 9, burnin = 200)
  )
  expect_equal(ncol(se$CI), 5)
  expect_equal(length(se$SEsim), 5)
})

# --- Coef and Print Methods ---

test_that("coef.svp_t includes rho for leverage models", {
  set.seed(42)
  y <- sim_svp(1000, phi = 0.9, sigy = 1, sigv = 0.5,
               errorType = "Student-t", leverage = TRUE, rho = -0.5, nu = 5)$y
  fit <- svp(y, p = 1, J = 50, leverage = TRUE, errorType = "Student-t")
  co <- coef(fit)
  expect_true("rho" %in% names(co))
  expect_equal(length(co), 5)
})

test_that("coef.svp_t excludes rho for non-leverage models", {
  set.seed(42)
  y <- sim_svp(1000, phi = 0.9, sigy = 1, sigv = 0.5,
               errorType = "Student-t", nu = 5)$y
  fit <- svp(y, p = 1, J = 50, errorType = "Student-t")
  co <- coef(fit)
  expect_false("rho" %in% names(co))
  expect_equal(length(co), 4)
})

#' @title wARMASVp: Winsorized ARMA Estimation for Higher-Order Stochastic
#'   Volatility Models
#'
#' @description Estimation, simulation, hypothesis testing, and forecasting for
#'   univariate higher-order stochastic volatility SV(p) models. Supports
#'   Gaussian, Student-t, and GED innovations with optional leverage effects.
#'
#' @details The main user-facing functions are:
#' \itemize{
#'   \item \code{\link{svp}} -- Estimate SV(p) model with optional leverage
#'   \item \code{\link{svpSE}} -- Simulation-based standard errors
#'   \item \code{\link{sim_svp}} -- Simulate SV(p) processes
#'   \item \code{\link{filter_svp}} -- Kalman/mixture/particle filtering
#'   \item \code{\link{forecast_svp}} -- Multi-step volatility forecasts
#'   \item \code{\link{lmc_ar}}, \code{\link{mmc_ar}} -- AR order tests
#'   \item \code{\link{lmc_lev}}, \code{\link{mmc_lev}} -- Leverage tests
#'   \item \code{\link{lmc_t}}, \code{\link{mmc_t}} -- Student-t tail tests
#'   \item \code{\link{lmc_ged}}, \code{\link{mmc_ged}} -- GED tail tests
#' }
#'
#' @references
#' Ahsan, M. N. and Dufour, J.-M. (2021). Simple estimators and inference for
#' higher-order stochastic volatility models. \emph{Journal of Econometrics},
#' 224(1), 181-197. \doi{10.1016/j.jeconom.2021.03.008}
#'
#' Ahsan, M. N., Dufour, J.-M., and Rodriguez-Rondon, G. (2025). Estimation and
#' inference for higher-order stochastic volatility models with leverage.
#' \emph{Journal of Time Series Analysis}, 46(6), 1064-1084.
#' \doi{10.1111/jtsa.12851}
#'
#' Ahsan, M. N., Dufour, J.-M., and Rodriguez-Rondon, G. (2026). Estimation and
#' inference for stochastic volatility models with heavy-tailed distributions.
#' Bank of Canada Staff Working Paper 2026-8. \doi{10.34989/swp-2026-8}
#'
#' @useDynLib wARMASVp, .registration = TRUE
#' @importFrom Rcpp sourceCpp
#' @importFrom gsignal poly
#' @importFrom stats qnorm quantile var rnorm uniroot dnorm pgamma qgamma
#'   rchisq rf rgamma rt runif setNames
#' @keywords internal
"_PACKAGE"

# =========================================================================== #
# S3 methods for wARMASVp classes
# =========================================================================== #

# ----------- svp class methods ----------- #

#' @export
print.svp <- function(x, ...) {
  p <- x$p
  mdl_type <- if (isTRUE(x$leverage) && !is.na(x$rho)) "SVL" else "SV"
  cat(sprintf("%s(%d) model (W-ARMA-SV estimation)\n", mdl_type, p))
  cat(sprintf("  phi: %s\n", paste(round(x$phi, 4), collapse = ", ")))
  cat(sprintf("  sigy: %.4f, sigv: %.4f\n", x$sigy, x$sigv))
  if (!is.na(x$rho)) {
    cat(sprintf("  rho (leverage): %.4f\n", x$rho))
  }
  cat(sprintf("  T = %d, J = %d\n", length(x$y), x$J))
  invisible(x)
}

#' @export
summary.svp <- function(object, ...) {
  p <- object$p
  mdl_type <- if (isTRUE(object$leverage) && !is.na(object$rho)) "SVL" else "SV"
  cat(sprintf("\n%s(%d) Model - W-ARMA-SV Estimation\n", mdl_type, p))
  cat(paste(rep("-", 50), collapse = ""), "\n")
  cat(sprintf("Sample size: %d\n", length(object$y)))
  cat(sprintf("Winsorizing parameter J: %d\n", object$J))
  if (!is.na(object$rho)) {
    cat(sprintf("Leverage correlation type: %s\n", object$rho_type))
  }
  cat(paste(rep("-", 50), collapse = ""), "\n")
  cat("Parameter estimates:\n\n")
  # Build parameter table
  pnames <- c(paste0("phi_", 1:p), "sigma_y", "sigma_v")
  pvals <- c(object$phi, object$sigy, object$sigv)
  if (!is.na(object$rho)) {
    pnames <- c(pnames, "rho")
    pvals <- c(pvals, object$rho)
  }
  df <- data.frame(Parameter = pnames, Estimate = round(pvals, 6))
  print(df, row.names = FALSE)
  cat("\n")
  if (!is.na(object$gammatilde)) {
    cat(sprintf("gamma_tilde: %.6f\n", object$gammatilde))
  }
  if (isTRUE(object$nonstationary_ind)) {
    cat("Note: Stationarity was enforced (roots projected inside unit circle).\n")
  }
  cat("\n")
  invisible(object)
}

#' @export
coef.svp <- function(object, ...) {
  p <- object$p
  nms <- c(paste0("phi_", 1:p), "sigma_y", "sigma_v")
  vals <- c(object$phi, object$sigy, object$sigv)
  if (!is.na(object$rho)) {
    nms <- c(nms, "rho")
    vals <- c(vals, object$rho)
  }
  names(vals) <- nms
  return(vals)
}

# ----------- svp_t class methods ----------- #

#' @export
print.svp_t <- function(x, ...) {
  p <- length(x$phi)
  mdl_type <- if (isTRUE(x$leverage) && !is.na(x$rho)) "SVL" else "SV"
  cat(sprintf("%s(%d) model with Student-t errors (W-ARMA-SV estimation)\n", mdl_type, p))
  cat(sprintf("  phi: %s\n", paste(round(x$phi, 4), collapse = ", ")))
  cat(sprintf("  sigy: %.4f, sigv: %.4f\n", x$sigy, x$sigv))
  cat(sprintf("  nu (d.f.): %.4f\n", x$v))
  if (!is.na(x$rho)) {
    cat(sprintf("  rho (leverage): %.4f\n", x$rho))
  }
  cat(sprintf("  T = %d, J = %d\n", length(x$y), x$J))
  invisible(x)
}

#' @export
summary.svp_t <- function(object, ...) {
  p <- length(object$phi)
  mdl_type <- if (isTRUE(object$leverage) && !is.na(object$rho)) "SVL" else "SV"
  cat(sprintf("\n%s(%d) Model with Student-t Errors\n", mdl_type, p))
  cat(paste(rep("-", 50), collapse = ""), "\n")
  cat(sprintf("Sample size: %d\n", length(object$y)))
  cat(sprintf("Winsorizing parameter J: %d\n", object$J))
  if (!is.na(object$rho)) {
    cat(sprintf("Leverage correlation type: %s\n", object$rho_type))
  }
  cat(paste(rep("-", 50), collapse = ""), "\n")
  cat("Parameter estimates:\n\n")
  pnames <- c(paste0("phi_", seq_len(p)), "sigma_y", "sigma_v", "nu")
  pvals <- c(object$phi, object$sigy, object$sigv, object$v)
  if (!is.na(object$rho)) {
    pnames <- c(pnames, "rho")
    pvals <- c(pvals, object$rho)
  }
  df <- data.frame(Parameter = pnames, Estimate = round(pvals, 6))
  print(df, row.names = FALSE)
  cat("\n")
  if (!is.na(object$gammatilde)) {
    cat(sprintf("gamma_tilde: %.6f\n", object$gammatilde))
  }
  cat("\n")
  invisible(object)
}

#' @export
coef.svp_t <- function(object, ...) {
  p <- length(object$phi)
  nms <- c(paste0("phi_", seq_len(p)), "sigma_y", "sigma_v", "nu")
  vals <- c(object$phi, object$sigy, object$sigv, object$v)
  if (!is.na(object$rho)) {
    nms <- c(nms, "rho")
    vals <- c(vals, object$rho)
  }
  names(vals) <- nms
  return(vals)
}

# ----------- svp_ged class methods ----------- #

#' @export
print.svp_ged <- function(x, ...) {
  p <- length(x$phi)
  mdl_type <- if (isTRUE(x$leverage) && !is.na(x$rho)) "SVL" else "SV"
  cat(sprintf("%s(%d) model with GED errors (W-ARMA-SV estimation)\n", mdl_type, p))
  cat(sprintf("  phi: %s\n", paste(round(x$phi, 4), collapse = ", ")))
  cat(sprintf("  sigy: %.4f, sigv: %.4f\n", x$sigy, x$sigv))
  cat(sprintf("  nu (shape): %.4f\n", x$v))
  if (!is.na(x$rho)) {
    cat(sprintf("  rho (leverage): %.4f\n", x$rho))
  }
  cat(sprintf("  T = %d, J = %d\n", length(x$y), x$J))
  invisible(x)
}

#' @export
summary.svp_ged <- function(object, ...) {
  p <- length(object$phi)
  mdl_type <- if (isTRUE(object$leverage) && !is.na(object$rho)) "SVL" else "SV"
  cat(sprintf("\n%s(%d) Model with GED Errors\n", mdl_type, p))
  cat(paste(rep("-", 50), collapse = ""), "\n")
  cat(sprintf("Sample size: %d\n", length(object$y)))
  cat(sprintf("Winsorizing parameter J: %d\n", object$J))
  if (!is.na(object$rho)) {
    cat(sprintf("Leverage correlation type: %s\n", object$rho_type))
  }
  cat(paste(rep("-", 50), collapse = ""), "\n")
  cat("Parameter estimates:\n\n")
  pnames <- c(paste0("phi_", seq_len(p)), "sigma_y", "sigma_v", "nu")
  pvals <- c(object$phi, object$sigy, object$sigv, object$v)
  if (!is.na(object$rho)) {
    pnames <- c(pnames, "rho")
    pvals <- c(pvals, object$rho)
  }
  df <- data.frame(Parameter = pnames, Estimate = round(pvals, 6))
  print(df, row.names = FALSE)
  cat("\n")
  if (!is.na(object$gammatilde)) {
    cat(sprintf("gamma_tilde: %.6f\n", object$gammatilde))
  }
  cat("\n")
  invisible(object)
}

#' @export
coef.svp_ged <- function(object, ...) {
  p <- length(object$phi)
  nms <- c(paste0("phi_", seq_len(p)), "sigma_y", "sigma_v", "nu")
  vals <- c(object$phi, object$sigy, object$sigv, object$v)
  if (!is.na(object$rho)) {
    nms <- c(nms, "rho")
    vals <- c(vals, object$rho)
  }
  names(vals) <- nms
  return(vals)
}

# ----------- svp_filter class methods ----------- #

#' @export
print.svp_filter <- function(x, ...) {
  T_obs <- length(x$w_filtered)
  mdl <- x$model
  p <- length(mdl$phi)
  mdl_type <- if (isTRUE(mdl$leverage) && !is.na(mdl$rho)) "SVL" else "SV"
  dist_label <- if (inherits(mdl, "svp_t")) "Student-t"
                else if (inherits(mdl, "svp_ged")) "GED"
                else "Gaussian"
  cat(sprintf("%s(%d) Filter [%s, %s]\n", mdl_type, p, x$method, dist_label))
  cat(sprintf("  T = %d, log-likelihood = %.2f\n", T_obs, x$loglik))
  cat(sprintf("  Mean filtered MSE: %.4f\n", mean(x$P_filtered)))
  invisible(x)
}

#' @export
summary.svp_filter <- function(object, ...) {
  print.svp_filter(object, ...)
  cat(sprintf("  w_filtered range: [%.4f, %.4f]\n",
              min(object$w_filtered), max(object$w_filtered)))
  cat(sprintf("  w_smoothed range: [%.4f, %.4f]\n",
              min(object$w_smoothed), max(object$w_smoothed)))
  invisible(object)
}

#' @export
plot.svp_filter <- function(x, ...) {
  T_obs <- length(x$w_filtered)
  time_idx <- seq_len(T_obs)
  ylim <- range(c(x$w_filtered, x$w_smoothed))
  plot(time_idx, x$w_filtered, type = "l", col = "steelblue",
       xlab = "Time", ylab = "Log-volatility",
       main = sprintf("Filtered & Smoothed Log-Volatility [%s]", x$method),
       ylim = ylim, ...)
  graphics::lines(time_idx, x$w_smoothed, col = "darkred", lwd = 1.5)
  graphics::legend("topright", legend = c("Filtered", "Smoothed"),
                   col = c("steelblue", "darkred"), lty = 1, lwd = c(1, 1.5))
  invisible(x)
}

# ----------- svp_forecast class methods ----------- #

#' @export
print.svp_forecast <- function(x, ...) {
  H <- x$H
  mdl <- x$mdl
  p <- mdl$p
  mdl_type <- if (isTRUE(mdl$leverage) && !is.na(mdl$rho)) "SVL" else "SV"
  out_label <- if (!is.null(x$output)) x$output else "log-variance"
  cat(sprintf("%s(%d) Forecast (H = %d, output = %s)\n", mdl_type, p, H, out_label))
  cat(sprintf("  Forecasted values:\n"))
  for (h in seq_len(min(H, 10))) {
    cat(sprintf("    h=%d: %.4f\n", h, x$w_forecasted[h]))
  }
  if (H > 10) cat(sprintf("    ... (%d more horizons)\n", H - 10))
  invisible(x)
}

#' @export
plot.svp_forecast <- function(x, ...) {
  Tsize <- length(x$w_smoothed)
  H <- x$H
  # For plot, always show log-variance (smoothed + forecast)
  fc_vals <- if (!is.null(x$log_var_forecast)) x$log_var_forecast else x$w_forecasted
  all_w <- c(x$w_smoothed, fc_vals)
  time_idx <- seq_len(Tsize + H)
  plot(time_idx, all_w, type = "n",
       xlab = "Time", ylab = "Log-volatility",
       main = "Smoothed + Forecasted Log-Volatility", ...)
  graphics::lines(1:Tsize, x$w_smoothed, col = "black")
  graphics::lines((Tsize + 1):(Tsize + H), fc_vals, col = "red", lwd = 2)
  graphics::abline(v = Tsize + 0.5, lty = 2, col = "gray50")
  graphics::legend("topright", legend = c("Smoothed", "Forecast"),
                   col = c("black", "red"), lty = 1, lwd = c(1, 2))
  invisible(x)
}

# ----------- svp_test class methods ----------- #

#' @export
print.svp_test <- function(x, ...) {
  dir_label <- if (!is.null(x$direction) && x$direction != "two-sided") {
    sprintf(" [%s]", x$direction)
  } else ""
  cat(sprintf("%s Test%s\n", x$test_type, dir_label))
  cat(paste(rep("-", 40), collapse = ""), "\n")
  for (i in seq_along(x$null_param)) {
    cat(sprintf("  H0: %s = %s\n", x$null_param[i], x$null_value[i]))
  }
  cat(sprintf("  Test statistic (LR): %.4f\n", x$s0))
  if (!is.null(x$S_T)) {
    cat(sprintf("  Signed root (S_T): %.4f\n", x$S_T))
  }
  cat(sprintf("  p-value: %.4f\n", x$pval))
  cat(sprintf("  MC replications: %d\n", length(x$sN)))
  cat("\n")
  invisible(x)
}

#' Construct Companion Matrix for AR(p) Process
#'
#' Builds the companion matrix representation for an AR(p) process, which is
#' used in the state-space formulation of SV(p) models.
#'
#' @param phi Numeric vector of AR coefficients (length p).
#' @param p Integer. Order of the AR process.
#' @param q Integer. Dimension of each block (typically 1 for univariate).
#'
#' @return A \code{(q*p) x (q*p)} companion matrix.
#'
#' @keywords internal
companionMat <- function(phi, p, q) {
  F_tmp <- phi
  if (p > 1) {
    diagmat <- diag(q * (p - 1))
    diagzero <- matrix(0, q * (p - 1), q)
    Mn <- cbind(diagmat, diagzero)
    compMat <- rbind(F_tmp, Mn)
  } else {
    compMat <- matrix(F_tmp, nrow = q, ncol = q)
  }
  return(compMat)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

svpCpp_nolev <- function(u, p, J, del, wDecay = FALSE) {
    .Call(`_wARMASVp_svpCpp_nolev`, u, p, J, del, wDecay)
}

solve_lyapunov_discrete_cpp <- function(F_mat, Q) {
    .Call(`_wARMASVp_solve_lyapunov_discrete_cpp`, F_mat, Q)
}

kalman_filter_cpp <- function(y_star, y_raw, phi, sigma_y, sigma_v, delta_p, sig_eps2, var_zt, P0) {
    .Call(`_wARMASVp_kalman_filter_cpp`, y_star, y_raw, phi, sigma_y, sigma_v, delta_p, sig_eps2, var_zt, P0)
}

gmkf_filter_cpp <- function(y_star, y_raw, phi, sigma_y, sigma_v, delta_p, var_zt, mix_weights, mix_means, mix_vars, mu_intercept, P0) {
    .Call(`_wARMASVp_gmkf_filter_cpp`, y_star, y_raw, phi, sigma_y, sigma_v, delta_p, var_zt, mix_weights, mix_means, mix_vars, mu_intercept, P0)
}

fit_ksc_em_cpp <- function(eps, init_m, init_s2, init_q, max_iter = 500L, tol = 1e-8) {
    .Call(`_wARMASVp_fit_ksc_em_cpp`, eps, init_m, init_s2, init_q, max_iter, tol)
}

LRT_moment_lev_svp_cpp <- function(y, mdl_out, Amat, rho_type, del = 1e-10) {
    .Call(`_wARMASVp_LRT_moment_lev_svp_cpp`, y, mdl_out, Amat, rho_type, del)
}

LRT_moment_lev_t_cpp <- function(y, mdl_out, Amat, rho_type, del = 1e-10) {
    .Call(`_wARMASVp_LRT_moment_lev_t_cpp`, y, mdl_out, Amat, rho_type, del)
}

LRT_moment_lev_ged_cpp <- function(y, mdl_out, Amat, rho_type, del, gh_nodes, gh_weights) {
    .Call(`_wARMASVp_LRT_moment_lev_ged_cpp`, y, mdl_out, Amat, rho_type, del, gh_nodes, gh_weights)
}

LRT_moment_t_cpp <- function(y, mdl_out, Amat, WAmat = FALSE, del = 1e-10, Bartlett = TRUE, pinv_fn = NULL) {
    .Call(`_wARMASVp_LRT_moment_t_cpp`, y, mdl_out, Amat, WAmat, del, Bartlett, pinv_fn)
}

LRT_moment_ged_cpp <- function(y, mdl_out, Amat, WAmat = FALSE, del = 1e-10, Bartlett = TRUE, pinv_fn = NULL) {
    .Call(`_wARMASVp_LRT_moment_ged_cpp`, y, mdl_out, Amat, WAmat, del, Bartlett, pinv_fn)
}

LRT_moment_lev_svp_Amat_cpp <- function(y, mdl_out, rho_type, del = 1e-10, Bartlett = TRUE, pinv_fn = NULL) {
    .Call(`_wARMASVp_LRT_moment_lev_svp_Amat_cpp`, y, mdl_out, rho_type, del, Bartlett, pinv_fn)
}

LRT_moment_ar_Amat_cpp <- function(y, mdl_out, del = 1e-10, Bartlett = TRUE, pinv_fn = NULL) {
    .Call(`_wARMASVp_LRT_moment_ar_Amat_cpp`, y, mdl_out, del, Bartlett, pinv_fn)
}

LRT_moment_lev_t_Amat_cpp <- function(y, mdl_out, rho_type, del = 1e-10, Bartlett = TRUE, pinv_fn = NULL) {
    .Call(`_wARMASVp_LRT_moment_lev_t_Amat_cpp`, y, mdl_out, rho_type, del, Bartlett, pinv_fn)
}

LRT_moment_lev_ged_Amat_cpp <- function(y, mdl_out, rho_type, del, Bartlett, pinv_fn, gh_nodes, gh_weights) {
    .Call(`_wARMASVp_LRT_moment_lev_ged_Amat_cpp`, y, mdl_out, rho_type, del, Bartlett, pinv_fn, gh_nodes, gh_weights)
}

particle_filter_svp_cpp <- function(y_raw, phi, sigma_y, sigma_v, nu, dist_code, delta, M, seed, del) {
    .Call(`_wARMASVp_particle_filter_svp_cpp`, y_raw, phi, sigma_y, sigma_v, nu, dist_code, delta, M, seed, del)
}

find_nu_t_cpp <- function(se2b, nu_lower = 2.01, nu_upper = 500.0, logNu = FALSE, tol = 1e-6, maxiter = 1000L) {
    .Call(`_wARMASVp_find_nu_t_cpp`, se2b, nu_lower, nu_upper, logNu, tol, maxiter)
}

find_nu_ged_cpp <- function(se2, lower, upper, tol = 1e-6, maxiter = 1000L) {
    .Call(`_wARMASVp_find_nu_ged_cpp`, se2, lower, upper, tol, maxiter)
}

find_delta_ged_cpp <- function(target, sigv, nu, gh_nodes, gh_weights, tol = 1e-8) {
    .Call(`_wARMASVp_find_delta_ged_cpp`, target, sigv, nu, gh_nodes, gh_weights, tol)
}

sim_from_innov_gaussian_cpp <- function(phi, sigma_y, sigma_v, eta_vec, eps_vec, p, T_out, burnin) {
    .Call(`_wARMASVp_sim_from_innov_gaussian_cpp`, phi, sigma_y, sigma_v, eta_vec, eps_vec, p, T_out, burnin)
}

sim_from_innov_gaussian_lev_cpp <- function(phi, sigma_y, sigma_v, rho, zeta_vec, aux_vec, p, T_out, burnin) {
    .Call(`_wARMASVp_sim_from_innov_gaussian_lev_cpp`, phi, sigma_y, sigma_v, rho, zeta_vec, aux_vec, p, T_out, burnin)
}

sim_from_innov_t_cpp <- function(phi, sigma_y, sigma_v, eta_vec, eps_vec, p, T_out, burnin) {
    .Call(`_wARMASVp_sim_from_innov_t_cpp`, phi, sigma_y, sigma_v, eta_vec, eps_vec, p, T_out, burnin)
}

sim_from_innov_ged_cpp <- function(phi, sigma_y, sigma_v, eta_vec, eps_vec, p, T_out, burnin) {
    .Call(`_wARMASVp_sim_from_innov_ged_cpp`, phi, sigma_y, sigma_v, eta_vec, eps_vec, p, T_out, burnin)
}

sim_from_innov_t_lev_cpp <- function(phi, sigma_y, sigma_v, nu, rho, zeta_vec, aux_vec, U_chi2_vec, p, T_out, burnin) {
    .Call(`_wARMASVp_sim_from_innov_t_lev_cpp`, phi, sigma_y, sigma_v, nu, rho, zeta_vec, aux_vec, U_chi2_vec, p, T_out, burnin)
}

sim_from_innov_ged_lev_cpp <- function(phi, sigma_y, sigma_v, nu, rho, zeta_vec, aux_vec, p, T_out, burnin) {
    .Call(`_wARMASVp_sim_from_innov_ged_lev_cpp`, phi, sigma_y, sigma_v, nu, rho, zeta_vec, aux_vec, p, T_out, burnin)
}

sim_svp_norm_cpp <- function(beta, p, N, burnin) {
    .Call(`_wARMASVp_sim_svp_norm_cpp`, beta, p, N, burnin)
}

sim_svp_leverage_norm_cpp <- function(beta, p, N, burnin) {
    .Call(`_wARMASVp_sim_svp_leverage_norm_cpp`, beta, p, N, burnin)
}

sim_sv_t_cpp <- function(beta, p, N, burnin) {
    .Call(`_wARMASVp_sim_sv_t_cpp`, beta, p, N, burnin)
}

sim_sv_ged_cpp <- function(beta, p, N, burnin) {
    .Call(`_wARMASVp_sim_sv_ged_cpp`, beta, p, N, burnin)
}

sim_svp_leverage_t_cpp <- function(beta, p, N, burnin) {
    .Call(`_wARMASVp_sim_svp_leverage_t_cpp`, beta, p, N, burnin)
}

sim_svp_leverage_ged_cpp <- function(beta, p, N, burnin) {
    .Call(`_wARMASVp_sim_svp_leverage_ged_cpp`, beta, p, N, burnin)
}

kendall_corr <- function(x, y) {
    .Call(`_wARMASVp_kendall_corr`, x, y)
}

rged_cpp <- function(n, mean = 0.0, sd = 1.0, nu = 2.0) {
    .Call(`_wARMASVp_rged_cpp`, n, mean, sd, nu)
}

#' Simulate from a Stochastic Volatility Model
#'
#' Master simulation function for SV(p) models. Supports Gaussian, Student-t,
#' and GED error distributions, with optional leverage effects. This mirrors
#' the interface of \code{\link{svp}} for estimation.
#'
#' @details
#' The model is:
#' \deqn{y_t = \sigma_y \exp(w_t / 2) z_t}
#' \deqn{w_t = \phi_1 w_{t-1} + \cdots + \phi_p w_{t-p} + \sigma_v v_t}
#'
#' where \eqn{z_t} follows a distribution specified by \code{errorType}
#' (Gaussian, Student-t, or GED), and \eqn{v_t} is i.i.d. standard normal.
#' When \code{leverage = TRUE}, the correlation between \eqn{z_t} and
#' \eqn{v_{t+1}} is \eqn{\rho}.
#'
#' For Student-t errors with leverage, the scale-mixture representation
#' \eqn{z_t = \zeta_t \lambda_t^{-1/2}} is used, where leverage operates through
#' the Gaussian component \eqn{\zeta_t}. For GED errors with leverage, a Gaussian
#' copula construction \eqn{z_t = F_{\mathrm{GED}}^{-1}(\Phi(\zeta_t))} is used.
#' In both cases the returned \code{z} is the \emph{effective} return innovation
#' (not the latent \eqn{\zeta_t}), with marginal distribution matching the
#' \code{errorType}.
#'
#' @param n Integer. Length of the simulated series.
#' @param phi Numeric vector. AR coefficients for log-volatility (length p).
#' @param sigy Numeric. Unconditional standard deviation of returns.
#' @param sigv Numeric. Standard deviation of volatility innovations.
#' @param errorType Character. Error distribution: \code{"Gaussian"} (default),
#'   \code{"Student-t"}, or \code{"GED"}.
#' @param leverage Logical. If \code{TRUE}, simulate with leverage effects
#'   (correlated return and volatility shocks). Default is \code{FALSE}.
#' @param rho Numeric. Leverage parameter (correlation between return and
#'   volatility shocks). Must be in \code{[-1, 1]}. Only used when
#'   \code{leverage = TRUE}. Default is 0.
#' @param nu Numeric. Shape parameter for heavy-tailed distributions.
#'   Degrees of freedom for Student-t (must be > 2) or GED shape (must be > 0).
#'   Required when \code{errorType} is \code{"Student-t"} or \code{"GED"}.
#' @param burnin Integer. Number of initial observations to discard. Default 500.
#'
#' @return A named list of four length-\code{n} numeric vectors:
#' \describe{
#'   \item{\code{y}}{Observed returns \eqn{y_t}.}
#'   \item{\code{h}}{Log-volatility process \eqn{w_t} (equivalently \eqn{h_t}).}
#'   \item{\code{z}}{Return innovation such that
#'     \eqn{y_t = \sigma_y \exp(h_t/2)\, z_t}.
#'     Marginal distribution matches \code{errorType}: N(0,1) for Gaussian,
#'     t(\eqn{\nu}) for Student-t, unit-variance GED(\eqn{\nu}) for GED.}
#'   \item{\code{v}}{Volatility innovation such that
#'     \eqn{h_t - \sum_{j=1}^p \phi_j h_{t-j} = \sigma_v\, v_t}.
#'     Always N(0,1); under leverage, \eqn{v_t = \rho\, \zeta_{t-1} +
#'     \sqrt{1-\rho^2}\, \epsilon_t}.}
#' }
#'
#' @examples
#' \donttest{
#' # Gaussian SV(1), no leverage
#' sim <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2)
#' plot(sim$y, type = "l")
#'
#' # Gaussian SV(1) with leverage
#' sim_lev <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2,
#'                    leverage = TRUE, rho = -0.5)
#' plot(sim_lev$y, type = "l")
#'
#' # Student-t SV(1)
#' sim_t <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2,
#'                  errorType = "Student-t", nu = 5)
#' plot(sim_t$y, type = "l")
#'
#' # GED SV(1)
#' sim_ged <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2,
#'                    errorType = "GED", nu = 1.5)
#' plot(sim_ged$y, type = "l")
#' }
#'
#' @seealso \code{\link{svp}} for estimation.
#'
#' @export
sim_svp <- function(n, phi, sigy, sigv, errorType = "Gaussian",
                    leverage = FALSE, rho = 0, nu = NULL,
                    burnin = 500) {
  if (!is.numeric(n) || length(n) != 1L || n < 1L || n != as.integer(n))
    stop("'n' must be a positive integer.")
  n <- as.integer(n)
  phi <- as.numeric(phi)
  if (length(phi) < 1L || any(!is.finite(phi)))
    stop("'phi' must be a non-empty numeric vector with finite values.")
  p <- length(phi)
  if (!is.numeric(sigy) || length(sigy) != 1L || sigy <= 0)
    stop("'sigy' must be a positive number.")
  if (!is.numeric(sigv) || length(sigv) != 1L || sigv <= 0)
    stop("'sigv' must be a positive number.")
  if (!is.numeric(burnin) || length(burnin) != 1L || burnin < 0)
    stop("'burnin' must be a non-negative integer.")
  errorType <- match.arg(errorType, c("Gaussian", "Student-t", "GED"))

  if (errorType == "Gaussian") {
    if (leverage) {
      if (abs(rho) > 1) stop("rho must be in [-1, 1].")
      beta <- c(phi, sigy, sigv, rho)
      out <- sim_svp_leverage_norm_cpp(beta, p, n, burnin)
    } else {
      beta <- c(phi, sigy, sigv)
      out <- sim_svp_norm_cpp(beta, p, n, burnin)
    }
  } else if (errorType == "Student-t") {
    if (is.null(nu)) stop("nu is required for Student-t errors.")
    if (nu <= 2) stop("nu must be > 2 for Student-t errors.")
    if (leverage) {
      if (abs(rho) > 1) stop("rho must be in [-1, 1].")
      beta <- c(phi, sigy, sigv, nu, rho)
      out <- sim_svp_leverage_t_cpp(beta, p, n, burnin)
    } else {
      beta <- c(phi, sigy, sigv, nu)
      out <- sim_sv_t_cpp(beta, p, n, burnin)
    }
  } else {  # "GED"
    if (is.null(nu)) stop("nu is required for GED errors.")
    if (nu <= 0) stop("nu must be > 0 for GED errors.")
    if (leverage) {
      if (abs(rho) > 1) stop("rho must be in [-1, 1].")
      beta <- c(phi, sigy, sigv, nu, rho)
      out <- sim_svp_leverage_ged_cpp(beta, p, n, burnin)
    } else {
      beta <- c(phi, sigy, sigv, nu)
      out <- sim_sv_ged_cpp(beta, p, n, burnin)
    }
  }

  # Coerce 1-column matrices from C++ to plain numeric vectors.
  list(
    y = as.numeric(out$y),
    h = as.numeric(out$h),
    z = as.numeric(out$z),
    v = as.numeric(out$v)
  )
}

# =========================================================================== #
# Kalman Filtering, GMKF, and Filtering API for SV(p) Models
# =========================================================================== #


# --------------------------------------------------------------------------- #
# Helpers
# --------------------------------------------------------------------------- #

#' Solve Discrete Lyapunov Equation
#'
#' Solves X = F X t(F) + Q for X using the vectorization approach.
#'
#' @param F_mat Square matrix.
#' @param Q Square matrix (same dimensions as \code{F_mat}).
#' @return Solution matrix \code{X}.
#' @keywords internal
solve_lyapunov_discrete <- function(F_mat, Q) {
  solve_lyapunov_discrete_cpp(F_mat, Q)
}

#' CDF of Standardized GED
#'
#' Computes the CDF of the standardized GED(\eqn{\nu}) distribution with
#' unit variance.
#'
#' @param u Numeric vector. Evaluation points.
#' @param nu Numeric. GED shape parameter (\eqn{\nu > 0}).
#' @return Numeric vector of probabilities.
#' @keywords internal
pged_std <- function(u, nu) {
  a <- exp(0.5 * (lgamma(1 / nu) - lgamma(3 / nu)))
  ifelse(u >= 0,
         0.5 + 0.5 * pgamma((u / a)^nu, shape = 1 / nu),
         0.5 - 0.5 * pgamma((-u / a)^nu, shape = 1 / nu))
}


# --------------------------------------------------------------------------- #
# Measurement noise density functions (for verification and future BPF)
# --------------------------------------------------------------------------- #

#' Density of Centered log-F(1,nu) Measurement Noise (Student-t)
#'
#' @param y Numeric vector. Evaluation points (centered: E[eps] = 0).
#' @param nu Numeric. Student-t degrees of freedom.
#' @return Density values.
#' @keywords internal
density_eps_t <- function(y, nu) {
  mu_bar <- digamma(0.5) - digamma(nu / 2) + log(nu)
  x <- y + mu_bar
  log_dens <- -0.5 * log(nu) - lbeta(0.5, nu / 2) +
    x / 2 - ((1 + nu) / 2) * log(1 + exp(x) / nu)
  exp(log_dens)
}

#' Density of Centered log-GED^2 Measurement Noise
#'
#' @param y Numeric vector. Evaluation points (centered: E[eps] = 0).
#' @param nu Numeric. GED shape parameter.
#' @return Density values.
#' @keywords internal
density_eps_ged <- function(y, nu) {
  a <- exp(0.5 * (lgamma(1 / nu) - lgamma(3 / nu)))
  mu_bar_g <- (2 / nu) * digamma(1 / nu) + lgamma(1 / nu) - lgamma(3 / nu)
  x <- y + mu_bar_g
  u_abs <- exp(x / 2)
  log_dens <- log(nu) - log(2) - log(a) - lgamma(1 / nu) -
    (u_abs / a)^nu + x / 2
  exp(log_dens)
}


# --------------------------------------------------------------------------- #
# KSC Gaussian Mixture Fitting
# --------------------------------------------------------------------------- #

#' Fit K-Component Gaussian Mixture to Measurement Noise Density
#'
#' Uses EM algorithm to approximate the measurement noise density with a
#' Gaussian mixture. For Gaussian SV, returns the pre-computed KSC (1998)
#' 7-component table.
#'
#' @param distribution Character: \code{"gaussian"}, \code{"student_t"}, or
#'   \code{"ged"}.
#' @param nu Numeric. Shape parameter (ignored for Gaussian).
#' @param K Integer. Number of mixture components. Default 7.
#' @param n_sample Integer. Sample size for EM fitting. Default 10000.
#' @param max_iter Integer. Maximum EM iterations. Default 500.
#' @param tol Numeric. Convergence tolerance. Default 1e-8.
#' @param seed Integer. Random seed. Default 42.
#' @return List with \code{weights}, \code{means}, \code{vars}, \code{KL_div}.
#' @keywords internal
fit_ksc_mixture <- function(distribution = c("gaussian", "student_t", "ged"),
                            nu = NULL, K = 7, n_sample = 10000,
                            max_iter = 500, tol = 1e-8, seed = 42) {
  distribution <- match.arg(distribution)

  # Pre-computed KSC (1998) Table 4 for Gaussian (7 components)
  # These approximate the raw log(chi^2_1) density (mean ≈ -1.27, var ≈ pi^2/2)
  if (distribution == "gaussian" && K == 7) {
    w_ksc <- c(0.00730, 0.10556, 0.00002, 0.04395, 0.34001, 0.24566, 0.25750)
    w_ksc <- w_ksc / sum(w_ksc)  # normalize to sum exactly to 1
    return(list(
      weights = w_ksc,
      means   = c(-11.40039, -5.24321, -9.83726, 1.50746, -0.65098, 0.52478, -2.35859),
      vars    = c(5.79596, 2.61369, 5.17950, 0.16735, 0.64009, 0.34023, 1.26261),
      KL_div  = NA_real_,
      nu      = NULL,
      distribution = "gaussian"
    ))
  }

  # Generate large sample from exact density
  set.seed(seed)
  eps <- switch(distribution,
    gaussian = {
      log(rchisq(n_sample, 1)) - (digamma(0.5) + log(2))
    },
    student_t = {
      if (is.null(nu) || nu <= 0) stop("nu must be positive for Student-t.")
      mu_bar <- digamma(0.5) - digamma(nu / 2) + log(nu)
      log(rf(n_sample, 1, nu)) - mu_bar
    },
    ged = {
      if (is.null(nu) || nu <= 0) stop("nu must be positive for GED.")
      a <- exp(0.5 * (lgamma(1 / nu) - lgamma(3 / nu)))
      x_gam <- rgamma(n_sample, shape = 1 / nu, rate = 1)
      u <- sign(runif(n_sample) - 0.5) * a * x_gam^(1 / nu)
      mu_bar_g <- (2 / nu) * digamma(1 / nu) + lgamma(1 / nu) - lgamma(3 / nu)
      log(u^2 + 1e-300) - mu_bar_g
    }
  )

  # EM algorithm for K-component Gaussian mixture
  # Initialize with K-means-style quantile-based initialization
  breaks <- quantile(eps, probs = seq(0, 1, length.out = K + 1))
  m <- numeric(K)
  s2 <- numeric(K)
  q <- rep(1 / K, K)
  for (k in 1:K) {
    idx <- which(eps >= breaks[k] & eps < breaks[k + 1])
    if (k == K) idx <- which(eps >= breaks[k])
    if (length(idx) < 2) idx <- sample(length(eps), max(2, length(eps) %/% K))
    m[k] <- mean(eps[idx])
    s2[k] <- var(eps[idx])
    q[k] <- length(idx) / n_sample
  }

  # EM loop in C++ (returns sorted weights/means/vars)
  fit <- fit_ksc_em_cpp(eps = eps, init_m = m, init_s2 = s2, init_q = q,
                        max_iter = max_iter, tol = tol)

  list(
    weights = fit$weights,
    means   = fit$means,
    vars    = fit$vars,
    KL_div  = NA_real_,
    nu      = nu,
    distribution = distribution
  )
}


# --------------------------------------------------------------------------- #
# Internal: extract filter parameters from model object
# --------------------------------------------------------------------------- #

.get_filter_params <- function(model) {
  phi <- model$phi
  p <- length(phi)
  delta_p <- if (is.null(model$rho) || is.na(model$rho)) 0 else model$rho
  sigma_y <- model$sigy
  sigma_v <- model$sigv

  # Measurement noise variance: distribution-specific sigma_eps^2
  if (inherits(model, "svp_t") && !is.null(model$v) && is.finite(model$v)) {
    sig_eps2 <- psigamma(0.5, 1) + psigamma(model$v / 2, 1)
  } else if (inherits(model, "svp_ged") && !is.null(model$v) && is.finite(model$v)) {
    sig_eps2 <- (2 / model$v)^2 * psigamma(1 / model$v, 1)
  } else {
    sig_eps2 <- (pi^2) / 2
  }

  # Var(z_t) for the leverage prediction covariance.  Per SVHT Remark 3.5
  # (eq:ckf_leverage_prediction), the *conditional* state-innovation
  # covariance at one-step-ahead under the u-proxy is sigma_v^2 * (1-delta^2),
  # corresponding to var_zt = 0 (z_t is treated as observed at filter time t,
  # so its conditional variance is zero).  This applies uniformly across all
  # error types and leverage configurations.
  var_zt <- 0.0

  # Measurement intercept mu_bar
  if (inherits(model, "svp_t") && !is.null(model$v) && is.finite(model$v)) {
    mu_bar <- digamma(0.5) - digamma(model$v / 2) + log(model$v)
  } else if (inherits(model, "svp_ged") && !is.null(model$v) && is.finite(model$v)) {
    mu_bar <- (2 / model$v) * digamma(1 / model$v) +
      lgamma(1 / model$v) - lgamma(3 / model$v)
  } else {
    mu_bar <- digamma(0.5) + log(2)
  }
  mu_intercept <- log(sigma_y^2) + mu_bar

  # Distribution name for GMKF
  dist_name <- if (inherits(model, "svp_t")) "student_t"
               else if (inherits(model, "svp_ged")) "ged"
               else "gaussian"

  list(phi = phi, p = p, delta_p = delta_p, sigma_y = sigma_y,
       sigma_v = sigma_v, sig_eps2 = sig_eps2, var_zt = var_zt,
       mu_intercept = mu_intercept, dist_name = dist_name,
       nu = if (!is.null(model$v)) model$v else NULL,
       mu = model$mu)
}


# --------------------------------------------------------------------------- #
# filter_svp() — Main user-facing filter function
# --------------------------------------------------------------------------- #

#' Filter Latent Volatility from an Estimated SV(p) Model
#'
#' Applies Kalman filtering (corrected or Gaussian mixture) and RTS smoothing
#' to extract the latent log-volatility process from an estimated SV(p) model.
#'
#' @param object An \code{"svp"}, \code{"svp_t"}, or \code{"svp_ged"} object
#'   from \code{\link{svp}}.
#' @param method Character. Filter method: \code{"corrected"} (default) for
#'   standard Kalman with distribution-specific \eqn{\sigma_\varepsilon^2(\nu)},
#'   \code{"mixture"} for the Gaussian Mixture Kalman Filter (GMKF), or
#'   \code{"particle"} for the Bootstrap Particle Filter (BPF).
#' @param proxy Character. Leverage proxy for the state-space prediction
#'   mean \eqn{\hat{z}_{t-1}} that enters the leverage shift
#'   \eqn{\sigma_\nu \delta_p \hat{z}_{t-1}} (the prediction covariance is
#'   independently \eqn{\sigma_\nu^2(1-\delta_p^2)} per Rodriguez-Rondon,
#'   Dufour and Ahsan (2026), i.e.\ \code{var_zt = 0L}).
#'   \code{"bayes_optimal"} (default) uses the posterior mean
#'   \eqn{E[\zeta_{t-1} \mid u_{t-1}]} for Student-t leverage, which
#'   corrects the marginal variance inflation of using \eqn{\hat{u}_{t-1}}
#'   directly (\eqn{\mathrm{Var}(\hat{u}) = \nu/(\nu - 2) > 1}) and is
#'   what the IC functions \code{\link{svp_IC}} / \code{\link{svp_AR_order}}
#'   use internally. \code{"u"} reproduces the paper-faithful proxy of
#'   Remark~3.5 (\eqn{\hat{z}_{t-1} = \hat{u}_{t-1}}). Has no effect for
#'   Gaussian or GED leverage (the proxy is closed-form in both cases) and
#'   no effect when \code{leverage = FALSE}.
#' @param K Integer. Number of mixture components for GMKF. Default 7.
#' @param M Integer. Number of particles for BPF. Default 1000.
#' @param seed Integer. Random seed for BPF. Default 42.
#' @param del Numeric. Small constant for log transformation. Default \code{1e-10}.
#'
#' @return An object of class \code{"svp_filter"}, a list containing:
#' \describe{
#'   \item{w_filtered}{Filtered log-volatility (T-vector).}
#'   \item{w_smoothed}{Smoothed log-volatility (T-vector).}
#'   \item{zt}{Filtered standardized residuals.}
#'   \item{zt_smoothed}{Smoothed standardized residuals.}
#'   \item{P_filtered}{Filtered MSE of first state component.}
#'   \item{P_predicted}{Predicted MSE of first state component.}
#'   \item{xi_filtered}{Full filtered state vectors (p x T matrix).}
#'   \item{xi_smoothed}{Full smoothed state vectors (p x T matrix).}
#'   \item{loglik}{Approximate log-likelihood.}
#'   \item{method}{Filter method used.}
#'   \item{model}{The input model object.}
#' }
#'
#' @examples
#' \donttest{
#' y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2)$y
#' fit <- svp(y, p = 1)
#' filt <- filter_svp(fit)
#' plot(filt$w_smoothed, type = "l")
#' }
#'
#' @export
filter_svp <- function(object, method = c("corrected", "mixture", "particle"),
                       proxy = c("bayes_optimal", "u"),
                       K = 7, M = 1000, seed = 42, del = 1e-10) {
  if (!inherits(object, c("svp", "svp_t", "svp_ged"))) {
    stop("object must be of class 'svp', 'svp_t', or 'svp_ged'.")
  }
  method <- match.arg(method)
  proxy <- match.arg(proxy)
  if (!is.numeric(K) || length(K) != 1L || K < 1L)
    stop("'K' must be a positive integer (>= 1).")
  if (!is.numeric(M) || length(M) != 1L || M < 1L)
    stop("'M' must be a positive integer (>= 1).")
  if (!is.numeric(del) || length(del) != 1L || del <= 0)
    stop("'del' must be a positive number.")
  y_vec <- as.numeric(object$y)
  params <- .get_filter_params(object)
  proxy_type <- if (proxy == "u") 0L else 1L
  dist_code  <- switch(params$dist_name, "gaussian" = 0L,
                                          "student_t" = 1L,
                                          "ged" = 2L)
  nu_val     <- if (is.null(params$nu)) 0.0 else as.numeric(params$nu)

  # Build companion matrix F
  p <- params$p
  F_mat <- matrix(0, p, p)
  F_mat[1, ] <- params$phi
  if (p > 1) for (j in 1:(p - 1)) F_mat[j + 1, j] <- 1

  # Lyapunov initialization
  r_vec <- c(1, rep(0, p - 1))
  Q_init <- params$sigma_v^2 * (r_vec %*% t(r_vec))
  P0 <- tryCatch(
    solve_lyapunov_discrete(F_mat, Q_init),
    error = function(e) diag(p) * params$sigma_v^2
  )

  if (method == "corrected") {
    # CKF: standard corrected Kalman filter
    y_star <- log(y_vec^2 + del) - params$mu

    result <- kalman_filter_cpp(
      y_star = as.numeric(y_star),
      y_raw = y_vec,
      phi = params$phi,
      sigma_y = params$sigma_y,
      sigma_v = params$sigma_v,
      delta_p = params$delta_p,
      sig_eps2 = params$sig_eps2,
      var_zt = params$var_zt,
      P0 = P0,
      dist_code = dist_code,
      nu = nu_val,
      proxy_type = proxy_type
    )

  } else if (method == "mixture") {
    # GMKF: Gaussian Mixture Kalman Filter
    mixture <- fit_ksc_mixture(params$dist_name, params$nu, K, seed = seed)

    # For GMKF, y_star is raw log(y^2) (NOT centered by mu).
    # The intercept depends on whether mixture means are raw or centered:
    #   - KSC Gaussian table: means ≈ -1.27 (raw log-chi2 density)
    #     → intercept = log(sigma_y^2) only (m_k already encodes E[log(z^2)])
    #   - EM-fitted (Student-t, GED): means ≈ 0 (centered)
    #     → intercept = log(sigma_y^2) + mu_bar (full mu_intercept)
    y_star_raw <- log(y_vec^2 + del)
    mix_mean <- sum(mixture$weights * mixture$means)
    # If mixture mean is far from 0, means are raw → use log(sigy^2) only
    # If near 0, means are centered → use full mu_intercept
    if (abs(mix_mean) > 0.5) {
      mu_intercept_gmkf <- log(params$sigma_y^2)
    } else {
      mu_intercept_gmkf <- params$mu_intercept
    }

    result <- gmkf_filter_cpp(
      y_star = as.numeric(y_star_raw),
      y_raw = y_vec,
      phi = params$phi,
      sigma_y = params$sigma_y,
      sigma_v = params$sigma_v,
      delta_p = params$delta_p,
      var_zt = params$var_zt,
      mix_weights = mixture$weights,
      mix_means = mixture$means,
      mix_vars = mixture$vars,
      mu_intercept = mu_intercept_gmkf,
      P0 = P0,
      dist_code = dist_code,
      nu = nu_val,
      proxy_type = proxy_type
    )
    result$mixture <- mixture

  } else if (method == "particle") {
    # BPF: Bootstrap Particle Filter (C++)
    dist_code <- switch(params$dist_name,
      "gaussian"  = 0L,
      "student_t" = 1L,
      "ged"       = 2L
    )
    nu_val <- if (is.null(params$nu)) 0.0 else params$nu

    result <- particle_filter_svp_cpp(
      y_raw = y_vec,
      phi = params$phi,
      sigma_y = params$sigma_y,
      sigma_v = params$sigma_v,
      nu = nu_val,
      dist_code = dist_code,
      delta = params$delta_p,
      M = as.integer(M),
      seed = as.integer(seed),
      del = del
    )

    # BPF doesn't produce smoothed states — use filtered as placeholder
    T_obs <- length(y_vec)
    result$w_smoothed <- result$w_filtered  # no smoother for PF
    result$w_predicted <- rep(NA_real_, T_obs)
    result$zt <- y_vec / (params$sigma_y * exp(result$w_filtered / 2))
    result$zt_smoothed <- result$zt
    result$P_predicted <- rep(NA_real_, T_obs)
    # Construct full p x T state matrix from BPF w_filtered
    # Companion state: xi_t = [w_t, w_{t-1}, ..., w_{t-p+1}]'
    xi_mat <- matrix(NA_real_, nrow = p, ncol = T_obs)
    xi_mat[1, ] <- result$w_filtered
    if (p > 1) {
      for (j in 2:p) {
        xi_mat[j, j:T_obs] <- result$w_filtered[1:(T_obs - j + 1)]
      }
    }
    result$xi_filtered <- xi_mat
    result$xi_smoothed <- xi_mat
  }

  # Build output
  p_out <- params$p
  # Full p x p filtered covariance at T (for forecast MSE recursion)
  if (!is.null(result$P_filt_T)) {
    P_filt_T_mat <- matrix(result$P_filt_T, nrow = p_out, ncol = p_out)
  } else {
    # BPF fallback: diagonal approximation
    P_filt_T_mat <- result$P_filtered[length(result$P_filtered)] * diag(p_out)
  }
  out <- list(
    w_filtered  = as.numeric(result$w_filtered),
    w_smoothed  = as.numeric(result$w_smoothed),
    w_predicted = as.numeric(result$w_predicted),
    zt          = as.numeric(result$zt),
    zt_smoothed = as.numeric(result$zt_smoothed),
    P_filtered  = as.numeric(result$P_filtered),
    P_predicted = as.numeric(result$P_predicted),
    P_filt_T    = P_filt_T_mat,
    xi_filtered = result$xi_filtered,
    xi_smoothed = result$xi_smoothed,
    loglik      = result$loglik,
    method      = method,
    model       = object
  )
  if (!is.null(result$mixture)) out$mixture <- result$mixture
  if (!is.null(result$ESS)) out$ESS <- as.numeric(result$ESS)
  class(out) <- "svp_filter"
  out
}

# =========================================================================== #
# Tests for filtering: CKF, GMKF, helpers, density functions
# =========================================================================== #

# --- Helper: simulate and estimate a model ---
.fit_model <- function(n = 500, phi = 0.95, sigy = 1, sigv = 0.3,
                       errorType = "Gaussian", nu = NULL,
                       leverage = FALSE, rho = 0, seed = 123) {
  set.seed(seed)
  sim <- sim_svp(n, phi = phi, sigy = sigy, sigv = sigv,
                 errorType = errorType, nu = nu,
                 leverage = leverage, rho = rho)
  y <- sim$y
  fit <- svp(y, p = 1, leverage = leverage, errorType = errorType)
  list(sim = sim, y = y, fit = fit)
}


# =========================================================================== #
# Lyapunov solver
# =========================================================================== #

test_that("solve_lyapunov_discrete satisfies X = F X F' + Q", {
  F_mat <- matrix(c(0.95, 0, 1, 0), 2, 2, byrow = TRUE)
  Q <- matrix(c(0.09, 0, 0, 0), 2, 2)
  X <- solve_lyapunov_discrete(F_mat, Q)
  residual <- X - F_mat %*% X %*% t(F_mat) - Q
  expect_true(max(abs(residual)) < 1e-10)
})

test_that("solve_lyapunov_discrete works for p=1", {
  F_mat <- matrix(0.95)
  Q <- matrix(0.09)
  X <- solve_lyapunov_discrete(F_mat, Q)
  expect_equal(as.numeric(X), 0.09 / (1 - 0.95^2), tolerance = 1e-10)
})


# =========================================================================== #
# pged_std (CDF of standardized GED)
# =========================================================================== #

test_that("pged_std: CDF at 0 is 0.5", {
  for (nu in c(0.5, 1, 1.5, 2, 3)) {
    expect_equal(pged_std(0, nu), 0.5, tolerance = 1e-12)
  }
})

test_that("pged_std: monotonically increasing", {
  u_grid <- seq(-5, 5, by = 0.5)
  for (nu in c(1, 1.5, 2)) {
    p_vals <- pged_std(u_grid, nu)
    expect_true(all(diff(p_vals) > 0))
  }
})

test_that("pged_std: limits at +/- infinity", {
  expect_true(pged_std(-20, 1.5) < 0.001)
  expect_true(pged_std(20, 1.5) > 0.999)
})


# =========================================================================== #
# Density functions
# =========================================================================== #

test_that("density_eps_t integrates to 1", {
  for (nu in c(3, 5, 10, 30)) {
    integral <- integrate(density_eps_t, -30, 30, nu = nu)$value
    expect_equal(integral, 1.0, tolerance = 0.01)
  }
})

test_that("density_eps_ged integrates to 1", {
  for (nu in c(0.8, 1.0, 1.5, 2.0)) {
    integral <- integrate(density_eps_ged, -30, 30, nu = nu)$value
    expect_equal(integral, 1.0, tolerance = 0.01)
  }
})


# =========================================================================== #
# fit_ksc_mixture
# =========================================================================== #

test_that("fit_ksc_mixture Gaussian returns KSC table", {
  mix <- fit_ksc_mixture("gaussian")
  expect_equal(length(mix$weights), 7)
  expect_equal(sum(mix$weights), 1.0, tolerance = 1e-10)
  # KSC table approximates raw log(chi^2_1), mean ≈ E[log(chi^2_1)] = psi(1/2)+log(2) ≈ -1.27
  mix_mean <- sum(mix$weights * mix$means)
  expect_equal(mix_mean, digamma(0.5) + log(2), tolerance = 0.05)
  # Variance should be approximately pi^2/2
  mix_var <- sum(mix$weights * (mix$vars + mix$means^2)) - mix_mean^2
  expect_equal(mix_var, pi^2 / 2, tolerance = 0.5)
})

test_that("fit_ksc_mixture Student-t: valid moments", {
  mix <- fit_ksc_mixture("student_t", nu = 5, K = 5, n_sample = 50000, seed = 42)
  expect_equal(length(mix$weights), 5)
  expect_equal(sum(mix$weights), 1.0, tolerance = 1e-4)
  mix_mean <- sum(mix$weights * mix$means)
  expect_true(abs(mix_mean) < 0.5)
  sigma_eps2_t <- psigamma(0.5, 1) + psigamma(5 / 2, 1)
  mix_var <- sum(mix$weights * (mix$vars + mix$means^2)) - mix_mean^2
  expect_equal(mix_var, sigma_eps2_t, tolerance = 1.5)
})

test_that("fit_ksc_mixture GED: valid moments", {
  mix <- fit_ksc_mixture("ged", nu = 1.5, K = 5, n_sample = 50000, seed = 42)
  expect_equal(length(mix$weights), 5)
  expect_equal(sum(mix$weights), 1.0, tolerance = 1e-4)
  mix_mean <- sum(mix$weights * mix$means)
  expect_true(abs(mix_mean) < 0.5)
})


# =========================================================================== #
# filter_svp — CKF (corrected Kalman)
# =========================================================================== #

test_that("filter_svp returns correct class and components (Gaussian)", {
  m <- .fit_model()
  filt <- filter_svp(m$fit)
  expect_s3_class(filt, "svp_filter")
  expect_true(all(c("w_filtered", "w_smoothed", "zt", "zt_smoothed",
                     "P_filtered", "P_predicted", "xi_filtered", "xi_smoothed",
                     "loglik", "method", "model") %in% names(filt)))
  expect_equal(filt$method, "corrected")
  expect_equal(length(filt$w_filtered), 500)
  expect_equal(length(filt$P_filtered), 500)
})

test_that("filter_svp CKF returns consistent filtered and smoothed output", {
  m <- .fit_model()
  filt <- filter_svp(m$fit)
  # Smoothed and filtered should be same length
  expect_equal(length(filt$w_filtered), length(filt$w_smoothed))
  # Both should be finite
  expect_true(all(is.finite(filt$w_filtered)))
  expect_true(all(is.finite(filt$w_smoothed)))
})

test_that("filter_svp: no NaN/Inf in output (all distributions)", {
  # Gaussian
  m <- .fit_model(errorType = "Gaussian")
  filt <- filter_svp(m$fit)
  expect_true(all(is.finite(filt$w_filtered)))
  expect_true(all(is.finite(filt$w_smoothed)))
  expect_true(is.finite(filt$loglik))

  # Student-t
  m_t <- .fit_model(errorType = "Student-t", nu = 5)
  filt_t <- filter_svp(m_t$fit)
  expect_true(all(is.finite(filt_t$w_filtered)))
  expect_true(is.finite(filt_t$loglik))

  # GED
  m_g <- .fit_model(errorType = "GED", nu = 1.5)
  filt_g <- filter_svp(m_g$fit)
  expect_true(all(is.finite(filt_g$w_filtered)))
  expect_true(is.finite(filt_g$loglik))
})

test_that("filter_svp: filtered w tracks true w on simulated data", {
  set.seed(42)
  sim <- sim_svp(2000, phi = 0.95, sigy = 1, sigv = 0.5,
                 leverage = TRUE, rho = 0)  # returns list with $h (=w)
  fit <- svp(sim$y, p = 1)
  filt <- filter_svp(fit)
  cor_val <- cor(filt$w_smoothed, sim$h)
  # Smoothed w should correlate with true w (higher sigma_v = more signal)
  expect_true(cor_val > 0.3)
})

test_that("filter_svp: P_filtered is positive", {
  m <- .fit_model()
  filt <- filter_svp(m$fit)
  expect_true(all(filt$P_filtered > 0))
  expect_true(all(filt$P_predicted > 0))
})

test_that("filter_svp: leverage affects prediction", {
  m <- .fit_model(leverage = TRUE, rho = -0.5)
  filt <- filter_svp(m$fit)
  expect_true(all(is.finite(filt$w_filtered)))
})


# =========================================================================== #
# filter_svp — GMKF (Gaussian mixture)
# =========================================================================== #

test_that("GMKF with K=1 approximates CKF", {
  m <- .fit_model(n = 300)
  filt_ckf <- filter_svp(m$fit, method = "corrected")
  # K=1 mixture should be very close to CKF
  # (Not exact match due to different y_star centering, but correlation should be high)
  filt_gmkf1 <- filter_svp(m$fit, method = "mixture", K = 1)
  cor_val <- cor(filt_ckf$w_filtered, filt_gmkf1$w_filtered)
  expect_true(cor_val > 0.95)
})

test_that("GMKF K=7 runs without error (Gaussian)", {
  m <- .fit_model(n = 300)
  filt <- filter_svp(m$fit, method = "mixture", K = 7)
  expect_s3_class(filt, "svp_filter")
  expect_equal(filt$method, "mixture")
  expect_true(all(is.finite(filt$w_filtered)))
  expect_true(is.finite(filt$loglik))
})

test_that("GMKF runs without error (Student-t)", {
  m <- .fit_model(n = 200, errorType = "Student-t", nu = 5)
  filt <- filter_svp(m$fit, method = "mixture", K = 3)
  expect_true(all(is.finite(filt$w_filtered)))
})

test_that("GMKF runs without error (GED)", {
  m <- .fit_model(n = 200, errorType = "GED", nu = 1.5)
  filt <- filter_svp(m$fit, method = "mixture", K = 3)
  expect_true(all(is.finite(filt$w_filtered)))
})


# =========================================================================== #
# Stress tests
# =========================================================================== #

test_that("filter_svp: very persistent SV(1) doesn't diverge", {
  set.seed(99)
  y <- sim_svp(500, phi = 0.99, sigy = 1, sigv = 0.1)$y
  fit <- svp(y, p = 1)
  filt <- filter_svp(fit)
  expect_true(all(is.finite(filt$w_filtered)))
  expect_true(max(abs(filt$w_filtered)) < 50)
})

test_that("filter_svp: heavy tails Student-t nu=3 doesn't collapse", {
  m <- .fit_model(n = 300, errorType = "Student-t", nu = 3)
  filt <- filter_svp(m$fit, method = "mixture", K = 3)
  expect_true(all(is.finite(filt$w_filtered)))
})

test_that("filter_svp: SV(2) companion form works", {
  set.seed(77)
  y <- sim_svp(500, phi = c(0.2, 0.63), sigy = 1, sigv = 0.5)$y
  fit <- svp(y, p = 2)
  filt <- filter_svp(fit)
  expect_true(all(is.finite(filt$w_filtered)))
  expect_equal(nrow(filt$xi_filtered), 2)
})

test_that("filter_svp: short series T=50 doesn't crash", {
  set.seed(55)
  y <- sim_svp(50, phi = 0.9, sigy = 1, sigv = 0.3)$y
  fit <- svp(y, p = 1)
  filt <- filter_svp(fit)
  expect_equal(length(filt$w_filtered), 50)
})


# =========================================================================== #
# S3 methods
# =========================================================================== #

test_that("print.svp_filter produces output", {
  m <- .fit_model(n = 200)
  filt <- filter_svp(m$fit)
  expect_output(print(filt), "Filter")
})

test_that("plot.svp_filter runs without error", {
  m <- .fit_model(n = 200)
  filt <- filter_svp(m$fit)
  expect_silent(suppressWarnings(plot(filt)))
})


# =========================================================================== #
# Regression tests: P_filt_T full matrix (Bug 3 fix) and pinv stability (Bug 4)
# =========================================================================== #

test_that("filter_svp returns P_filt_T as a p x p matrix (p=1)", {
  set.seed(42)
  y <- sim_svp(300, phi = 0.95, sigy = 1, sigv = 0.3)$y
  fit <- svp(y, p = 1)
  filt <- filter_svp(fit, method = "corrected")
  expect_true("P_filt_T" %in% names(filt))
  expect_equal(dim(filt$P_filt_T), c(1L, 1L))
  expect_true(filt$P_filt_T[1, 1] > 0)
})

test_that("filter_svp returns full P_filt_T with non-zero off-diagonals (p=2)", {
  set.seed(42)
  y <- sim_svp(500, phi = c(0.20, 0.63), sigy = 1, sigv = 1)$y
  fit <- svp(y, p = 2)
  filt_ckf  <- filter_svp(fit, method = "corrected")
  filt_gmkf <- filter_svp(fit, method = "mixture")
  # Dimension check
  expect_equal(dim(filt_ckf$P_filt_T),  c(2L, 2L))
  expect_equal(dim(filt_gmkf$P_filt_T), c(2L, 2L))
  # Off-diagonal should be non-zero for p=2 (was zero under diagonal approx)
  expect_true(abs(filt_ckf$P_filt_T[1, 2])  > 1e-6)
  expect_true(abs(filt_gmkf$P_filt_T[1, 2]) > 1e-6)
  # Matrix should be symmetric and positive definite
  expect_true(isSymmetric(filt_ckf$P_filt_T, tol = 1e-10))
  expect_true(all(eigen(filt_ckf$P_filt_T, only.values = TRUE)$values > 0))
})

test_that("RTS smoother (pinv) produces no NaN/Inf on near-unit-root SV(2)", {
  set.seed(42)
  y <- sim_svp(400, phi = c(0.50, 0.49), sigy = 1, sigv = 0.5)$y
  fit <- svp(y, p = 2)
  filt_ckf  <- filter_svp(fit, method = "corrected")
  filt_gmkf <- filter_svp(fit, method = "mixture")
  expect_false(any(is.nan(filt_ckf$w_smoothed)  | is.infinite(filt_ckf$w_smoothed)))
  expect_false(any(is.nan(filt_gmkf$w_smoothed) | is.infinite(filt_gmkf$w_smoothed)))
})

# =========================================================================== #
# Regression: BPF xi_filtered dimensions for p>=2 (Bug F fix, 2026-04-03)
# =========================================================================== #

test_that("BPF xi_filtered has correct p x T dimensions for p=2", {
  skip_on_cran()
  set.seed(42)
  y <- sim_svp(300, phi = c(0.20, 0.63), sigy = 1, sigv = 1)$y
  fit <- svp(y, p = 2)
  filt <- filter_svp(fit, method = "particle", M = 100)
  expect_equal(nrow(filt$xi_filtered), 2L)
  expect_equal(ncol(filt$xi_filtered), 300L)
  # Row 1 should match w_filtered
  expect_equal(filt$xi_filtered[1, ], filt$w_filtered)
  # Row 2 is lagged: xi[2,t] = w_filtered[t-1]
  expect_equal(filt$xi_filtered[2, 2:300], filt$w_filtered[1:299])
})

#' Multi-Step Ahead Volatility Forecast
#'
#' Applies Kalman filtering/smoothing to an estimated SV(p) model and produces
#' multi-step ahead volatility forecasts with uncertainty quantification.
#'
#' @param object An \code{"svp"}, \code{"svp_t"}, or \code{"svp_ged"} object
#'   from \code{\link{svp}}.
#' @param H Integer. Maximum forecast horizon. Default 1.
#' @param output Character. Primary output scale: \code{"log-variance"} (default,
#'   native log-volatility w_h), \code{"variance"} (conditional variance
#'   \eqn{\sigma^2_{T+h|T}}), or \code{"volatility"} (conditional std dev
#'   \eqn{\sigma_{T+h|T}}). All three are always computed and stored; this
#'   controls which is used by \code{print} and \code{plot} methods.
#' @param filter_method Character. Filter method: \code{"corrected"} (default),
#'   \code{"mixture"} (GMKF), or \code{"particle"} (BPF).
#' @param proxy Character. Leverage proxy for the filter and the h=1 forecast
#'   shift. \code{"bayes_optimal"} (default) uses the posterior mean
#'   \eqn{E[\zeta_{t-1} \mid u_{t-1}]} for Student-t leverage;
#'   \code{"u"} reproduces the paper-faithful proxy of Remark 3.5
#'   (\eqn{\hat{z}_{t-1} = \hat{u}_{t-1}}). Has no effect for Gaussian or
#'   GED leverage. See \code{\link{filter_svp}} for details.
#' @param K Integer. Number of mixture components for GMKF. Default 7.
#' @param M Integer. Number of particles for BPF. Default 1000.
#' @param seed Integer. Random seed for BPF. Default 42.
#' @param del Numeric. Small constant for log transformation. Default \code{1e-10}.
#'
#' @return An object of class \code{"svp_forecast"}, a list containing:
#' \describe{
#'   \item{w_forecasted}{Primary forecast (scale determined by \code{output}).}
#'   \item{log_var_forecast}{Log-volatility forecasts \eqn{w_{T+h|T}}.}
#'   \item{var_forecast}{Conditional variance forecasts \eqn{\sigma^2_{T+h|T}}.}
#'   \item{vol_forecast}{Conditional volatility forecasts \eqn{\sigma_{T+h|T}}.}
#'   \item{P_forecast}{Forecast MSE \eqn{P_{T+h|T}} for each horizon.}
#'   \item{w_estimated}{Filtered log-volatility.}
#'   \item{w_smoothed}{Smoothed log-volatility.}
#'   \item{zt}{Filtered standardized residuals.}
#'   \item{zt_smoothed}{Smoothed standardized residuals.}
#'   \item{ys}{Demeaned log-squared returns.}
#'   \item{mdl}{The estimated model object.}
#'   \item{H}{The forecast horizon.}
#'   \item{output}{The chosen output scale.}
#'   \item{filter_output}{The \code{"svp_filter"} object from filtering.}
#' }
#'
#' @examples
#' \donttest{
#' sim <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2,
#'                leverage = TRUE, rho = -0.3)
#' fit <- svp(sim$y, p = 1, leverage = TRUE)
#' fc <- forecast_svp(fit, H = 10)
#' plot(fc)
#' }
#'
#' @export
forecast_svp <- function(object, H = 1,
                         output = c("log-variance", "variance", "volatility"),
                         filter_method = "corrected",
                         proxy = c("bayes_optimal", "u"),
                         K = 7, M = 1000, seed = 42, del = 1e-10) {
  if (!inherits(object, c("svp", "svp_t", "svp_ged"))) {
    stop("object must be an svp/svp_t/svp_ged model from svp(). ",
         "Usage: fit <- svp(y, ...); fc <- forecast_svp(fit, H = 10)")
  }
  if (!is.numeric(H) || length(H) != 1L || H < 1L)
    stop("'H' must be a positive integer (>= 1).")
  H <- as.integer(H)
  if (!is.numeric(K) || length(K) != 1L || K < 1L)
    stop("'K' must be a positive integer (>= 1).")
  if (!is.numeric(M) || length(M) != 1L || M < 1L)
    stop("'M' must be a positive integer (>= 1).")
  if (!is.numeric(del) || length(del) != 1L || del <= 0)
    stop("'del' must be a positive number.")
  output <- match.arg(output)
  filter_method <- match.arg(filter_method, c("corrected", "mixture", "particle"))
  proxy <- match.arg(proxy)

  mdl <- object
  y_vec <- as.numeric(mdl$y)
  phi <- mdl$phi
  p_len <- length(phi)
  delta_p <- if (is.null(mdl$rho) || is.na(mdl$rho)) 0 else mdl$rho
  sigma_y <- mdl$sigy
  sigma_v <- mdl$sigv

  # Filter (forward proxy choice — relevant only for Student-t leverage)
  filt <- filter_svp(mdl, method = filter_method, proxy = proxy,
                     K = K, M = M, seed = seed, del = del)
  Tsize <- length(filt$w_filtered)

  # Extract final state and residual
  xi_T <- filt$xi_filtered[, Tsize]
  zT_raw <- filt$zt[Tsize]
  # Apply leverage proxy transform consistent with the filter's choice:
  #   Gaussian: zeta = u (exact)
  #   GED:      zeta = Phi^{-1}(F_GED(u; nu)) (exact via copula)
  #   Student-t with proxy="u": zeta = u (paper-faithful, Remark 3.5)
  #   Student-t with proxy="bayes_optimal": zeta = E[zeta|u]
  zT <- .leverage_zeta_proxy_R(zT_raw, mdl, proxy)

  # Build companion matrix
  Fmat <- companionMat(t(phi), p_len, 1)
  h_vec <- c(1, rep(0, p_len - 1))
  r_vec <- h_vec

  # State noise covariance — horizon-dependent under leverage.
  #
  # h = 1: predicting w_{T+1} from F_T.  z_T is observed (computed from y_T
  #        and the filtered h_T), so the conditional Var[z_T | F_T] = 0 under
  #        the u-proxy convention.  Per SVHT Remark 3.5:
  #            Q_1 = sigma_v^2 * (1 - delta_p^2).
  #
  # h >= 2: predicting w_{T+h} from F_T.  z_{T+h-1} is unobserved future
  #        noise with marginal Var = 1.  No further leverage shocks are
  #        observed, so:
  #            Q_h = sigma_v^2  (the marginal state-innovation variance).
  #
  # When delta_p = 0 both reduce to sigma_v^2 (no-leverage case).
  Q_h1_scalar   <- if (delta_p == 0) sigma_v^2 else sigma_v^2 * (1 - delta_p^2)
  Q_hge2_scalar <- sigma_v^2
  Q_h1   <- Q_h1_scalar   * (r_vec %*% t(r_vec))
  Q_hge2 <- Q_hge2_scalar * (r_vec %*% t(r_vec))

  # E[u^2] for variance forecast
  Eu2 <- 1.0
  if (inherits(mdl, "svp_t") && !is.null(mdl$v) && is.finite(mdl$v) && mdl$v > 2) {
    Eu2 <- mdl$v / (mdl$v - 2)
  }

  # h-step forecasts
  tilde_delta_p <- delta_p * sigma_v
  eta_T <- c(zT, rep(0, p_len - 1))

  # Get final filtered covariance for MSE recursion
  # Use full p x p matrix from filter output (exact for p>=2)
  P_T <- filt$P_filt_T

  log_var_fc <- numeric(H)
  P_fc <- numeric(H)
  var_fc <- numeric(H)
  vol_fc <- numeric(H)

  xi_h <- xi_T
  P_h <- P_T

  for (h in 1:H) {
    # State prediction (no leverage feedback beyond h=1)
    if (h == 1) {
      xi_h <- as.numeric(Fmat %*% xi_T) + tilde_delta_p * eta_T
    } else {
      xi_h <- as.numeric(Fmat %*% xi_h)
    }
    # Horizon-dependent Q: conditional at h=1 (z_T observed), marginal at h>=2
    Q_use <- if (h == 1L) Q_h1 else Q_hge2
    P_h <- Fmat %*% P_h %*% t(Fmat) + Q_use

    w_h <- sum(h_vec * xi_h)
    P_h_11 <- as.numeric(t(h_vec) %*% P_h %*% h_vec)

    log_var_fc[h] <- w_h
    P_fc[h] <- P_h_11
    # Variance forecast: sigma_y^2 * E[u^2] * E[exp(w)] where w ~ N(w_h, P_h_11)
    var_fc[h] <- sigma_y^2 * Eu2 * exp(w_h + P_h_11 / 2)
    vol_fc[h] <- sqrt(var_fc[h])
  }

  # Primary output
  w_forecasted <- switch(output,
    "log-variance" = log_var_fc,
    "variance"     = var_fc,
    "volatility"   = vol_fc
  )

  # Build output (backward compatible fields)
  ys <- log(y_vec^2 + del) - mdl$mu
  out <- list(
    w_forecasted    = w_forecasted,
    log_var_forecast = log_var_fc,
    var_forecast    = var_fc,
    vol_forecast    = vol_fc,
    P_forecast      = P_fc,
    w_estimated     = matrix(filt$w_filtered, ncol = 1),
    w_smoothed      = matrix(filt$w_smoothed, ncol = 1),
    zt              = matrix(filt$zt, ncol = 1),
    zt_smoothed     = matrix(filt$zt_smoothed, ncol = 1),
    ys              = as.numeric(ys),
    mdl             = mdl,
    H               = H,
    output          = output,
    filter_output   = filt,
    call            = match.call()
  )
  class(out) <- "svp_forecast"
  return(out)
}


# Internal: R-side mirror of the C++ leverage_zeta_proxy helper used in
# kalman_filter.cpp.  Computes the proxy zeta_hat from observed u given the
# fit's error type and the proxy choice.  Used by forecast_svp() to apply
# the proxy transform to zT before it enters the leverage shift at h=1.
#' @keywords internal
.leverage_zeta_proxy_R <- function(u, fit, proxy) {
  if (inherits(fit, "svp")) return(u)        # Gaussian: zeta = u exactly
  nu <- as.numeric(fit$v)
  if (inherits(fit, "svp_ged")) {
    # GED: zeta = Phi^{-1}(F_GED(u; nu)) deterministic
    a <- exp(0.5 * (lgamma(1 / nu) - lgamma(3 / nu)))
    p <- if (u >= 0) 0.5 + 0.5 * pgamma((u / a)^nu, shape = 1 / nu)
         else        0.5 - 0.5 * pgamma((-u / a)^nu, shape = 1 / nu)
    p <- min(max(p, 1e-15), 1 - 1e-15)
    return(qnorm(p))
  }
  # Student-t (svp_t)
  if (proxy == "u") return(u)
  # Bayes-optimal: E[zeta | u] = u * sqrt(2/(nu+u^2)) * Gamma((nu+2)/2)/Gamma((nu+1)/2)
  factor <- sqrt(2 / (nu + u^2)) *
            exp(lgamma((nu + 2) / 2) - lgamma((nu + 1) / 2))
  u * factor
}

# S3 class constructors and validators for wARMASVp
# Classes are created by their respective user-facing function svp()
# This file contains any shared validation logic.

#' @keywords internal
validate_svp <- function(x) {
  stopifnot(is.list(x))
  stopifnot("phi" %in% names(x))
  stopifnot("sigv" %in% names(x))
  stopifnot("sigy" %in% names(x))
  stopifnot("mu" %in% names(x))
  invisible(x)
}

---
title: "Getting Started with wARMASVp"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with wARMASVp}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6, fig.height = 4
)
```

The **wARMASVp** package provides closed-form estimation, simulation, hypothesis
testing, filtering, and forecasting for higher-order stochastic volatility SV(p)
models. It supports Gaussian, Student-t, and Generalized Error Distribution (GED)
innovations, with optional leverage effects.

## The SV(p) Model

The stochastic volatility model of order $p$ is:

$$y_t = \sigma_y \exp(w_t / 2)\, z_t$$
$$w_t = \phi_1 w_{t-1} + \cdots + \phi_p w_{t-p} + \sigma_v v_t$$

where $z_t$ is an i.i.d. innovation (Gaussian, Student-t, or GED) and
$v_t \sim N(0,1)$ drives the log-volatility.

## Simulation and Estimation

### Gaussian SV(1)

```{r gaussian-sv1}
library(wARMASVp)
set.seed(123)

# Simulate
sim <- sim_svp(2000, phi = 0.95, sigy = 1, sigv = 0.3)
y <- sim$y

# Estimate
fit <- svp(y, p = 1, J = 10)
summary(fit)
```

### Gaussian SV(2)

```{r gaussian-sv2}
y2 <- sim_svp(2000, phi = c(0.20, 0.63), sigy = 1, sigv = 0.5)$y
fit2 <- svp(y2, p = 2, J = 10)
summary(fit2)
```

### Student-t Innovations

```{r student-t}
yt <- sim_svp(2000, phi = 0.90, sigy = 1, sigv = 0.3,
              errorType = "Student-t", nu = 5)$y
fit_t <- svp(yt, p = 1, errorType = "Student-t")
summary(fit_t)
```

### GED Innovations

```{r ged}
yg <- sim_svp(2000, phi = 0.90, sigy = 1, sigv = 0.3,
              errorType = "GED", nu = 1.5)$y
fit_ged <- svp(yg, p = 1, errorType = "GED")
summary(fit_ged)
```

## Leverage Effects

When return and volatility shocks are correlated ($\rho \neq 0$), use the
leverage option:

```{r leverage}
sim_lev <- sim_svp(2000, phi = 0.95, sigy = 1, sigv = 0.3,
                   leverage = TRUE, rho = -0.5)
fit_lev <- svp(sim_lev$y, p = 1, leverage = TRUE)
summary(fit_lev)
```

Leverage is supported for all three distributions:

```{r leverage-t}
sim_lev_t <- sim_svp(2000, phi = 0.90, sigy = 1, sigv = 0.3,
                     errorType = "Student-t", nu = 5,
                     leverage = TRUE, rho = -0.5)
fit_lev_t <- svp(sim_lev_t$y, p = 1, errorType = "Student-t", leverage = TRUE)
summary(fit_lev_t)
```

## Hypothesis Testing

The package provides Local Monte Carlo (LMC) and Maximized Monte Carlo (MMC)
tests based on Dufour (2006).

### AR Order Testing

Test whether SV(1) is sufficient versus SV(2):

```{r test-ar}
y_test <- sim_svp(2000, phi = 0.95, sigy = 1, sigv = 0.3)$y

# H0: SV(1) vs H1: SV(2) — should not reject
test_ar <- lmc_ar(y_test, p_null = 1, p_alt = 2, N = 49)
print(test_ar)
```

### Leverage Testing

```{r test-lev}
test_lev <- lmc_lev(y_test, p = 1, N = 49, Amat = "Weighted")
print(test_lev)
```

### Distribution Testing

Test for heavy tails against a specific null value of the tail parameter:

```{r test-dist}
# Test H0: nu = 10 (mild tails) on Student-t data with true nu = 5
test_t <- lmc_t(yt, nu_null = 10, N = 49, Amat = "Weighted")
print(test_t)

# Directional test: H1: nu < 10 (heavier tails than null)
test_t_dir <- lmc_t(yt, nu_null = 10, N = 49, Amat = "Weighted", direction = "less")
print(test_t_dir)
```

## AR Order Selection via Information Criteria

In addition to the LMC/MMC pairwise AR-order test above, the package selects the
SV(p) lag order by information criteria. `svp_IC()` computes the criteria for a
single fitted model; `svp_AR_order()` sweeps over `p = 1, ..., pmax` and reports
the argmin for each criterion.

```{r ic-single}
fit_ic <- svp(y_test, p = 2, J = 10)
svp_IC(fit_ic)
```

```{r ic-sweep}
sel <- svp_AR_order(y_test, pmax = 4, J = 10)
sel$argmin
```

Four criteria are returned by default, spanning two estimation families and two
penalty philosophies: `BIC_Kalman` / `AIC_Kalman` use the QML log-likelihood from
the Gaussian mixture Kalman filter, while `BIC_HR` / `AIC_HR` use a two-stage
Hannan-Rissanen ARMA(p, p) residual variance. Additional criteria (`AICc_Kalman`,
`BIC_Whittle`, and the Yule-Walker variants) are available opt-in via the
`criteria` argument. Both functions read `errorType` and `leverage` from the
fitted model, so heavy-tailed and leverage specifications are handled
automatically. See Ahsan, Dufour, and Rodriguez-Rondon (2026) for the
theoretical motivation and consistency simulations.

## Filtering

Three methods are available via `filter_svp()`, which takes a fitted model:

- **Corrected Kalman Filter (CKF)**: Gaussian approximation, fast.
- **Gaussian Mixture Kalman Filter (GMKF)**: KSC (1998) 7-component mixture.
  Recommended — dominates CKF even under Gaussianity.
- **Bootstrap Particle Filter (BPF)**: Exact density weights. Gold-standard benchmark.

```{r filtering}
# Fit model
fit_filt <- svp(y, p = 1, J = 10)

# GMKF (recommended)
filt <- filter_svp(fit_filt, method = "mixture")
plot(filt)
```

## Forecasting

Multi-step ahead volatility forecasts using Kalman filtering. Pass a fitted
model object from `svp()`:

```{r forecast}
fit_fc <- svp(sim_lev$y, p = 1, leverage = TRUE)
fc <- forecast_svp(fit_fc, H = 20)
plot(fc)
```

Output scales can be chosen: `"log-variance"` (default), `"variance"`, or
`"volatility"`. All three are always computed and stored.

## References

- Ahsan, M. N. and Dufour, J.-M. (2021). Simple estimators and inference for
  higher-order stochastic volatility models. *Journal of Econometrics*,
  224(1), 181-197.
- Ahsan, M. N., Dufour, J.-M., and Rodriguez-Rondon, G. (2025). Estimation and
  inference for higher-order stochastic volatility models with leverage.
  *Journal of Time Series Analysis*, 46(6), 1064-1084.
- Ahsan, M. N., Dufour, J.-M., and Rodriguez-Rondon, G. (2026). Estimation and
  inference for stochastic volatility models with heavy-tailed distributions.
  Bank of Canada Staff Working Paper 2026-8.
  [doi:10.34989/swp-2026-8](https://doi.org/10.34989/swp-2026-8)
- Dufour, J.-M. (2006). Monte Carlo tests with nuisance parameters: A general
  approach to finite-sample inference and nonstandard asymptotics.
  *Journal of Econometrics*, 133(2), 443-477.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{companionMat}
\alias{companionMat}
\title{Construct Companion Matrix for AR(p) Process}
\usage{
companionMat(phi, p, q)
}
\arguments{
\item{phi}{Numeric vector of AR coefficients (length p).}

\item{p}{Integer. Order of the AR process.}

\item{q}{Integer. Dimension of each block (typically 1 for univariate).}
}
\value{
A \code{(q*p) x (q*p)} companion matrix.
}
\description{
Builds the companion matrix representation for an AR(p) process, which is
used in the state-space formulation of SV(p) models.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svp_internal.R
\name{correction_factor_t}
\alias{correction_factor_t}
\title{Correction factor C_t(nu) for Student-t leverage estimation}
\usage{
correction_factor_t(nu)
}
\arguments{
\item{nu}{Degrees of freedom (nu > 1)}
}
\value{
C_t(nu), always > 1 for finite nu (approaches 1 as nu -> Inf)
}
\description{
Under scale mixture \eqn{u_t = z_t \lambda_t^{-1/2}},
\eqn{C_t(\nu) = [E[\lambda^{-1/2}]]^2 = (\nu/2) [\Gamma((\nu-1)/2) / \Gamma(\nu/2)]^2}.
Exact, parameter-free.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svp_internal.R
\name{qged_std}
\alias{qged_std}
\title{Quantile function for standardized GED(nu) with Var = 1
Uses the relationship between GED CDF and incomplete gamma function.}
\usage{
qged_std(p, nu)
}
\arguments{
\item{p}{Probability (0 < p < 1). Clamped to [1e-15, 1-1e-15] internally.}

\item{nu}{GED shape parameter (nu > 0)}
}
\value{
Quantile value
}
\description{
Quantile function for standardized GED(nu) with Var = 1
Uses the relationship between GED CDF and incomplete gamma function.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kalman.R
\name{filter_svp}
\alias{filter_svp}
\title{Filter Latent Volatility from an Estimated SV(p) Model}
\usage{
filter_svp(
  object,
  method = c("corrected", "mixture", "particle"),
  K = 7,
  M = 1000,
  seed = 42,
  del = 1e-10
)
}
\arguments{
\item{object}{An \code{"svp"}, \code{"svp_t"}, or \code{"svp_ged"} object
from \code{\link{svp}}.}

\item{method}{Character. Filter method: \code{"corrected"} (default) for
standard Kalman with distribution-specific \eqn{\sigma_\varepsilon^2(\nu)},
\code{"mixture"} for the Gaussian Mixture Kalman Filter (GMKF), or
\code{"particle"} for the Bootstrap Particle Filter (BPF).}

\item{K}{Integer. Number of mixture components for GMKF. Default 7.}

\item{M}{Integer. Number of particles for BPF. Default 1000.}

\item{seed}{Integer. Random seed for BPF. Default 42.}

\item{del}{Numeric. Small constant for log transformation. Default \code{1e-10}.}
}
\value{
An object of class \code{"svp_filter"}, a list containing:
\describe{
  \item{w_filtered}{Filtered log-volatility (T-vector).}
  \item{w_smoothed}{Smoothed log-volatility (T-vector).}
  \item{zt}{Filtered standardized residuals.}
  \item{zt_smoothed}{Smoothed standardized residuals.}
  \item{P_filtered}{Filtered MSE of first state component.}
  \item{P_predicted}{Predicted MSE of first state component.}
  \item{xi_filtered}{Full filtered state vectors (p x T matrix).}
  \item{xi_smoothed}{Full smoothed state vectors (p x T matrix).}
  \item{loglik}{Approximate log-likelihood.}
  \item{method}{Filter method used.}
  \item{model}{The input model object.}
}
}
\description{
Applies Kalman filtering (corrected or Gaussian mixture) and RTS smoothing
to extract the latent log-volatility process from an estimated SV(p) model.
}
\examples{
\donttest{
y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2)$y
fit <- svp(y, p = 1)
filt <- filter_svp(fit)
plot(filt$w_smoothed, type = "l")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estim.R
\name{svp}
\alias{svp}
\title{Estimate a Stochastic Volatility Model}
\usage{
svp(
  y,
  p = 1,
  J = 10,
  leverage = FALSE,
  errorType = "Gaussian",
  rho_type = "pearson",
  del = 1e-10,
  trunc_lev = TRUE,
  wDecay = FALSE,
  logNu = FALSE,
  sigvMethod = "factored",
  winsorize_eps = 0
)
}
\arguments{
\item{y}{Numeric vector. Observed returns (e.g., de-meaned log returns).}

\item{p}{Integer. Order of the volatility process. Default is 1.}

\item{J}{Integer. Winsorizing parameter controlling the number of
autocovariance blocks used. Default is 10.}

\item{leverage}{Logical. If \code{TRUE}, estimate leverage parameter
\eqn{\rho}. Default is \code{FALSE}.}

\item{errorType}{Character. Error distribution: \code{"Gaussian"} (default),
\code{"Student-t"}, or \code{"GED"}.}

\item{rho_type}{Character. Correlation type for leverage estimation. One of
\code{"pearson"} (default), \code{"kendall"}, or \code{"both"}.}

\item{del}{Numeric. Small constant for log transformation:
\eqn{\log(y_t^2 + \delta)}. Default is \code{1e-10}.}

\item{trunc_lev}{Logical. If \code{TRUE}, truncate the estimated leverage
parameter to \code{[-0.999, 0.999]}. Default is \code{TRUE}. Setting to
\code{FALSE} can reduce bias in some cases but may yield estimates
outside the parameter space.}

\item{wDecay}{Logical. Use linearly decaying weights in the WLS estimation.
Default is \code{FALSE}.}

\item{logNu}{Logical. Solve for \eqn{\nu} in log-space for numerical
stability (Student-t only). Default is \code{FALSE}.}

\item{sigvMethod}{Character. Method for estimating \eqn{\sigma_v}. One of:
\code{"factored"} (default) — factored-variance estimator (recommended;
dominates RMSE in most settings, see ADRR 2025);
\code{"direct"} — direct variance decomposition;
\code{"hybrid"} — AD2021 closed-form for \eqn{p = 1}, falls back to
\code{"direct"} for \eqn{p \ge 2} (Student-t and GED only).}

\item{winsorize_eps}{Integer. Number of extreme autocovariance lags to
winsorize (\code{0} = none). Used in Student-t and GED
\eqn{\sigma_\varepsilon^2} estimation. Default \code{0}.}
}
\value{
Depending on \code{errorType}:
\itemize{
  \item \code{"Gaussian"}: An object of class \code{"svp"} (see below).
  \item \code{"Student-t"}: An object of class \code{"svp_t"}.
  \item \code{"GED"}: An object of class \code{"svp_ged"}.
}

The \code{"svp"} class contains:
\describe{
  \item{mu}{Mean of \eqn{\log(y_t^2 + \delta)}.}
  \item{phi}{Numeric vector of AR coefficients.}
  \item{sigv}{Standard deviation of volatility innovations.}
  \item{sigy}{Unconditional standard deviation.}
  \item{rho}{Leverage parameter (if estimated, otherwise \code{NA}).}
  \item{y}{The original data.}
  \item{p, J}{Model order and winsorizing parameter.}
  \item{errorType}{The error distribution used.}
  \item{call}{The matched call.}
}
}
\description{
Master estimation function for SV(p) models using the Winsorized ARMA-SV
(W-ARMA-SV) method. Supports Gaussian, Student-t, and GED error
distributions, with optional leverage effects.
}
\details{
The model is:
\deqn{y_t = \sigma_y \exp(w_t / 2) z_t}
\deqn{w_t = \phi_1 w_{t-1} + \cdots + \phi_p w_{t-p} + \sigma_v v_t}

where \eqn{z_t} follows a distribution specified by \code{errorType}
(Gaussian, Student-t, or GED), and \eqn{v_t} is i.i.d. standard normal.
When \code{leverage = TRUE}, the correlation between \eqn{z_t} and
\eqn{v_t} is estimated as \eqn{\rho}.

For Student-t errors with leverage, the correction factor
\eqn{C_t(\nu)} from the scale-mixture representation is applied.
For GED errors with leverage, the exact implicit equation is solved
via 1D root-finding with Gauss-Hermite quadrature.
}
\examples{
\donttest{
# Gaussian SV(1) without leverage (default)
y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2)$y
fit <- svp(y)
summary(fit)

# With leverage
y2 <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2, leverage = TRUE, rho = -0.3)$y
fit2 <- svp(y2, leverage = TRUE)
coef(fit2)

# Student-t errors
y3 <- sim_svp(1000, phi = 0.9, sigy = 1, sigv = 0.2, errorType = "Student-t", nu = 5)$y
fit3 <- svp(y3, errorType = "Student-t")
summary(fit3)
}

}
\references{
Ahsan, N. and Dufour, J.-M. (2021). Simple estimators and inference for
higher-order stochastic volatility models. \emph{Journal of Econometrics},
224(1), 181-197.

Ahsan, N., Dufour, J.-M., and Rodriguez Rondon, G. (2025). Estimation and
inference for stochastic volatility models with heavy-tailed distributions.
}
\seealso{
\code{\link{svpSE}} for standard errors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kalman.R
\name{density_eps_t}
\alias{density_eps_t}
\title{Density of Centered log-F(1,nu) Measurement Noise (Student-t)}
\usage{
density_eps_t(y, nu)
}
\arguments{
\item{y}{Numeric vector. Evaluation points (centered: E[eps] = 0).}

\item{nu}{Numeric. Student-t degrees of freedom.}
}
\value{
Density values.
}
\description{
Density of Centered log-F(1,nu) Measurement Noise (Student-t)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svp_test.R
\name{lmc_lev}
\alias{lmc_lev}
\title{LMC Test for Leverage in SV(p) Models}
\usage{
lmc_lev(
  y,
  p = 1,
  J = 10,
  N = 99,
  rho_null = 0,
  burnin = 500,
  rho_type = "pearson",
  del = 1e-10,
  trunc_lev = TRUE,
  wDecay = FALSE,
  Bartlett = FALSE,
  Amat = NULL,
  errorType = "Gaussian",
  logNu = FALSE,
  sigvMethod = "factored",
  winsorize_eps = 0
)
}
\arguments{
\item{y}{Numeric vector. Observed returns.}

\item{p}{Integer. Order of the volatility process. Default 1.}

\item{J}{Integer. Winsorizing parameter. Default 10.}

\item{N}{Integer. Number of Monte Carlo replications. Default 99.}

\item{rho_null}{Numeric. Value of \eqn{\rho} under the null. Default 0.}

\item{burnin}{Integer. Burn-in for simulation. Default 500.}

\item{rho_type}{Character. Correlation type. Default \code{"pearson"}.}

\item{del}{Numeric. Small constant for log transformation. Default \code{1e-10}.}

\item{trunc_lev}{Logical. Truncate leverage correlation estimate to
\code{[-0.999, 0.999]}. Default \code{TRUE}.}

\item{wDecay}{Logical. Use decaying weights. Default \code{FALSE}.}

\item{Bartlett}{Logical. If \code{TRUE}, use Bartlett kernel HAC weighting
matrix. If \code{FALSE}, use identity matrix. Default \code{FALSE}.}

\item{Amat}{Weighting matrix specification. \code{NULL} (default) for identity
weighting, \code{"Weighted"} for data-driven HAC, or a numeric matrix of
dimension \code{(p+3)x(p+3)} (Gaussian) or \code{(p+4)x(p+4)} (heavy-tail).
Takes precedence over \code{Bartlett}.}

\item{errorType}{Character. Error distribution: \code{"Gaussian"} (default),
\code{"Student-t"}, or \code{"GED"}.}

\item{logNu}{Logical. Use log-space for nu estimation (Student-t only).
Default \code{FALSE}.}

\item{sigvMethod}{Method for sigma_v estimation: \code{"factored"} (default),
\code{"direct"}, or \code{"hybrid"}.}

\item{winsorize_eps}{Number of extreme autocovariance lags to winsorize
(0 = none). Default 0.}
}
\value{
An object of class \code{"svp_test"}, a list containing:
\describe{
  \item{s0}{Test statistic from observed data.}
  \item{sN}{Simulated null distribution (vector of length N).}
  \item{pval}{Monte Carlo p-value.}
  \item{test_type}{Character string identifying the test.}
  \item{null_param}{Name of the parameter tested.}
  \item{null_value}{Value under the null hypothesis.}
  \item{call}{The matched call.}
}
}
\description{
Performs a Local Monte Carlo (LMC) test of the null hypothesis
\eqn{H_0: \rho = \rho_0} (typically \eqn{\rho_0 = 0}, i.e., no leverage)
using a GMM likelihood-ratio type statistic.
}
\examples{
\donttest{
y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2, leverage = TRUE, rho = -0.3)$y
test <- lmc_lev(y, p = 1, J = 10, N = 99)
print(test)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svp_information_criteria.R
\name{svp_IC}
\alias{svp_IC}
\title{Information criteria for SV(p) AR-order selection}
\usage{
svp_IC(
  fit,
  criteria = c("BIC_Kalman", "AIC_Kalman", "BIC_HR", "AIC_HR"),
  filter_method = c("mixture", "corrected", "particle"),
  proxy = c("bayes_optimal", "u"),
  K = 7L,
  M = 1000L,
  seed = 42L,
  del = 1e-10
)
}
\arguments{
\item{fit}{Output of \code{\link{svp}}. Must carry the original \code{y}
series (which \code{svp()} stores by default), \code{errorType}, and
\code{leverage} fields.}

\item{criteria}{Character vector. Subset of
\code{c("BIC_Kalman", "AIC_Kalman", "AICc_Kalman", "BIC_Whittle",
"BIC_HR", "AIC_HR", "BIC_YW", "AIC_YW")}. Default returns the four
recommended criteria: \code{c("BIC_Kalman", "AIC_Kalman", "BIC_HR",
"AIC_HR")}. See the description for the rationale for each
opt-in criterion.}

\item{filter_method}{Character. Filter method passed to
\code{\link{filter_svp}} for \code{*_Kalman} criteria. One of
\code{"mixture"} (default, recommended), \code{"corrected"}, or
\code{"particle"}.}

\item{proxy}{Character. Leverage proxy passed to \code{\link{filter_svp}}.
\code{"bayes_optimal"} (default here, unlike \code{filter_svp})
removes the \eqn{O(T)} log-likelihood bias of the û-proxy under
Student-t leverage and restores Schwarz consistency of \code{BIC_Kalman}.
\code{"u"} reproduces the paper-faithful (Remark 3.5) Kalman likelihood;
set this if you need IC values that match the filter's default behavior.}

\item{K}{Integer. Number of mixture components for
\code{filter_method = "mixture"}. Default 7 (KSC).}

\item{M}{Integer. Number of particles for
\code{filter_method = "particle"}. Default 1000. Ignored for other
filter methods.}

\item{seed}{Integer. Random seed for the bootstrap particle filter.
Default 42. Ignored for non-particle filters.}

\item{del}{Numeric. Small constant added inside \eqn{\log} to avoid
\eqn{\log 0}. Default \code{1e-10}.}
}
\value{
Named numeric vector of the requested criteria. Lower is better.
}
\description{
Computes information criteria for an \code{\link{svp}} fit to support
AR-order selection. Eight criteria are computable; \strong{four are
returned by default} — \code{BIC_Kalman}, \code{AIC_Kalman},
\code{BIC_HR}, \code{AIC_HR}. These span two families (state-space QML
and Hannan--Rissanen two-stage ARMA) and two penalty philosophies
(Schwarz-consistent BIC / Shibata-efficient AIC), and were selected as
the most informative criteria across the simulation grid of the
SVHT methodology paper (Ahsan, Dufour and Rodriguez-Rondon 2026).
The remaining four are available on request via the \code{criteria}
argument.
}
\details{
\strong{Default criteria (returned by \code{svp_IC(fit)}):}
\itemize{
  \item \code{BIC_Kalman}, \code{AIC_Kalman}: \eqn{-2\,\hat\ell_K + k\log T}
        and \eqn{-2\,\hat\ell_K + 2k} where \eqn{\hat\ell_K} is the
        (quasi-)log-likelihood from \code{\link{filter_svp}}; default
        \code{filter_method = "mixture"} uses the Gaussian mixture
        Kalman filter (Kim, Shephard and Chib 1998). \code{BIC_Kalman}
        is the primary recommended criterion: Schwarz-consistent under
        the Bayes-optimal leverage proxy (see \code{proxy} argument)
        and strong finite-sample performance across the simulation
        grid (Ahsan, Dufour and Rodriguez-Rondon 2026).
        \code{AIC_Kalman} is Shibata-efficient
        and often selects larger \eqn{p} sooner at \eqn{p_{\mathrm{true}}
        \ge 2}.
  \item \code{BIC_HR}, \code{AIC_HR}: Hannan--Rissanen (1982) two-stage
        ARMA(\eqn{p,p}) criteria. Stage 1: long-AR pre-whitening at
        order \eqn{L = \lfloor 1.5\, T^{1/3}\rfloor} produces residuals
        \eqn{\hat\varepsilon_t}. Stage 2: OLS regression of \eqn{y_t^*}
        on AR lags \eqn{1{:}p} of \eqn{y_t^*} and MA lags \eqn{1{:}p}
        of \eqn{\hat\varepsilon_t} gives \eqn{\hat\sigma_u^2}. Then
        \eqn{T_{\mathrm{eff}} \log \hat\sigma_u^2 + \{2(2p{+}1),
        (2p{+}1)\log T_{\mathrm{eff}}\}}. Filter-free anchor, robust to
        mis-specification of the GMKF mixture. \code{BIC_HR} is
        Schwarz-consistent for ARMA(\eqn{p,p}) (Hannan & Rissanen
        1982; Pötscher 1989).
}

\strong{Opt-in criteria (request via \code{criteria = ...}):}
\itemize{
  \item \code{AICc_Kalman}: \code{AIC_Kalman} with the Hurvich--Tsai
        (1989) small-sample correction \eqn{2k(k+1)/(T-k-1)}.
        Numerically equivalent to \code{AIC_Kalman} at \eqn{T \ge 500};
        use when \eqn{T < 500}.
  \item \code{BIC_Whittle}: \eqn{-2\,\hat\ell_W + k\log T} where
        \eqn{\hat\ell_W} is the Whittle log-likelihood evaluated at the
        SV(\eqn{p}) signal-plus-noise spectral density
        \eqn{f(\omega) = \sigma_v^2 / |1 - \sum_j \phi_j e^{-ij\omega}|^2
                       + \sigma_\varepsilon^2(\nu)}.
        Schwarz-consistent but collapses to \eqn{\hat p = 1} in 98--100\%
        of cells at \eqn{p_{\mathrm{true}} \ge 2} under near-unit-root
        persistence (Ahsan, Dufour and Rodriguez-Rondon 2026). Useful
        as a conservative
        diagnostic: a Whittle selection of \eqn{p > 1} is strong
        evidence against \eqn{p = 1}.
  \item \code{AIC_YW}, \code{BIC_YW}: \emph{Legacy / not recommended.}
        Yule--Walker projection-error criteria on
        \eqn{y_t^* = \log(y_t^2 + \delta) - \mu}, computed as
        \eqn{T \log \hat\sigma_{\mathrm{pred}}^2 + \{2k, k\log T\}}
        with the AR(\eqn{p}) projection-error variance. Under
        SV(\eqn{p}), \eqn{y_t^*} is ARMA(\eqn{p,p}) (not AR(\eqn{p})),
        so the AR projection error does not saturate at
        \eqn{p_{\mathrm{true}}} and the criteria are
        \emph{inconsistent}: the AR(\eqn{p}) projection-error variance
        keeps decreasing past \eqn{p_{\mathrm{true}}}, producing
        non-monotone (sometimes anti-Schwarz) behaviour in \eqn{T}.
        Simulation evidence: 0--29\% correct selection at
        \eqn{p_{\mathrm{true}} = 2} across all DGP cells and
        \eqn{T \le 10{,}000} (Ahsan, Dufour and Rodriguez-Rondon
        2026). Retained for paper-reproducibility of the
        documented failure-case results; \strong{use \code{BIC_HR} /
        \code{AIC_HR} for theoretically consistent AR-order selection.}
}

Lower is better; \code{argmin} over a grid of candidate \code{p} (see
\code{\link{svp_AR_order}}) selects the AR order.
}
\section{Leverage invariance of non-Kalman criteria}{

Leverage does not affect \code{AIC_YW}, \code{BIC_YW}, or
\code{BIC_Whittle}: under the W-ARMA-SV parameterization
\eqn{\mathrm{Cov}(v_t, \varepsilon_{t-1}) = 0} for all three error
distributions (odd-times-even moment symmetry), so the autocovariance
structure of \eqn{y_t^*} is invariant to the leverage parameter. The
\code{*_HR} and \code{*_Kalman} criteria do incorporate leverage
through the estimated \eqn{\delta_p} and the conditional state
innovation variance.
}

\examples{
\donttest{
set.seed(1)
y <- sim_svp(2000, phi = 0.95, sigy = 1, sigv = 0.5)$y
fit1 <- svp(y, p = 1)
fit2 <- svp(y, p = 2)
svp_IC(fit1)
svp_IC(fit2)
}

}
\references{
Akaike, H. (1974). A new look at the statistical model identification.
\emph{IEEE Transactions on Automatic Control} 19(6), 716--723.
\doi{10.1109/TAC.1974.1100705}

Schwarz, G. (1978). Estimating the dimension of a model.
\emph{Annals of Statistics} 6(2), 461--464.
\doi{10.1214/aos/1176344136}

Shibata, R. (1976). Selection of the order of an autoregressive model
by Akaike's information criterion. \emph{Biometrika} 63(1), 117--126.
\doi{10.1093/biomet/63.1.117}

Hannan, E. J. (1980). The estimation of the order of an ARMA process.
\emph{Annals of Statistics} 8(5), 1071--1081.
\doi{10.1214/aos/1176345144}

Hannan, E. J., and Rissanen, J. (1982). Recursive estimation of mixed
autoregressive-moving average order. \emph{Biometrika} 69(1), 81--94.
\doi{10.1093/biomet/69.1.81}

Pötscher, B. M. (1989). Model selection under nonstationarity:
Autoregressive models and stochastic linear regression models.
\emph{Annals of Statistics} 17(3), 1257--1274.
\doi{10.1214/aos/1176347267}

Whittle, P. (1953). Estimation and information in stationary time series.
\emph{Arkiv f\"or Matematik} 2, 423--434. \doi{10.1007/BF02590998}

Dunsmuir, W. (1979). A central limit theorem for parameter estimation in
stationary vector time series and its application to models for a signal
observed with noise. \emph{Annals of Statistics} 7(3), 490--506.
\doi{10.1214/aos/1176344671}

Hurvich, C. M., and Tsai, C.-L. (1989). Regression and time series model
selection in small samples. \emph{Biometrika} 76(2), 297--307.
\doi{10.1093/biomet/76.2.297}

Kim, S., Shephard, N., and Chib, S. (1998). Stochastic volatility:
likelihood inference and comparison with ARCH models.
\emph{Review of Economic Studies} 65(3), 361--393.
\doi{10.1111/1467-937X.00050}

White, H. (1982). Maximum likelihood estimation of misspecified models.
\emph{Econometrica} 50(1), 1--25. \doi{10.2307/1912526}

Ahsan, M. N., Dufour, J.-M., and Rodriguez-Rondon, G. (2026). Estimation and
inference for stochastic volatility models with heavy-tailed distributions.
Bank of Canada Staff Working Paper 2026-8. \doi{10.34989/swp-2026-8}
}
\seealso{
\code{\link{svp_AR_order}}, \code{\link{svp}}, \code{\link{filter_svp}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svp_internal.R
\name{.gauss_hermite_normal}
\alias{.gauss_hermite_normal}
\title{Gauss-Hermite quadrature nodes and weights for N(0,1) integration
Computes nodes z_i and weights w_i such that sum(w_i * f(z_i)) approximates E[f(Z)]
where Z ~ N(0,1). Uses the Golub-Welsch algorithm.}
\usage{
.gauss_hermite_normal(n = 200L)
}
\arguments{
\item{n}{Number of quadrature points (default 200)}
}
\value{
List with components nodes and weights
}
\description{
Gauss-Hermite quadrature nodes and weights for N(0,1) integration
Computes nodes z_i and weights w_i such that sum(w_i * f(z_i)) approximates E[f(Z)]
where Z ~ N(0,1). Uses the Golub-Welsch algorithm.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kalman.R
\name{solve_lyapunov_discrete}
\alias{solve_lyapunov_discrete}
\title{Solve Discrete Lyapunov Equation}
\usage{
solve_lyapunov_discrete(F_mat, Q)
}
\arguments{
\item{F_mat}{Square matrix.}

\item{Q}{Square matrix (same dimensions as \code{F_mat}).}
}
\value{
Solution matrix \code{X}.
}
\description{
Solves X = F X t(F) + Q for X using the vectorization approach.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svp_internal.R
\name{.compute_EH}
\alias{.compute_EH}
\title{Compute EH cross-moment for leverage estimation}
\usage{
.compute_EH(y, rho_type = "pearson")
}
\arguments{
\item{y}{Numeric vector of observations}

\item{rho_type}{"pearson" or "kendall"}
}
\value{
EH value
}
\description{
EH = demeaned sample cross-moment \eqn{(1/(T-2)) \sum(|y_t| - \bar{|y|})(y_{t-1} - \bar{y})}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wARMASVp-package.R
\docType{package}
\name{wARMASVp-package}
\alias{wARMASVp}
\alias{wARMASVp-package}
\title{wARMASVp: Winsorized ARMA Estimation for Higher-Order Stochastic
  Volatility Models}
\description{
Estimation, simulation, hypothesis testing, and forecasting for
  univariate higher-order stochastic volatility SV(p) models. Supports
  Gaussian, Student-t, and GED innovations with optional leverage effects.
}
\details{
The main user-facing functions are:
\itemize{
  \item \code{\link{svp}} -- Estimate SV(p) model with optional leverage
  \item \code{\link{svpSE}} -- Simulation-based standard errors
  \item \code{\link{sim_svp}} -- Simulate SV(p) processes
  \item \code{\link{filter_svp}} -- Kalman/mixture/particle filtering
  \item \code{\link{forecast_svp}} -- Multi-step volatility forecasts
  \item \code{\link{lmc_ar}}, \code{\link{mmc_ar}} -- AR order tests
  \item \code{\link{lmc_lev}}, \code{\link{mmc_lev}} -- Leverage tests
  \item \code{\link{lmc_t}}, \code{\link{mmc_t}} -- Student-t tail tests
  \item \code{\link{lmc_ged}}, \code{\link{mmc_ged}} -- GED tail tests
}
}
\references{
Ahsan, M. N. and Dufour, J.-M. (2021). Simple estimators and inference for
higher-order stochastic volatility models. \emph{Journal of Econometrics},
224(1), 181-197. \doi{10.1016/j.jeconom.2021.03.008}

Ahsan, M. N., Dufour, J.-M., and Rodriguez-Rondon, G. (2025). Estimation and
inference for higher-order stochastic volatility models with leverage.
\emph{Journal of Time Series Analysis}, 46(6), 1064-1084.
\doi{10.1111/jtsa.12851}

Ahsan, M. N., Dufour, J.-M., and Rodriguez-Rondon, G. (2026). Estimation and
inference for stochastic volatility models with heavy-tailed distributions.
Bank of Canada Staff Working Paper 2026-8. \doi{10.34989/swp-2026-8}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/roga11/wARMASVp}
  \item Report bugs at \url{https://github.com/roga11/wARMASVp/issues}
}

}
\author{
\strong{Maintainer}: Gabriel Rodriguez-Rondon \email{gabriel.rodriguezrondon@mail.mcgill.ca} (\href{https://orcid.org/0009-0005-3769-9921}{ORCID})

Authors:
\itemize{
  \item Md. Nazmul Ahsan
  \item Jean-Marie Dufour
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svp_internal.R
\name{.add_leverage}
\alias{.add_leverage}
\title{Add leverage estimation to a fitted SV(p) model (post-processing)
Works for all error distributions: Gaussian, Student-t, GED.
- Gaussian: closed form (C_F = 1)
- Student-t: closed form with C_t(nu) correction
- GED: exact 1D root-finding via uniroot + Gauss-Hermite quadrature}
\usage{
.add_leverage(out, y, p, rho_type, del, trunc_lev, wDecay, errorType)
}
\arguments{
\item{out}{Model object from .svp_gaussian/.svp_t/.svp_ged (without leverage)}

\item{y}{Numeric vector of observations}

\item{p}{AR order}

\item{rho_type}{"pearson", "kendall", or "both"}

\item{del}{Small constant for log transform}

\item{trunc_lev}{Logical: truncate rho to [-0.999, 0.999]}

\item{wDecay}{Logical: decaying weights (passed from original estimation)}

\item{errorType}{"Gaussian", "Student-t", or "GED"}
}
\value{
Updated model object with leverage fields added
}
\description{
Add leverage estimation to a fitted SV(p) model (post-processing)
Works for all error distributions: Gaussian, Student-t, GED.
- Gaussian: closed form (C_F = 1)
- Student-t: closed form with C_t(nu) correction
- GED: exact 1D root-finding via uniroot + Gauss-Hermite quadrature
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forecast.R
\name{forecast_svp}
\alias{forecast_svp}
\title{Multi-Step Ahead Volatility Forecast}
\usage{
forecast_svp(
  object,
  H = 1,
  output = c("log-variance", "variance", "volatility"),
  filter_method = "corrected",
  K = 7,
  M = 1000,
  seed = 42,
  del = 1e-10
)
}
\arguments{
\item{object}{An \code{"svp"}, \code{"svp_t"}, or \code{"svp_ged"} object
from \code{\link{svp}}.}

\item{H}{Integer. Maximum forecast horizon. Default 1.}

\item{output}{Character. Primary output scale: \code{"log-variance"} (default,
native log-volatility w_h), \code{"variance"} (conditional variance
\eqn{\sigma^2_{T+h|T}}), or \code{"volatility"} (conditional std dev
\eqn{\sigma_{T+h|T}}). All three are always computed and stored; this
controls which is used by \code{print} and \code{plot} methods.}

\item{filter_method}{Character. Filter method: \code{"corrected"} (default),
\code{"mixture"} (GMKF), or \code{"particle"} (BPF).}

\item{K}{Integer. Number of mixture components for GMKF. Default 7.}

\item{M}{Integer. Number of particles for BPF. Default 1000.}

\item{seed}{Integer. Random seed for BPF. Default 42.}

\item{del}{Numeric. Small constant for log transformation. Default \code{1e-10}.}
}
\value{
An object of class \code{"svp_forecast"}, a list containing:
\describe{
  \item{w_forecasted}{Primary forecast (scale determined by \code{output}).}
  \item{log_var_forecast}{Log-volatility forecasts \eqn{w_{T+h|T}}.}
  \item{var_forecast}{Conditional variance forecasts \eqn{\sigma^2_{T+h|T}}.}
  \item{vol_forecast}{Conditional volatility forecasts \eqn{\sigma_{T+h|T}}.}
  \item{P_forecast}{Forecast MSE \eqn{P_{T+h|T}} for each horizon.}
  \item{w_estimated}{Filtered log-volatility.}
  \item{w_smoothed}{Smoothed log-volatility.}
  \item{zt}{Filtered standardized residuals.}
  \item{zt_smoothed}{Smoothed standardized residuals.}
  \item{ys}{Demeaned log-squared returns.}
  \item{mdl}{The estimated model object.}
  \item{H}{The forecast horizon.}
  \item{output}{The chosen output scale.}
  \item{filter_output}{The \code{"svp_filter"} object from filtering.}
}
}
\description{
Applies Kalman filtering/smoothing to an estimated SV(p) model and produces
multi-step ahead volatility forecasts with uncertainty quantification.
}
\examples{
\donttest{
sim <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2,
               leverage = TRUE, rho = -0.3)
fit <- svp(sim$y, p = 1, leverage = TRUE)
fc <- forecast_svp(fit, H = 10)
plot(fc)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svp_internal.R
\name{ged_E_abs_u}
\alias{ged_E_abs_u}
\title{E[|u|] for standardized GED(nu) with Var = 1
Closed form: sqrt(Gamma(1/nu)/Gamma(3/nu)) * Gamma(2/nu) / Gamma(1/nu)}
\usage{
ged_E_abs_u(nu)
}
\arguments{
\item{nu}{GED shape parameter (nu > 0)}
}
\value{
E[|u|]
}
\description{
E[|u|] for standardized GED(nu) with Var = 1
Closed form: sqrt(Gamma(1/nu)/Gamma(3/nu)) * Gamma(2/nu) / Gamma(1/nu)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estim.R
\name{svpSE}
\alias{svpSE}
\title{Simulation-Based Standard Errors for SV(p) Models}
\usage{
svpSE(object, n_sim = 199, alpha = 0.05, burnin = 500, logNu = FALSE)
}
\arguments{
\item{object}{A fitted model object from \code{\link{svp}}. Can be of class
\code{"svp"}, \code{"svp_t"}, or \code{"svp_ged"}.}

\item{n_sim}{Integer. Number of Monte Carlo replications. Default 199.}

\item{alpha}{Numeric. Significance level for confidence intervals. Default 0.05.}

\item{burnin}{Integer. Burn-in period for simulation. Default 500.}

\item{logNu}{Logical. Solve for \eqn{\nu} in log-space for numerical
stability (Student-t only). Default is \code{FALSE}.}
}
\value{
A list with:
\describe{
  \item{CI}{2 x k matrix of confidence intervals (lower, upper).}
  \item{SEsim0}{Standard errors relative to true parameter values.}
  \item{SEsim}{Standard errors relative to sample mean.}
  \item{ISEconservative}{Conservative interval-based standard errors.}
  \item{ISEliberal}{Liberal interval-based standard errors.}
  \item{thetamat}{Matrix of parameter estimates from simulations.}
}
}
\description{
Computes standard errors and confidence intervals for estimated parameters
by simulating from the fitted model and re-estimating. Supports all model
types returned by \code{\link{svp}}: Gaussian (with or without leverage),
Student-t, and GED.
}
\examples{
\donttest{
# Gaussian SV(1)
y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2)$y
fit <- svp(y)
se <- svpSE(fit, n_sim = 49)
se$CI
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kalman.R
\name{pged_std}
\alias{pged_std}
\title{CDF of Standardized GED}
\usage{
pged_std(u, nu)
}
\arguments{
\item{u}{Numeric vector. Evaluation points.}

\item{nu}{Numeric. GED shape parameter (\eqn{\nu > 0}).}
}
\value{
Numeric vector of probabilities.
}
\description{
Computes the CDF of the standardized GED(\eqn{\nu}) distribution with
unit variance.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svp_test.R
\name{lmc_ar}
\alias{lmc_ar}
\title{LMC Test for AR Order in SV(p) Models}
\usage{
lmc_ar(
  y,
  p_null,
  p_alt,
  J = 10,
  N = 99,
  burnin = 500,
  del = 1e-10,
  wDecay = FALSE,
  Bartlett = FALSE,
  Amat = NULL,
  errorType = "Gaussian",
  sigvMethod = "factored",
  logNu = TRUE,
  winsorize_eps = 0
)
}
\arguments{
\item{y}{Numeric vector. Observed returns.}

\item{p_null}{Integer. Order under the null hypothesis.}

\item{p_alt}{Integer. Order under the alternative (\code{p_alt > p_null}).}

\item{J}{Integer. Winsorizing parameter. Default 10.}

\item{N}{Integer. Number of Monte Carlo replications. Default 99.}

\item{burnin}{Integer. Burn-in for simulation. Default 500.}

\item{del}{Numeric. Small constant for log transformation. Default \code{1e-10}.}

\item{wDecay}{Logical. Use decaying weights. Default \code{FALSE}.}

\item{Bartlett}{Logical. If \code{TRUE}, use Bartlett kernel HAC weighting
matrix for a GMM-LRT-type test statistic. If \code{FALSE} (default), use
the sum of squared extra AR coefficients.}

\item{Amat}{Weighting matrix specification. \code{NULL} (default) for identity
weighting, or \code{"Weighted"} for data-driven HAC. Takes precedence over
\code{Bartlett}. User-supplied matrices are not supported for AR order tests.}

\item{errorType}{Character. Error distribution of the return innovations:
\code{"Gaussian"} (default), \code{"Student-t"}, or \code{"GED"}. Heavy-tail
options reuse the same moment-based GMM-LRT machinery as \code{lmc_t}/
\code{lmc_ged}; \eqn{\nu} is held at the null MLE during the simulation
(it is not a varied nuisance for the AR-order test).}

\item{sigvMethod}{Character. Method for \eqn{\sigma_v} estimation:
\code{"factored"} (default), \code{"hybrid"}, or \code{"direct"}.}

\item{logNu}{Logical. Use log-space for \eqn{\nu} estimation (Student-t/GED
only). Default \code{TRUE}.}

\item{winsorize_eps}{Number of extreme autocovariance lags to winsorize
(heavy-tail only). Default 0.}
}
\value{
An object of class \code{"svp_test"}, a list containing:
\describe{
  \item{s0}{Test statistic from observed data (capped at 1e-10 if negative).}
  \item{sN}{Simulated null distribution (vector of length N).}
  \item{pval}{Monte Carlo p-value.}
  \item{test_type}{Character string identifying the test.}
  \item{null_param}{Name of the parameter(s) tested.}
  \item{null_value}{Value(s) under the null hypothesis.}
  \item{errorType}{Error distribution used.}
  \item{call}{The matched call.}
}
}
\description{
Performs a Local Monte Carlo (LMC) test of the null hypothesis
\eqn{H_0: \phi_{p_0+1} = \cdots = \phi_p = 0} (i.e., that an SV(\eqn{p_0})
model is sufficient against an SV(\eqn{p}) alternative).
}
\details{
When \code{Bartlett = FALSE} (default), the test statistic is
\eqn{T \sum_{j=p_0+1}^{p} \hat\phi_j^2}, i.e., the sum of squared extra
AR coefficients scaled by sample size.

When \code{Bartlett = TRUE}, the test statistic is based on the GMM-LRT
approach with a Bartlett kernel HAC weighting matrix:
\eqn{S = T \times (M_{H_0} - M_{H_1})}, where \eqn{M} denotes the
GMM criterion evaluated at the null and alternative estimates. Both the
observed and simulated test statistics are capped at \code{1e-10} when
negative; a negative observed statistic raises a warning (it indicates strong
evidence in favour of the null, since the alternative does not improve the
GMM criterion).
}
\examples{
\donttest{
y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2)$y
test <- lmc_ar(y, p_null = 1, p_alt = 2, J = 10, N = 49)
print(test)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svp_test.R
\name{lmc_ged}
\alias{lmc_ged}
\title{LMC Test for GED Shape Parameter}
\usage{
lmc_ged(
  y,
  p = 1,
  J = 10,
  N = 99,
  nu_null,
  burnin = 500,
  del = 1e-10,
  wDecay = FALSE,
  Bartlett = FALSE,
  Amat = NULL,
  direction = c("two-sided", "less", "greater"),
  sigvMethod = "factored",
  winsorize_eps = 0
)
}
\arguments{
\item{y}{Numeric vector. Observed returns.}

\item{p}{Integer. AR order of the volatility process. Default 1.}

\item{J}{Integer. Winsorizing parameter. Default 10.}

\item{N}{Integer. Number of Monte Carlo replications. Default 99.}

\item{nu_null}{Numeric. Value of \eqn{\nu} under the null hypothesis.}

\item{burnin}{Integer. Burn-in for simulation. Default 500.}

\item{del}{Numeric. Small constant for log transformation. Default \code{1e-10}.}

\item{wDecay}{Logical. Use decaying weights. Default \code{FALSE}.}

\item{Bartlett}{Logical. Use Bartlett kernel HAC for weighting matrix.
Default \code{FALSE}.}

\item{Amat}{Weighting matrix specification. \code{NULL} (default) for identity
weighting, \code{"Weighted"} for data-driven HAC, or a \code{(p+3)x(p+3)}
matrix. Takes precedence over \code{Bartlett}.}

\item{direction}{Character. Test direction: \code{"two-sided"} (default),
\code{"less"} (H1: nu < nu_null), or \code{"greater"} (H1: nu > nu_null).
Uses signed root of the LR statistic for one-sided tests.}

\item{sigvMethod}{Character. Method for \eqn{\sigma_v} estimation:
\code{"factored"} (default), \code{"hybrid"}, or \code{"direct"}.}

\item{winsorize_eps}{Numeric. Winsorization threshold for moment conditions.
Default 0 (no winsorization).}
}
\value{
An object of class \code{"svp_test"}.
}
\description{
Performs a Local Monte Carlo (LMC) test of the null hypothesis
\eqn{H_0: \nu = \nu_0} for the shape parameter in an SV(p) model
with GED errors. Testing \eqn{\nu_0 = 2} corresponds to testing normality.
}
\examples{
\donttest{
y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2, errorType = "GED", nu = 1.5)$y
test <- lmc_ged(y, p = 1, J = 10, N = 49, nu_null = 2)
print(test)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svp_information_criteria.R
\name{svp_AR_order}
\alias{svp_AR_order}
\title{AR-order selection sweep for SV(p)}
\usage{
svp_AR_order(
  y,
  pmax = 6L,
  J = 10L,
  leverage = FALSE,
  errorType = "Gaussian",
  rho_type = "pearson",
  del = 1e-10,
  trunc_lev = TRUE,
  wDecay = FALSE,
  logNu = FALSE,
  sigvMethod = "factored",
  winsorize_eps = 0L,
  filter_method = "mixture",
  proxy = c("bayes_optimal", "u"),
  K = 7L,
  M = 1000L,
  seed = 42L,
  criteria = c("BIC_Kalman", "AIC_Kalman", "BIC_HR", "AIC_HR")
)
}
\arguments{
\item{y}{Numeric vector. Observed returns.}

\item{pmax}{Integer. Maximum AR order to consider. Default 6.}

\item{J}{Integer. Winsorizing parameter passed to \code{\link{svp}}.
Default 10.}

\item{leverage}{Logical. Whether to estimate leverage. Default
\code{FALSE}.}

\item{errorType}{Character. \code{"Gaussian"}, \code{"Student-t"}, or
\code{"GED"}. Default \code{"Gaussian"}.}

\item{rho_type, del, trunc_lev, wDecay, logNu, sigvMethod, winsorize_eps}{Other
arguments passed to \code{\link{svp}}.}

\item{filter_method}{Character. Filter method for \code{*_Kalman}
criteria. Default \code{"mixture"}.}

\item{proxy}{Character. Leverage proxy. Default \code{"bayes_optimal"}
for IC consistency under Student-t leverage. See \code{\link{svp_IC}}.}

\item{K, M, seed}{Filter arguments passed to \code{\link{filter_svp}}.}

\item{criteria}{Character vector of criteria to compute. Default
returns the four recommended criteria: \code{c("BIC_Kalman",
"AIC_Kalman", "BIC_HR", "AIC_HR")}. See \code{\link{svp_IC}} for the
full set of eight valid names and the rationale for each opt-in
criterion.}
}
\value{
A list with components:
\describe{
  \item{IC}{Numeric matrix, one row per criterion, one column per
    candidate \code{p} in \code{1:pmax}.}
  \item{argmin}{Named integer vector, one entry per criterion, giving the
    selected \code{p}. \code{NA_integer_} if all entries for that
    criterion are \code{NA}.}
  \item{fits}{List of length \code{pmax} containing the fitted
    \code{svp()} objects (or \code{NULL} if a fit failed).}
}
}
\description{
Convenience wrapper around \code{\link{svp_IC}}: fits \code{\link{svp}}
at each \code{p = 1, ..., pmax} and returns a matrix of information
criteria along with the argmin per criterion.
}
\examples{
\donttest{
set.seed(1)
y <- sim_svp(2000, phi = 0.95, sigy = 1, sigv = 0.5)$y
res <- svp_AR_order(y, pmax = 4)
res$IC
res$argmin
}

}
\seealso{
\code{\link{svp_IC}}, \code{\link{svp}}, \code{\link{filter_svp}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kalman.R
\name{density_eps_ged}
\alias{density_eps_ged}
\title{Density of Centered log-GED^2 Measurement Noise}
\usage{
density_eps_ged(y, nu)
}
\arguments{
\item{y}{Numeric vector. Evaluation points (centered: E[eps] = 0).}

\item{nu}{Numeric. GED shape parameter.}
}
\value{
Density values.
}
\description{
Density of Centered log-GED^2 Measurement Noise
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simu.R
\name{sim_svp}
\alias{sim_svp}
\title{Simulate from a Stochastic Volatility Model}
\usage{
sim_svp(
  n,
  phi,
  sigy,
  sigv,
  errorType = "Gaussian",
  leverage = FALSE,
  rho = 0,
  nu = NULL,
  burnin = 500
)
}
\arguments{
\item{n}{Integer. Length of the simulated series.}

\item{phi}{Numeric vector. AR coefficients for log-volatility (length p).}

\item{sigy}{Numeric. Unconditional standard deviation of returns.}

\item{sigv}{Numeric. Standard deviation of volatility innovations.}

\item{errorType}{Character. Error distribution: \code{"Gaussian"} (default),
\code{"Student-t"}, or \code{"GED"}.}

\item{leverage}{Logical. If \code{TRUE}, simulate with leverage effects
(correlated return and volatility shocks). Default is \code{FALSE}.}

\item{rho}{Numeric. Leverage parameter (correlation between return and
volatility shocks). Must be in \code{[-1, 1]}. Only used when
\code{leverage = TRUE}. Default is 0.}

\item{nu}{Numeric. Shape parameter for heavy-tailed distributions.
Degrees of freedom for Student-t (must be > 2) or GED shape (must be > 0).
Required when \code{errorType} is \code{"Student-t"} or \code{"GED"}.}

\item{burnin}{Integer. Number of initial observations to discard. Default 500.}
}
\value{
A named list of four length-\code{n} numeric vectors:
\describe{
  \item{\code{y}}{Observed returns \eqn{y_t}.}
  \item{\code{h}}{Log-volatility process \eqn{w_t} (equivalently \eqn{h_t}).}
  \item{\code{z}}{Return innovation such that
    \eqn{y_t = \sigma_y \exp(h_t/2)\, z_t}.
    Marginal distribution matches \code{errorType}: N(0,1) for Gaussian,
    t(\eqn{\nu}) for Student-t, unit-variance GED(\eqn{\nu}) for GED.}
  \item{\code{v}}{Volatility innovation such that
    \eqn{h_t - \sum_{j=1}^p \phi_j h_{t-j} = \sigma_v\, v_t}.
    Always N(0,1); under leverage, \eqn{v_t = \rho\, \zeta_{t-1} +
    \sqrt{1-\rho^2}\, \epsilon_t}.}
}
}
\description{
Master simulation function for SV(p) models. Supports Gaussian, Student-t,
and GED error distributions, with optional leverage effects. This mirrors
the interface of \code{\link{svp}} for estimation.
}
\details{
The model is:
\deqn{y_t = \sigma_y \exp(w_t / 2) z_t}
\deqn{w_t = \phi_1 w_{t-1} + \cdots + \phi_p w_{t-p} + \sigma_v v_t}

where \eqn{z_t} follows a distribution specified by \code{errorType}
(Gaussian, Student-t, or GED), and \eqn{v_t} is i.i.d. standard normal.
When \code{leverage = TRUE}, the correlation between \eqn{z_t} and
\eqn{v_{t+1}} is \eqn{\rho}.

For Student-t errors with leverage, the scale-mixture representation
\eqn{z_t = \zeta_t \lambda_t^{-1/2}} is used, where leverage operates through
the Gaussian component \eqn{\zeta_t}. For GED errors with leverage, a Gaussian
copula construction \eqn{z_t = F_{\mathrm{GED}}^{-1}(\Phi(\zeta_t))} is used.
In both cases the returned \code{z} is the \emph{effective} return innovation
(not the latent \eqn{\zeta_t}), with marginal distribution matching the
\code{errorType}.
}
\examples{
\donttest{
# Gaussian SV(1), no leverage
sim <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2)
plot(sim$y, type = "l")

# Gaussian SV(1) with leverage
sim_lev <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2,
                   leverage = TRUE, rho = -0.5)
plot(sim_lev$y, type = "l")

# Student-t SV(1)
sim_t <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2,
                 errorType = "Student-t", nu = 5)
plot(sim_t$y, type = "l")

# GED SV(1)
sim_ged <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2,
                   errorType = "GED", nu = 1.5)
plot(sim_ged$y, type = "l")
}

}
\seealso{
\code{\link{svp}} for estimation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svp_test.R
\name{mmc_ar}
\alias{mmc_ar}
\title{MMC Test for AR Order in SV(p) Models}
\usage{
mmc_ar(
  y,
  p_null,
  p_alt,
  J = 10,
  N = 99,
  burnin = 500,
  eps = NULL,
  threshold = 1,
  method = "pso",
  maxit = NULL,
  del = 1e-10,
  wDecay = FALSE,
  Bartlett = FALSE,
  Amat = NULL,
  errorType = "Gaussian",
  sigvMethod = "factored",
  logNu = TRUE,
  winsorize_eps = 0
)
}
\arguments{
\item{y}{Numeric vector. Observed returns.}

\item{p_null}{Integer. Order under the null hypothesis.}

\item{p_alt}{Integer. Order under the alternative (\code{p_alt > p_null}).}

\item{J}{Integer. Winsorizing parameter. Default 10.}

\item{N}{Integer. Number of Monte Carlo replications. Default 99.}

\item{burnin}{Integer. Burn-in for simulation. Default 500.}

\item{eps}{Numeric vector. Half-width of search region around estimated
nuisance parameters. Default \code{rep(0.3, p_null+2)}.}

\item{threshold}{Numeric. Target p-value. Default 1.}

\item{method}{Character. Optimization method: \code{"pso"} or \code{"GenSA"}.
Default \code{"pso"}.}

\item{maxit}{Integer. Maximum iterations/evaluations. Default depends on method.}

\item{del}{Numeric. Small constant for log transformation. Default \code{1e-10}.}

\item{wDecay}{Logical. Use decaying weights. Default \code{FALSE}.}

\item{Bartlett}{Logical. If \code{TRUE}, use Bartlett kernel HAC weighting
matrix for a GMM-LRT-type test statistic. If \code{FALSE} (default), use
the sum of squared extra AR coefficients.}

\item{Amat}{Weighting matrix specification. \code{NULL} (default) for identity
weighting, or \code{"Weighted"} for data-driven HAC. Takes precedence over
\code{Bartlett}. User-supplied matrices are not supported for AR order tests.}

\item{errorType}{Character. Error distribution of the return innovations:
\code{"Gaussian"} (default), \code{"Student-t"}, or \code{"GED"}. Heavy-tail
options reuse the same moment-based GMM-LRT machinery as \code{lmc_t}/
\code{lmc_ged}; \eqn{\nu} is held at the null MLE during the simulation
(it is not a varied nuisance for the AR-order test).}

\item{sigvMethod}{Character. Method for \eqn{\sigma_v} estimation:
\code{"factored"} (default), \code{"hybrid"}, or \code{"direct"}.}

\item{logNu}{Logical. Use log-space for \eqn{\nu} estimation (Student-t/GED
only). Default \code{TRUE}.}

\item{winsorize_eps}{Number of extreme autocovariance lags to winsorize
(heavy-tail only). Default 0.}
}
\value{
A list with optimization output including \code{value} (maximized p-value)
  and \code{par} (nuisance parameters at the maximum).
}
\description{
Performs a Maximized Monte Carlo (MMC) test of
\eqn{H_0: \phi_{p_0+1} = \cdots = \phi_p = 0}
by maximizing the MC p-value over nuisance parameters
(\eqn{\phi_1, \ldots, \phi_{p_0}, \sigma_y, \sigma_v}).
}
\examples{
\donttest{
y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2)$y
mmc <- mmc_ar(y, p_null = 1, p_alt = 2, J = 10, N = 19,
              method = "pso", maxit = 10)
mmc$value
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svp_test.R
\name{mmc_lev}
\alias{mmc_lev}
\title{MMC Test for Leverage in SV(p) Models}
\usage{
mmc_lev(
  y,
  p = 1,
  J = 10,
  N = 99,
  rho_null = 0,
  burnin = 500,
  eps = NULL,
  threshold = 1,
  method = "pso",
  maxit = NULL,
  rho_type = "pearson",
  del = 1e-10,
  trunc_lev = TRUE,
  wDecay = FALSE,
  Bartlett = FALSE,
  Amat = NULL,
  errorType = "Gaussian",
  logNu = FALSE,
  sigvMethod = "factored",
  winsorize_eps = 0
)
}
\arguments{
\item{y}{Numeric vector. Observed returns.}

\item{p}{Integer. Order of the volatility process. Default 1.}

\item{J}{Integer. Winsorizing parameter. Default 10.}

\item{N}{Integer. Number of Monte Carlo replications. Default 99.}

\item{rho_null}{Numeric. Value of \eqn{\rho} under the null. Default 0.}

\item{burnin}{Integer. Burn-in for simulation. Default 500.}

\item{eps}{Numeric vector. Half-width of the search region around the
estimated nuisance parameters. For Gaussian: length \code{p+2}
(phi, sigma_y, sigma_v). For Student-t/GED: length \code{p+2}
(phi, sigma_y, sigma_v; nu bounds set proportionally at +/-30%) or
length \code{p+3} (phi, sigma_y, sigma_v, nu). Default \code{NULL}
which uses \code{rep(0.3, p+2)} with proportional nu bounds.}

\item{threshold}{Numeric. Target p-value (optimization stops if reached).
Default 1.}

\item{method}{Character. Optimization method: \code{"pso"} (particle swarm),
\code{"GenSA"} (generalized simulated annealing).
Default \code{"pso"}.}

\item{maxit}{Integer or list. Maximum iterations/evaluations for the
optimizer. Default depends on method.}

\item{rho_type}{Character. Correlation type. Default \code{"pearson"}.}

\item{del}{Numeric. Small constant for log transformation. Default \code{1e-10}.}

\item{trunc_lev}{Logical. Truncate leverage correlation estimate to
\code{[-0.999, 0.999]}. Default \code{TRUE}.}

\item{wDecay}{Logical. Use decaying weights. Default \code{FALSE}.}

\item{Bartlett}{Logical. If \code{TRUE}, use Bartlett kernel HAC weighting
matrix. If \code{FALSE}, use identity matrix. Default \code{FALSE}.}

\item{Amat}{Weighting matrix specification. \code{NULL} (default) for identity
weighting, \code{"Weighted"} for data-driven HAC, or a numeric matrix of
dimension \code{(p+3)x(p+3)} (Gaussian) or \code{(p+4)x(p+4)} (heavy-tail).
Takes precedence over \code{Bartlett}.}

\item{errorType}{Character. Error distribution: \code{"Gaussian"} (default),
\code{"Student-t"}, or \code{"GED"}.}

\item{logNu}{Logical. Use log-space for nu estimation (Student-t only).
Default \code{FALSE}.}

\item{sigvMethod}{Method for sigma_v estimation: \code{"factored"} (default),
\code{"direct"}, or \code{"hybrid"}.}

\item{winsorize_eps}{Number of extreme autocovariance lags to winsorize
(0 = none). Default 0.}
}
\value{
A list with the optimization output including:
\describe{
  \item{value}{Maximized p-value.}
  \item{par}{Nuisance parameter values at the maximum.}
}
Additional fields depend on the optimization method used.
}
\description{
Performs a Maximized Monte Carlo (MMC) test of the null hypothesis
\eqn{H_0: \rho = \rho_0} by maximizing the MC p-value over nuisance
parameters (phi, sigma_y, sigma_v).
}
\examples{
\donttest{
y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2, leverage = TRUE, rho = -0.3)$y
mmc <- mmc_lev(y, p = 1, J = 10, N = 19, method = "pso", maxit = 10)
mmc$value
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svp_test.R
\name{mmc_t}
\alias{mmc_t}
\title{MMC Test for Student-t Tail Parameter}
\usage{
mmc_t(
  y,
  p = 1,
  J = 10,
  N = 99,
  nu_null,
  burnin = 500,
  eps = NULL,
  threshold = 1,
  method = "pso",
  maxit = NULL,
  del = 1e-10,
  wDecay = FALSE,
  Bartlett = FALSE,
  Amat = NULL,
  logNu = TRUE,
  direction = c("two-sided", "less", "greater"),
  sigvMethod = "factored",
  winsorize_eps = 0
)
}
\arguments{
\item{y}{Numeric vector. Observed returns.}

\item{p}{Integer. AR order of the volatility process. Default 1.}

\item{J}{Integer. Winsorizing parameter. Default 10.}

\item{N}{Integer. Number of Monte Carlo replications. Default 99.}

\item{nu_null}{Numeric. Value of \eqn{\nu} under the null hypothesis.}

\item{burnin}{Integer. Burn-in for simulation. Default 500.}

\item{eps}{Numeric vector. Half-width of search region around estimated nuisance
parameters. Must have length \code{p+2} (one entry per nuisance parameter:
\eqn{\phi_1,\ldots,\phi_p, \sigma_y, \sigma_v}). Default \code{rep(0.3, p+2)}.}

\item{threshold}{Numeric. Target p-value. Default 1.}

\item{method}{Character. Optimization method: \code{"pso"} or \code{"GenSA"}.
Default \code{"pso"}.}

\item{maxit}{Integer. Maximum iterations/evaluations. Default depends on method.}

\item{del}{Numeric. Small constant for log transformation. Default \code{1e-10}.}

\item{wDecay}{Logical. Use decaying weights. Default \code{FALSE}.}

\item{Bartlett}{Logical. Use Bartlett kernel HAC for weighting matrix.
Default \code{FALSE}.}

\item{Amat}{Weighting matrix specification. \code{NULL} (default) for identity
weighting, \code{"Weighted"} for data-driven HAC, or a \code{(p+3)x(p+3)}
matrix. Takes precedence over \code{Bartlett}.}

\item{logNu}{Logical. Use log-space for nu estimation. Default \code{TRUE}.}

\item{direction}{Character. Test direction: \code{"two-sided"} (default),
\code{"less"} (H1: nu < nu_null), or \code{"greater"} (H1: nu > nu_null).
Uses signed root of the LR statistic for one-sided tests.}

\item{sigvMethod}{Character. Method for \eqn{\sigma_v} estimation:
\code{"factored"} (default), \code{"hybrid"}, or \code{"direct"}.}

\item{winsorize_eps}{Numeric. Winsorization threshold for moment conditions.
Default 0 (no winsorization).}
}
\value{
A list with optimization output including \code{value} (maximized p-value)
  and \code{par} (nuisance parameters at the maximum).
}
\description{
Performs a Maximized Monte Carlo (MMC) test of \eqn{H_0: \nu = \nu_0}
by maximizing the MC p-value over nuisance parameters (phi, sigma_y, sigma_v).
}
\examples{
\donttest{
y <- sim_svp(1000, phi = 0.95, sigy = 1, sigv = 0.2, errorType = "Student-t", nu = 5)$y
mmc <- mmc_t(y, p = 1, J = 10, N = 19, nu_null = 5, method = "pso", maxit = 10)
mmc$value
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svp_internal.R
\name{LRT_moment_lev_ged_Amat}
\alias{LRT_moment_lev_ged_Amat}
\title{GMM moments for SVL(p)-GED with HAC weighting (p+4 conditions)}
\usage{
LRT_moment_lev_ged_Amat(y, mdl_out, rho_type, del = 1e-10, Bartlett = TRUE)
}
\description{
GMM moments for SVL(p)-GED with HAC weighting (p+4 conditions)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svp_internal.R
\name{correction_factor_ged_approx}
\alias{correction_factor_ged_approx}
\title{Approximate correction factor C_g(nu) for GED leverage (diagnostic use only)}
\usage{
correction_factor_ged_approx(nu, n_nodes = 200L)
}
\arguments{
\item{nu}{GED shape parameter (nu > 0)}

\item{n_nodes}{Number of GH quadrature nodes (default 200)}
}
\value{
Approximate C_g(nu)
}
\description{
\eqn{C_g(\nu) = E[|u|] \cdot \textrm{Cov}(z, F_{GED}^{-1}(\Phi(z))) / \sqrt{2/\pi}}.
This is a first-order approximation; estimation uses the exact implicit equation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kalman.R
\name{fit_ksc_mixture}
\alias{fit_ksc_mixture}
\title{Fit K-Component Gaussian Mixture to Measurement Noise Density}
\usage{
fit_ksc_mixture(
  distribution = c("gaussian", "student_t", "ged"),
  nu = NULL,
  K = 7,
  n_sample = 10000,
  max_iter = 500,
  tol = 1e-08,
  seed = 42
)
}
\arguments{
\item{distribution}{Character: \code{"gaussian"}, \code{"student_t"}, or
\code{"ged"}.}

\item{nu}{Numeric. Shape parameter (ignored for Gaussian).}

\item{K}{Integer. Number of mixture components. Default 7.}

\item{n_sample}{Integer. Sample size for EM fitting. Default 10000.}

\item{max_iter}{Integer. Maximum EM iterations. Default 500.}

\item{tol}{Numeric. Convergence tolerance. Default 1e-8.}

\item{seed}{Integer. Random seed. Default 42.}
}
\value{
List with \code{weights}, \code{means}, \code{vars}, \code{KL_div}.
}
\description{
Uses EM algorithm to approximate the measurement noise density with a
Gaussian mixture. For Gaussian SV, returns the pre-computed KSC (1998)
7-component table.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svp_internal.R
\name{.get_gh}
\alias{.get_gh}
\title{Get cached Gauss-Hermite nodes/weights for N(0,1)}
\usage{
.get_gh(n = 200L)
}
\arguments{
\item{n}{Number of nodes (default 200)}
}
\value{
List with nodes and weights
}
\description{
Get cached Gauss-Hermite nodes/weights for N(0,1)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svp_internal.R
\name{LRT_moment_lev_ged}
\alias{LRT_moment_lev_ged}
\title{GMM moments for SVL(p)-GED with fixed A matrix (p+4 conditions)
Uses exact GED leverage moment.}
\usage{
LRT_moment_lev_ged(y, mdl_out, Amat, rho_type, del = 1e-10)
}
\description{
GMM moments for SVL(p)-GED with fixed A matrix (p+4 conditions)
Uses exact GED leverage moment.
}
\keyword{internal}
